@@ -1,4 +1,4 @@
-# $Id: Utilities.pm,v 1.20.4.3 2006/10/02 23:10:12 sendu Exp $
+# $Id: Utilities.pm,v 1.9 2003/06/04 08:36:36 heikki Exp $
 #
 # BioPerl module for Bio::Align::Utilities
 #
@@ -17,20 +17,9 @@ and manipulating alignment objects
 
 =head1 SYNOPSIS
 
-  use Bio::Align::Utilities qw(:all);
-  # %dnaseqs is a hash of CDS sequences (spliced)
+use Bio::Align::Utilities qw(aa_to_dna_aln);
 
-
-  # Even if the protein alignments are local make sure the start/end
-  # stored in the LocatableSeq objects are to the full length protein.
-  # The CoDing Sequence that is passed in should still be the full 
-  # length CDS as the nt alignment will be generated.
-  #
-  my $dna_aln = &aa_to_dna_aln($aa_aln,\%dnaseqs);
-
-
-  # generate bootstraps
-  my $replicates = &bootstrap_replicates($aln,$count);
+my $dna_aln = aa_to_dna_aln($aaaln,\%dnaseqs);
 
 
 =head1 DESCRIPTION
@@ -53,21 +42,26 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org              - General discussion
+  http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via the
-web:
+of the bugs and their resolution. Bug reports can be submitted via
+email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bioperl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
 Email jason@bioperl.org
 
+=head1 CONTRIBUTORS
+
+Additional contributors names and emails here
+
 =head1 APPENDIX
 
 The rest of the documentation details each of the object methods.
@@ -80,22 +74,18 @@ Internal methods are usually preceded with a _
 
 
 package Bio::Align::Utilities;
-use vars qw(@EXPORT @EXPORT_OK $GAP $CODONGAP %EXPORT_TAGS);
+use vars qw(@ISA @EXPORT @EXPORT_OK);
 use strict;
 use Carp;
 use Bio::Root::Version;
 require Exporter;
 
-use base qw(Exporter);
+@ISA = qw(Exporter);
 
 @EXPORT = qw();
-@EXPORT_OK = qw(aa_to_dna_aln bootstrap_replicates);
-%EXPORT_TAGS = (all =>[@EXPORT, @EXPORT_OK]);
-BEGIN {
-    use constant CODONSIZE => 3;
-    $GAP = '-';
-    $CODONGAP = $GAP x CODONSIZE;
-}
+@EXPORT_OK = qw(aa_to_dna_aln);
+
+use constant CODONSIZE => 3;
 
 =head2 aa_to_dna_aln
 
@@ -129,89 +119,45 @@ sub aa_to_dna_aln {
 	croak('Must provide a valid Bio::Align::AlignI object as the first argument to aa_to_dna_aln, see the documentation for proper usage and the method signature');
     }
     my $alnlen = $aln->length;
+    #print "HSP length is $alnlen\n";
     my $dnaalign = new Bio::SimpleAlign;
-    $aln->map_chars('\.',$GAP);
-
     foreach my $seq ( $aln->each_seq ) {    
-	my $aa_seqstr = $seq->seq();
-	my $id = $seq->display_id;
-	my $dnaseq = $dnaseqs->{$id} || $aln->throw("cannot find ".
-						     $seq->display_id);
-	my $start_offset = ($seq->start - 1) * CODONSIZE;
-
-	$dnaseq = $dnaseq->seq();
-	my $dnalen = $dnaseqs->{$id}->length;
-	my $nt_seqstr;
-	my $j = 0;
-	for( my $i = 0; $i < $alnlen; $i++ ) {
-	    my $char = substr($aa_seqstr,$i + $start_offset,1);	    
-	    if ( $char eq $GAP || $j >= $dnalen )  { 
-		$nt_seqstr .= $CODONGAP;
+	my $newseq;	    
+	my $dnaseq = $dnaseqs->{$seq->display_id} || croak("cannot find ".
+							 $seq->display_id);
+	foreach my $pos ( 1..$alnlen ) {
+	    my $loc = $seq->location_from_column($pos);
+	    my $dna = ''; 
+	    if( !defined $loc || $loc->location_type ne 'EXACT' ) {
+		$dna = '---';
 	    } else {
-		$nt_seqstr .= substr($dnaseq,$j,CODONSIZE);
-		$j += CODONSIZE;
-	    }
-	}
-	$nt_seqstr .= $GAP x (($alnlen * 3) - length($nt_seqstr));
+		# To readjust to codon boundaries
+		# end needs to be +1 so we can just multiply by CODONSIZE 
+		# to get this		    
 
-	my $newdna = new Bio::LocatableSeq(-display_id  => $id,
-					   -alphabet    => 'dna',
-					   -start       => $start_offset+1,
-					   -end         => ($seq->end * 
-							    CODONSIZE),
-					   -strand      => 1,
-					   -seq         => $nt_seqstr);    
+		my ($start,$end) = ((($loc->start - 1)* CODONSIZE) +1,
+				    ($loc->end)* CODONSIZE);
+		
+		if( $start <=0 || $end > $dnaseq->length() ) {
+		    print STDERR "start is ", $loc->start, " end is ", $loc->end, " while dnaseq length is ", $dnaseq->length(), " and start/end projected are $start,$end \n";
+		    warn("codons don't seem to be matching up for $start,$end");
+		    $dna = '---';			    
+		} else {
+		    $dna = $dnaseq->subseq($start,$end);
+		}
+	    }
+	    $newseq .= $dna;
+	}
+	# funky looking math is to readjust to codon boundaries and deal
+	# with fact that sequence start with 1
+	my $newdna = new Bio::LocatableSeq(-display_id  => $seq->id(),
+					   -start => (($seq->start - 1) * 
+						      CODONSIZE) + 1, 
+					   -end   => ($seq->end * CODONSIZE),
+					   -strand => $seq->strand,
+					   -seq   => $newseq);    
 	$dnaalign->add_seq($newdna);
     }
     return $dnaalign;
 }
-
-=head2 bootstrap_replicates
-
- Title   : bootstrap_replicates
- Usage   : my $alns = &bootstrap_replicates($aln,100);
- Function: Generate a pseudo-replicate of the data by randomly
-           sampling, with replacement, the columns from an alignment for
-           the non-parametric bootstrap.
- Returns : Arrayref of L<Bio::SimpleAlign> objects
- Args    : L<Bio::SimpleAlign> object
-           Number of replicates to generate
-
-=cut
-
-sub bootstrap_replicates {
-   my ($aln,$count) = @_;
-   $count ||= 1;
-   my $alen = $aln->length;
-   my (@seqs,@nm);
-   $aln->set_displayname_flat(1);
-   for my $s ( $aln->each_seq ) {
-       push @seqs, $s->seq();
-       push @nm, $s->id;
-   }
-   my (@alns,$i);
-   while( $count-- > 0 ) {
-       my @newseqs;
-       for($i =0; $i < $alen; $i++ ) {
-	   my $index = int(rand($alen));
-	   my $c = 0;
-	   for ( @seqs ) {
-	       $newseqs[$c++] .= substr($_,$index,1);
-	   }
-       }
-       my $newaln = Bio::SimpleAlign->new();
-       my $i = 0;
-       for my $s ( @newseqs ) {
-
-	   $newaln->add_seq( Bio::LocatableSeq->new
-			     (-start         => 1,
-			      -end           => $alen,
-			      -display_id    => $nm[$i++],
-			      -seq           => $s));
-       }
-       push @alns, $newaln;
-   }
-   return \@alns;
-}
-
 1;
