@@ -1,4 +1,4 @@
-# $Id: phylip.pm,v 1.24 2002/12/14 19:32:41 shawnh Exp $
+# $Id: phylip.pm,v 1.26 2003/03/28 22:39:54 jason Exp $
 #
 # BioPerl module for Bio::AlignIO::phylip
 #
@@ -17,12 +17,12 @@ Bio::AlignIO::phylip - PHYLIP format sequence input/output stream
     use Bio::SimpleAlign;
 	#you can set the name length to something other than the default 10
 	#if you use a version of phylip (hacked) that accepts ids > 10
-    my $phylipstream = new Bio::AlignIO(-format => 'phylip',
-					-fh   => \*STDOUT,
+    my $phylipstream = new Bio::AlignIO(-format  => 'phylip',
+					-fh      => \*STDOUT,
 					-idlength=>30);
     # convert data from one format to another
     my $gcgstream     =  new Bio::AlignIO(-format => 'msf',
-					  -file   => 't/data/cysprot1a.msf');    
+					  -file   => 't/data/cysprot1a.msf');
 
     while( my $aln = $gcgstream->next_aln ) {
 	$phylipstream->write_aln($aln);
@@ -152,40 +152,62 @@ sub next_aln {
     # first alignment section
     my $idlen = $self->idlength;
     $count = 0;
+    my $iter = 1;
+    my $non_interleaved = ! $self->interleaved ;
+    
     while( $entry = $self->_readline) {
-    	$entry =~ /^\s$/ and last;
-    	$entry =~ /^(.{$idlen})\s+(.*)\s$/ && do {
+	last if( $entry =~ /^\s?$/ && ! $non_interleaved );
+
+	if( $entry =~ /^\s+(\d+)\s+(\d+)\s*$/) { 
+	    $self->_pushback($entry);
+	    last;
+	}
+	if( $entry =~ /^\s+(.+)$/ ) {
+	    $str = $1;
+	    $non_interleaved = 1;
+	    $str =~ s/\s//g;
+	    unless( ! $non_interleaved ) {
+		$count = scalar @names;
+		$hash{$count} .= $str;
+	    } else { 
+		$hash{$iter++} .= $str;
+		$iter = 1 if $iter > $count;
+	    }
+	} elsif( $entry =~ /^(.{$idlen})\s+(.*)\s$/ ||		 
+		 $entry =~ /^(.{$idlen})(\S{$idlen}\s+.+)\s$/ # Handle weirdnes s when id is too long
+		 ) {
 	    $name = $1;
 	    $str = $2;
 	    $name =~ s/[\s\/]/_/g;
 	    $name =~ s/_+$//; # remove any trailing _'s
 	    push @names, $name;
-	    
 	    $str =~ s/\s//g;
 	    $count = scalar @names;
 	    $hash{$count} = $str;
-	};
+	}
 	$self->throw("Not a valid interleaved PHYLIP file!") if $count > $seqcount; 
     }
     
-    # interleaved sections
-    $count = 0;
-    while( $entry = $self->_readline) {
-      #finish current entry
-      if($entry =~/\s*\d+\s+\d+/){
-          $self->_pushback($entry);
-          last;
-      }
-    	$count = 0, next if $entry =~ /^\s$/;
-    	$entry =~ /\s*(.*)$/ && do {
-	    $str = $1;
-	    $str =~ s/\s//g;
-	    $count++;
-	    $hash{$count} .= $str;
-	};
-	$self->throw("Not a valid interleaved PHYLIP file!") if $count > $seqcount; 
+    unless( $non_interleaved ) {    
+	# interleaved sections
+	$count = 0;
+	while( $entry = $self->_readline) {
+	    
+            # finish current entry
+	    if($entry =~/\s*\d+\s+\d+/){
+		$self->_pushback($entry);
+		last;
+	    }
+	    $count = 0, next if $entry =~ /^\s$/;
+	    $entry =~ /\s*(.*)$/ && do {
+		$str = $1;
+		$str =~ s/\s//g;
+		$count++;
+		$hash{$count} .= $str;
+	    };
+	    $self->throw("Not a valid interleaved PHYLIP file!") if $count > $seqcount; 
+	}
     }
-    
     return 0 if scalar @names < 1;
     
     # sequence creation
@@ -203,17 +225,16 @@ sub next_aln {
 	    $str =~ s/[^A-Za-z]//g;
 	    $end = length($str);
 	}
-	
 	# consistency test
-	$self->throw("Length of sequence [$seqname] is not [$residuecount]! ") 
+	$self->throw("Length of sequence [$seqname] is not [$residuecount] it is ".CORE::length($hash{$count})."! ") 
 	    unless CORE::length($hash{$count}) == $residuecount; 
-
+	
        $seq = new Bio::LocatableSeq('-seq'=>$hash{$count},
 				    '-id'=>$seqname,
 				    '-start'=>$start,
 				    '-end'=>$end,
 				    );
-
+	
        $aln->add_seq($seq);
 
    }
