@@ -1,4 +1,4 @@
-# $Id: Reference.pm,v 1.24.4.3 2006/10/02 23:10:12 sendu Exp $
+# $Id: Reference.pm,v 1.19 2003/12/22 08:50:00 juguang Exp $
 #
 # BioPerl module for Bio::Annotation::Reference
 #
@@ -16,10 +16,10 @@ Bio::Annotation::Reference - Specialised DBLink object for Literature References
 
 =head1 SYNOPSIS
 
-    $reg = Bio::Annotation::Reference->new( -title    => 'title line',
-                                            -location => 'location line',
-                                            -authors  => 'author line',
-                                            -medline  => 998122 );
+    $reg = Bio::Annotation::Reference->new( -title => 'title line',
+					    -location => 'location line',
+					    -authors => 'author line',
+					    -medline => 998122 );
 
 =head1 DESCRIPTION
 
@@ -37,9 +37,9 @@ without losing data, we keep them as strings. Feel free to post the
 list for a better solution, but in general this gets very messy very
 fast...
 
-=head1 AUTHOR - Ewan Birney 
+=head1 CONTACT
 
-Email birney@ebi.ac.uk
+Describe contact details here
 
 =head1 APPENDIX
 
@@ -51,13 +51,14 @@ The rest of the documentation details each of the object methods. Internal metho
 # Let the code begin...
 
 package Bio::Annotation::Reference;
+use vars qw(@ISA);
 use strict;
-use overload '""' => sub { $_[0]->title || ''};
-use overload 'eq' => sub { "$_[0]" eq "$_[1]" };
+# use overload '""' => \&as_text;
 
+use Bio::Annotation::DBLink;
 use Bio::AnnotationI;
 
-use base qw(Bio::Annotation::DBLink);
+@ISA = qw(Bio::Annotation::DBLink);
 
 =head2 new
 
@@ -69,8 +70,8 @@ use base qw(Bio::Annotation::DBLink);
  Function:
  Example :
  Returns : a new Bio::Annotation::Reference object
- Args    : a hash with optional title, authors, location, medline, pubmed,
-           start, end, consortium, rp and rg attributes
+ Args    : a hash with optional title, authors, location, medline, start and end
+           attributes
 
 
 =cut
@@ -80,30 +81,24 @@ sub new{
 
     my $self = $class->SUPER::new(@args);
 
-    my ($start,$end,$authors,$consortium,$location,$title,$medline,
-	$pubmed,$rp,$rg) =
+    my ($start,$end,$authors,$location,$title,$medline,$tag) =
 	$self->_rearrange([qw(START
 			      END
 			      AUTHORS
-				  CONSORTIUM
 			      LOCATION
 			      TITLE
 			      MEDLINE
-				  PUBMED
-				  RP
-				  RG
+			      TAGNAME
 			      )],@args);
 
     defined $start    && $self->start($start);
     defined $end      && $self->end($end);
     defined $authors  && $self->authors($authors);
-	defined $consortium  && $self->consortium($consortium);
     defined $location && $self->location($location);
     defined $title    && $self->title($title);
     defined $medline  && $self->medline($medline);
-    defined $pubmed   && $self->pubmed($pubmed);
-    defined $rp       && $self->rp($rp);
-    defined $rg       && $self->rg($rg);
+    defined $tag      && $self->tagname($tag);
+
     return $self;
 }
 
@@ -118,7 +113,7 @@ sub new{
  Usage   :
  Function:
  Example :
- Returns :
+ Returns : 
  Args    :
 
 
@@ -138,31 +133,28 @@ sub as_text{
  Usage   :
  Function:
  Example :
- Returns :
+ Returns : 
  Args    :
 
 
 =cut
 
 sub hash_tree{
-	my ($self) = @_;
+   my ($self) = @_;
+   
+   my $h = {};
+   $h->{'title'}   = $self->title;
+   $h->{'authors'} = $self->authors;
+   $h->{'location'} = $self->location;
+   if( defined $self->start ) {
+       $h->{'start'}   = $self->start;
+   }
+   if( defined $self->end ) {
+       $h->{'end'} = $self->end;
+   }
+   $h->{'medline'} = $self->medline;
 
-	my $h = {};
-	$h->{'title'}   = $self->title;
-	$h->{'authors'} = $self->authors;
-	$h->{'location'} = $self->location;
-	if (defined $self->start) {
-		$h->{'start'}   = $self->start;
-	}
-	if (defined $self->end) {
-		$h->{'end'} = $self->end;
-	}
-	$h->{'medline'} = $self->medline;
-	if (defined $self->pubmed) {
-		$h->{'pubmed'} = $self->pubmed;
-	}
-
-	return $h;
+   return $h;
 }
 
 =head2 tagname
@@ -176,13 +168,21 @@ sub hash_tree{
            obtaining an AnnotationI object from the collection, the collection
            will set the value to the tag under which it was stored unless the
            object has a tag stored already.
- Example :
+ Example : 
  Returns : value of tagname (a scalar)
  Args    : new value (a scalar, optional)
 
 
 =cut
 
+sub tagname{
+    my ($self,$value) = @_;
+    if( defined $value) {
+	$self->{'tagname'} = $value;
+    }
+    return $self->{'tagname'};
+}
+
 
 =head1 Specific accessors for References
 
@@ -194,7 +194,7 @@ sub hash_tree{
  Title   : start
  Usage   : $self->start($newval)
  Function: Gives the reference start base
- Example :
+ Example : 
  Returns : value of start
  Args    : newvalue (optional)
 
@@ -215,7 +215,7 @@ sub start {
  Title   : end
  Usage   : $self->end($newval)
  Function: Gives the reference end base
- Example :
+ Example : 
  Returns : value of end
  Args    : newvalue (optional)
 
@@ -235,7 +235,7 @@ sub end {
  Title   : rp
  Usage   : $self->rp($newval)
  Function: Gives the RP line. No attempt is made to parse this line.
- Example :
+ Example : 
  Returns : value of rp
  Args    : newvalue (optional)
 
@@ -248,29 +248,7 @@ sub rp{
       $self->{'rp'} = $value;
     }
     return $self->{'rp'};
-}
 
-=head2 rg
-
- Title   : rg
- Usage   : $obj->rg($newval)
- Function: Gives the RG line. This is Swissprot/Uniprot specific, and
-           if set will usually be identical to the authors attribute,
-           but the swissprot manual does allow both RG and RA (author)
-           to be present for the same reference.
-
- Example :
- Returns : value of rg (a scalar)
- Args    : on set, new value (a scalar or undef, optional)
-
-
-=cut
-
-sub rg{
-    my $self = shift;
-
-    return $self->{'rg'} = shift if @_;
-    return $self->{'rg'};
 }
 
 =head2 authors
@@ -278,7 +256,7 @@ sub rg{
  Title   : authors
  Usage   : $self->authors($newval)
  Function: Gives the author line. No attempt is made to parse the author line
- Example :
+ Example : 
  Returns : value of authors
  Args    : newvalue (optional)
 
@@ -299,7 +277,7 @@ sub authors{
  Title   : location
  Usage   : $self->location($newval)
  Function: Gives the location line. No attempt is made to parse the location line
- Example :
+ Example : 
  Returns : value of location
  Args    : newvalue (optional)
 
@@ -320,7 +298,7 @@ sub location{
  Title   : title
  Usage   : $self->title($newval)
  Function: Gives the title line (if exists)
- Example :
+ Example : 
  Returns : value of title
  Args    : newvalue (optional)
 
@@ -341,7 +319,7 @@ sub title{
  Title   : medline
  Usage   : $self->medline($newval)
  Function: Gives the medline number
- Example :
+ Example : 
  Returns : value of medline
  Args    : newvalue (optional)
 
@@ -362,7 +340,7 @@ sub medline{
  Usage   : $refobj->pubmed($newval)
  Function: Get/Set the PubMed number, if it is different from the MedLine
            number.
- Example :
+ Example : 
  Returns : value of medline
  Args    : newvalue (optional)
 
@@ -381,47 +359,37 @@ sub pubmed {
 
  Title   : database
  Usage   :
- Function: Overrides DBLink database to be hard coded to 'MEDLINE' (or 'PUBMED'
-		   if only pubmed id has been supplied), unless the database has been
-		   set explicitely before.
+ Function: Overrides DBLink database to be hard coded to 'MEDLINE', unless
+           the database has been set explicitely before.
  Example :
- Returns :
+ Returns : 
  Args    :
 
 
 =cut
 
 sub database{
-	my ($self, @args) = @_;
-	my $default = 'MEDLINE';
-	if (! defined $self->medline && defined $self->pubmed) {
-		$default = 'PUBMED';
-	}
-	return $self->SUPER::database(@args) || $default;
+   my ($self, @args) = @_;
+
+   return $self->SUPER::database(@args) || 'MEDLINE';
 }
 
 =head2 primary_id
 
  Title   : primary_id
  Usage   :
- Function: Overrides DBLink primary_id to provide medline number, or pubmed
-           number if only that has been defined
+ Function: Overrides DBLink primary_id to provide medline number
  Example :
- Returns :
+ Returns : 
  Args    :
 
 
 =cut
 
 sub primary_id{
-	my ($self, @args) = @_;
-	if (@args) {
-		$self->medline(@args);
-	}
-	if (! defined $self->medline && defined $self->pubmed) {
-		return $self->pubmed;
-	}
-	return $self->medline;
+   my ($self, @args) = @_;
+
+   return $self->medline(@args);
 }
 
 =head2 optional_id
@@ -430,7 +398,7 @@ sub primary_id{
  Usage   :
  Function: Overrides DBLink optional_id to provide the PubMed number.
  Example :
- Returns :
+ Returns : 
  Args    :
 
 
@@ -447,7 +415,7 @@ sub optional_id{
  Title   : publisher
  Usage   : $self->publisher($newval)
  Function: Gives the publisher line. No attempt is made to parse the publisher line
- Example :
+ Example : 
  Returns : value of publisher
  Args    : newvalue (optional)
 
@@ -468,7 +436,7 @@ sub publisher {
  Title   : editors
  Usage   : $self->editors($newval)
  Function: Gives the editors line. No attempt is made to parse the editors line
- Example :
+ Example : 
  Returns : value of editors
  Args    : newvalue (optional)
 
@@ -491,7 +459,7 @@ sub editors {
  Function: Gives the encoded_ref line. No attempt is made to parse the encoded_ref line
  	(this is added for reading PDB records (REFN record), where this contains
 	 ISBN/ISSN/ASTM code)
- Example :
+ Example : 
  Returns : value of encoded_ref
  Args    : newvalue (optional)
 
@@ -506,51 +474,5 @@ sub encoded_ref {
    return $self->{'encoded_ref'};
 }
 
-=head2 consortium
-
- Title   : consortium
- Usage   : $self->consortium($newval)
- Function: Gives the consortium line. No attempt is made to parse the consortium line
- Example :
- Returns : value of consortium
- Args    : newvalue (optional)
-
-
-=cut
-
-sub consortium{
-   my ($self,$value) = @_;
-   if( defined $value) {
-      $self->{'consortium'} = $value;
-    }
-    return $self->{'consortium'};
-
-}
-
-=head2 gb_reference
-
- Title   : gb_reference
- Usage   : $obj->gb_reference($newval)
- Function: Gives the generic GenBank REFERENCE line. This is GenBank-specific.
-           If set, this includes everything on the reference line except
-	   the REFERENCE tag and the reference count.  This is mainly a
-	   fallback for the few instances when REFERENCE lines have unusual
-	   additional information such as split sequence locations, feature
-	   references, etc.  See Bug 2020 in Bugzilla for more information.
- Example :
- Returns : value of gb_reference (a scalar)
- Args    : on set, new value (a scalar or undef, optional)
-
-
-=cut
-
-sub gb_reference{
-   my ($self,$value) = @_;
-   if( defined $value) {
-      $self->{'gb_reference'} = $value;
-    }
-    return $self->{'gb_reference'};
-
-}
 
 1;
