@@ -1,4 +1,4 @@
-# $Id: Scaffold.pm,v 1.2 2002/11/11 18:16:30 lapp Exp $
+# $Id: Scaffold.pm,v 1.3 2003/06/04 08:36:36 heikki Exp $
 #
 #  BioPerl module for Bio::Assembly::Scaffold
 #
@@ -66,13 +66,12 @@ methods. Internal methods are usually preceded with a _
 package Bio::Assembly::Scaffold;
 
 use strict;
-use vars qw(@ISA $VERSION);
+use vars qw(@ISA);
 
 use Bio::Root::Root;
 use Bio::Assembly::ScaffoldI;
 use Bio::Annotation::Collection;
 
-$VERSION = '0.0.1';
 @ISA = qw(Bio::Root::Root Bio::Assembly::ScaffoldI);
 
 =head2 new ()
