@@ -1,4 +1,4 @@
-# $Id: Provider.pm,v 1.11.4.1 2006/10/02 23:10:12 sendu Exp $
+# $Id: Provider.pm,v 1.6 2003/05/30 15:33:00 jason Exp $
 #
 # BioPerl module for Bio::Biblio::Provider
 #
@@ -39,11 +39,11 @@ and they are inherited by all provider types.
 
 =item *
 
-OpenBQS home page: http://www.ebi.ac.uk/~senger/openbqs/
+OpenBQS home page: http://industry.ebi.ac.uk/openBQS
 
 =item *
 
-Comments to the Perl client: http://www.ebi.ac.uk/~senger/openbqs/Client_perl.html
+Comments to the Perl client: http://industry.ebi.ac.uk/openBQS/Client_perl.html
 
 =back
 
@@ -55,20 +55,21 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org              - General discussion
+  http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via the
-web:
+of the bugs and their resolution. Bug reports can be submitted via
+email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bioperl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHORS
 
-Heikki Lehvaslaiho (heikki-at-bioperl-dot-org),
+Heikki Lehvaslaiho (heikki@ebi.ac.uk),
 Martin Senger (senger@ebi.ac.uk)
 
 =head1 COPYRIGHT
@@ -90,10 +91,11 @@ This software is provided "as is" without warranty of any kind.
 
 package Bio::Biblio::Provider;
 use strict;
-use vars qw($AUTOLOAD);
+use vars qw(@ISA $AUTOLOAD);
 
+use Bio::Biblio::BiblioBase;
 
-use base qw(Bio::Biblio::BiblioBase);
+@ISA = qw(Bio::Biblio::BiblioBase);
 
 #
 # a closure with a list of allowed attribute names (these names
