@@ -1,4 +1,4 @@
-# $Id$
+# $Id: Thesis.pm,v 1.7 2003/05/30 15:33:00 jason Exp $
 #
 # BioPerl module for Bio::Biblio::Thesis
 #
@@ -15,7 +15,7 @@ Bio::Biblio::Thesis - Representation of thesis
 
     $obj = new Bio::Biblio::Thesis (-title => 'Perl on the edge');
 
- --- OR ---
+ #--- OR ---
 
     $obj = new Bio::Biblio::Thesis;
     $obj->title ('Perl on the edge');
@@ -33,7 +33,7 @@ There are no specific attributes in this class
 
 =head1 SEE ALSO
 
-=over
+=over 4
 
 =item *
 
