@@ -1,4 +1,4 @@
-# $Id: UniGene.pm,v 1.23 2002/12/05 13:46:30 heikki Exp $
+# $Id: UniGene.pm,v 1.28 2003/09/11 23:51:40 andrew Exp $
 #
 # BioPerl module for Bio::Cluster::UniGene.pm
 #
@@ -6,7 +6,7 @@
 #
 # Copyright Andrew Macgregor, Jo-Ann Stanton, David Green
 # Molecular Embryology Group, Anatomy & Structural Biology, University of Otago
-# http://anatomy.otago.ac.nz/meg
+# http://meg.otago.ac.nz/
 #
 # You may distribute this module under the same terms as perl itself
 #
@@ -170,7 +170,7 @@ methods. Internal methods are usually preceded with a "_".
 
 
 package Bio::Cluster::UniGene;
-use vars qw(@ISA $VERSION);
+use vars qw(@ISA);
 use strict;
 
 
@@ -186,29 +186,37 @@ use Bio::Factory::SequenceStreamI;
 use Bio::Seq::SeqFactory;
 use Bio::Cluster::UniGeneI;
 
-$VERSION = '1.1';
 @ISA = qw(Bio::Root::Root Bio::Cluster::UniGeneI
 	  Bio::IdentifiableI Bio::DescribableI Bio::AnnotatableI
 	  Bio::Factory::SequenceStreamI);
 
 my %species_map = (
-		 'Hs'  => "Homo sapiens",
-		 'Mm'  => "Mus musculus",
-		 'Rn'  => "Rattus norvegicus",
-		 'Bt'  => "Bos taurus",
-		 'Dr'  => "Danio rerio",
-		 'Dm'  => "Drosophila melanogaster",
-		 'Aga' => "Anopheles gambiae",
-		 'Xl'  => "Xenopus laevis",
-		 'At'  => "Arabidopsis thaliana",
-		 'Gma' => "Glycine max",
-		 'Hv'  => "Hordeum vulgare",
-		 'Les' => "Lycopersicon esculentum",
-		 'Mtr' => "Medicago truncatula",
-		 'Os'  => "Oryza sativa",
-		 'Ta'  => "Triticum aestivum",
-		 'Zm'  => "Zea mays",
-		 );
+		   'Aga' => "Anopheles gambiae",
+		   'At'  => "Arabidopsis thaliana",
+		   'Bt'  => "Bos taurus",
+		   'Cel' => "Caenorhabditis elegans",
+		   'Cin' => "Ciona intestinalis",
+		   'Cre' => "Chlamydomonas reinhardtii",
+		   'Ddi'  => "Dictyostelium discoideum",
+		   'Dr'  => "Danio rerio",
+		   'Dm'  => "Drosophila melanogaster",
+		   'Gga' => "Gallus gallus",
+		   'Gma' => "Glycine max",
+		   'Hs'  => "Homo sapiens",
+		   'Hv'  => "Hordeum vulgare",
+		   'Les' => "Lycopersicon esculentum",
+		   'Mtr' => "Medicago truncatula",
+		   'Mm'  => "Mus musculus",
+		   'Os'  => "Oryza sativa",
+		   'Ola' => "Oryzias latipes",
+		   'Rn'  => "Rattus norvegicus",
+		   'Str' => "Silurana tropicalis",
+		   'Sbi' => "Sorghum bicolor",
+		   'Ssc' => "Sus scrofa",
+		   'Ta'  => "Triticum aestivum",
+		   'Xl'  => "Xenopus laevis",
+		   'Zm'  => "Zea mays",
+		   );
 		 
 
 =head2 new
@@ -1185,7 +1193,7 @@ sub next_seq {
     # assemble the annotation collection
     my $ac = Bio::Annotation::Collection->new();
     foreach my $k (keys %$seq_h) {
-	next if $k =~ /acc|pid|nid/;
+	next if $k =~ /acc|pid|nid|version/;
 	my $ann = Bio::Annotation::SimpleValue->new(-tagname => $k,
 						    -value   => $seq_h->{$k});
 	$ac->add_Annotation($ann);
@@ -1198,6 +1206,7 @@ sub next_seq {
 	  -primary_id       => $seq_h->{nid} && $seq_h->{nid} =~ /^g\d+$/ ?
 				     substr($seq_h->{nid},1) : $seq_h->{nid},
 	  -display_id       => $seq_h->{acc},
+	  -seq_version		=> $seq_h->{version},
 	  -alphabet         => $obj->{'_alphabet'},
 	  -namespace        => $seq_h->{acc} =~ /^NM_/ ? 'RefSeq' : 'GenBank',
 	  -authority        => $obj->authority(), # default is NCBI
