@@ -1,4 +1,4 @@
-# $Id: Pair.pm,v 1.9 2002/11/09 09:09:36 heikki Exp $
+# $Id: Pair.pm,v 1.13 2003/12/19 02:59:17 jason Exp $
 #
 # bioperl module for Bio::Coordinate::Pair
 #
@@ -24,28 +24,36 @@ Bio::Coordinate::Pair - Continuous match between two coordinate sets
   my $match2 = Bio::Location::Simple->new
       (-seq_id => 'peptide', -start => 1, -end => 20, -strand=>1 );
   my $pair = Bio::Coordinate::Pair->new(-in => $match1,
-  					-out => $match2,
-  				        -negative => 0, # false, default
+  					-out => $match2
+                                        );
   # location to match
   $pos = Bio::Location::Simple->new 
       (-start => 25, -end => 25, -strand=> -1 );
 
-  # results are in Bio::Coordinate::Result
+  # results are in a Bio::Coordinate::Result
   # they can be Matches and Gaps; are  Bio::LocationIs
   $res = $pair->map($pos);
   $res->isa('Bio::Coordinate::Result');
-  $res->each_match, 1;
-  $res->each_gap, 0;
-  $res->each_location, 1;
-  $res->match->start, 5;
-  $res->match->end, 5;
-  $res->match->strand, -1;
-  $res->match->seq_id, 'peptide';
+  $res->each_match == 1;
+  $res->each_gap == 0;
+  $res->each_Location == 1;
+  $res->match->start == 5;
+  $res->match->end == 5;
+  $res->match->strand == -1;
+  $res->match->seq_id eq 'peptide';
 
 
 =head1 DESCRIPTION
 
-Class
+This class represents a one continuous match between two coordinate
+systems represented by Bio::Location::Simple objects. The relationship
+is directed and reversible. It implements methods to ensure internal
+consistency, and map continuous and split locations from one
+coordinate system to another.
+
+The map() method returns Bio::Coordinate::Results with
+Bio::Coordinate::Result::Gaps. The calling code have to deal (process
+or ignore) them.
 
 =head1 FEEDBACK
 
@@ -198,7 +206,7 @@ sub strand {
    $self->warn("Incoming coordinates are not defined")
        unless $self->in;
 
-   return $self->in->strand * $self->out->strand;
+   return ($self->in->strand || 0) * ($self->out->strand || 0);
 }
 
 =head2 test
@@ -226,11 +234,11 @@ sub test {
 =head2 map
 
  Title   : map
- Usage   : $newpos = $obj->map(5);
- Function: Map the location from the input coordinate system 
+ Usage   : $newpos = $obj->map($pos);
+ Function: Map the location from the input coordinate system
            to a new value in the output coordinate system.
  Example :
- Returns : new Bio::LocationI in the output coordiante system
+ Returns : new Bio::LocationI in the output coordinate system or undef
  Args    : Bio::LocationI object
 
 =cut
@@ -247,12 +255,44 @@ sub map {
    $self->throw("Output coordinate system not set")
        unless $self->out;
 
+
+   if ($value->isa("Bio::Location::SplitLocationI")) {
+
+       my $result = new Bio::Coordinate::Result;
+       my $split = new Bio::Location::Split(-seq_id=>$self->out->seq_id);
+       foreach my $loc ( $value->sub_Location(1) ) {
+           my $res = $self->_map($loc);
+           map { $result->add_sub_Location($_) } $res->each_Location;
+       }
+       return $result;
+   } else {
+       return $self->_map($value);
+   }
+}
+
+
+=head2 _map
+
+ Title   : _map
+ Usage   : $newpos = $obj->_map($simpleloc);
+ Function: Internal method that does the actual mapping. Called
+           multiple times by map() if the location to be mapped is a
+           split location
+ Example :
+ Returns : new location in the output coordinate system or undef
+ Args    : Bio::Location::Simple
+
+=cut
+
+sub _map {
+   my ($self,$value) = @_;
+
    my $result = new Bio::Coordinate::Result;
 
    my $offset = $self->in->start - $self->out->start;
-   my $start = $value->start - $offset;
-   my $end = $value->end - $offset;
-
+   my $start  = $value->start - $offset;
+   my $end    = $value->end - $offset;
+   
    my $match = Bio::Location::Simple->new;
    $match->location_type($value->location_type);
    $match->strand($self->strand);
@@ -265,7 +305,7 @@ sub map {
        $match->seq_id($self->out->seq_id);
        $result->seq_id($self->out->seq_id);
 
-       if ($self->strand == 1) {
+       if ($self->strand >= 0) {
 	   $match->start($start);
 	   $match->end($end);
        } else {
@@ -318,7 +358,7 @@ sub map {
        # match
        $match->seq_id($self->out->seq_id);
 
-       if ($self->strand == 1) {
+       if ($self->strand >= 0) {
 	   $match->start($self->out->start);
 	   $match->end($end);
        } else {
@@ -339,7 +379,7 @@ sub map {
 	   $match->strand($match->strand * $value->strand);
 	   $result->strand($match->strand);
        }
-       if ($self->strand == 1) {
+       if ($self->strand >= 0) {
 	   $match->start($start);
 	   $match->end($self->out->end);
        } else {
@@ -400,4 +440,5 @@ sub map {
    return $result;
 }
 
+
 1;
