@@ -1,4 +1,4 @@
-# $Id: Result.pm,v 1.5 2002/11/12 09:13:11 heikki Exp $
+# $Id: Result.pm,v 1.8 2003/12/19 03:02:43 jason Exp $
 #
 # bioperl module for Bio::Coordinate::Result
 #
@@ -23,9 +23,10 @@ Bio::Coordinate::Result - Results from coordinate transformation
 
 =head1 DESCRIPTION
 
-The results from Bio::Coordinate::MapperI are kept in an array. The
-results are either Matches or Gaps.  SeeL<Bio::Coordinate::Result::Match>
-and L<Bio::Coordinate::Result::Match>.
+The results from Bio::Coordinate::MapperI are kept in an object which
+itself is a split location, See L<Bio::Location::Split>. The results
+are either Matches or Gaps.  See L<Bio::Coordinate::Result::Match> and
+L<Bio::Coordinate::Result::Match>.
 
 If only one Match is returned, there is a convenience method of
 retrieving it or accessing its methods. Same holds true for a Gap.
@@ -78,14 +79,15 @@ use vars qw(@ISA );
 use strict;
 
 use Bio::Location::Split;
+use Bio::Coordinate::ResultI;
 
-@ISA = qw(Bio::Location::Split);
+@ISA = qw(Bio::Location::Split Bio::Coordinate::ResultI);
 
 
 =head2 add_location
 
- Title   : add_location
- Usage   : $obj->add_location($variant)
+ Title   : add_sub_Location
+ Usage   : $obj->add_sub_Location($variant)
  Function: 
 
            Pushes one Bio::LocationI into the list of variants.
@@ -128,8 +130,7 @@ sub add_result {
   $self->throw("Is not a Bio::Coordinate::Result but [$value]")
       unless $value->isa('Bio::Coordinate::Result');
 
-  map {  $self->add_sub_Location($_);} $value->each_Location;
-
+  map { $self->add_sub_Location($_) } $value->each_Location;
 }
 
 =head2 seq_id
@@ -156,7 +157,6 @@ sub seq_id {
     } else {
 	return undef;
     }
-
 }
 
 
@@ -179,10 +179,9 @@ These methods are shortcuts to Match and Gap locations.
 
 =cut
 
-sub each_gap{
+sub each_gap {
    my ($self) = @_;
 
-
    my @gaps;
    foreach my $gap ($self->each_Location) {
        push @gaps, $gap if $gap->isa('Bio::Coordinate::Result::Gap');
@@ -261,4 +260,34 @@ sub gap {
    return $self->{'_gap'};
 }
 
+
+=head2 purge_gaps
+
+ Title   : purge_gaps
+ Usage   : $gap_count = $obj->purge_gaps;
+ Function: remove all gaps from the Result
+ Returns : count of removed gaps
+ Args    : 
+
+=cut
+
+sub purge_gaps {
+    my ($self) = @_;
+    my @matches;
+    my $count = 0;
+
+    foreach my $loc ($self->each_Location) {
+        if ($loc->isa('Bio::Coordinate::Result::Match')) {
+            push @matches, $loc;
+        } else {
+            $count++
+        }
+    }
+    @{$self->{'_sublocations'}} = ();
+    delete $self->{'_gap'} ;
+    push @{$self->{'_sublocations'}}, @matches;
+    return $count;
+}
+
+
 1;
