@@ -2,7 +2,7 @@ package Bio::DB::GFF::Adaptor::memory;
 
 =head1 NAME
 
-Bio::DB::GFF::Adaptor::memory -- Bio::DB::GFF database adaptor for in-memory databases
+Bio::DB::GFF::Adaptor::dbi::mysql -- Database adaptor for a specific mysql schema
 
 =head1 SYNOPSIS
 
@@ -59,7 +59,7 @@ it under the same terms as Perl itself.
 =cut
 
 use strict;
-# $Id: memory.pm,v 1.46.4.1 2006/10/02 23:10:16 sendu Exp $
+# $Id: memory.pm,v 1.21 2003/12/17 18:25:42 scain Exp $
 # AUTHOR: Shulamit Avraham
 # This module needs to be cleaned up and documented
 
@@ -69,15 +69,19 @@ use strict;
 # Inherits from Bio::DB::GFF.
 
 
+#use lib './blib/lib';
+#use lib '/u/swiss/shuly/bioperl-live';
+# use lib '/a/swiss/export/home/shuly/bioperl-live';
+use Bio::DB::GFF;
 use Bio::DB::GFF::Util::Rearrange; # for rearrange()
-use Bio::DB::GFF::Adaptor::memory::iterator;
+use Bio::DB::GFF::Adaptor::memory_iterator;
 use File::Basename 'dirname';
-use Bio::DB::GFF::Adaptor::memory::feature_serializer qw(@hash2array_map);
 
+use vars qw(@ISA);
 
-use constant MAX_SEGMENT => 1_000_000_000;  # the largest a segment can get
+use constant MAX_SEGMENT => 100_000_000;  # the largest a segment can get
 
-use base qw(Bio::DB::GFF);
+@ISA =  qw(Bio::DB::GFF);
 
 sub new {
   my $class = shift ;
@@ -105,8 +109,8 @@ sub load_or_store_fasta {
       or
       (-d $fasta && -w $fasta)) {
     require Bio::DB::Fasta;
-    my $dna_db = eval {Bio::DB::Fasta->new($fasta);}
-      or warn "$@\nCan't open sequence file(s). Use -gff instead of -dir if you wish to load features without sequence.\n";
+    my $dna_db = eval {Bio::DB::Fasta->new($fasta)} 
+      or warn "No sequence available. Use -gff instead of -dir if you wish to load features without sequence.\n";
     $dna_db && $self->dna_db($dna_db);
   } else {
     $self->load_fasta($fasta);
@@ -134,9 +138,8 @@ sub get_dna {
   if (my $dna_db = $self->dna_db) {
     return $dna_db->seq($id,$start=>$stop);
   }
-  return '' unless $self->{dna};
 
-  return $self->{dna}{$id} unless defined $start || defined $stop;
+  return $self->{dna}{$id} if !defined $start || !defined $stop;
   $start = 1 if !defined $start;
 
   my $reversed = 0;
@@ -153,33 +156,14 @@ sub get_dna {
   $dna;
 }
 
-sub setup_load {
-  my $self = shift;
-  $self->{tmp}  = {};
-  $self->{data} = [];
-  1;
-}
-
-sub finish_load {
-  my $self = shift;
-  my $idx  = 0;
-  foreach my $arrayref (values %{$self->{tmp}}) {
-    foreach (@$arrayref) {$_->{feature_id} = $idx++; }
-    push @{$self->{data}},@$arrayref;
-  }
-  1;
-}
-
 # this method loads the feature as a hash into memory -
 # keeps an array of features-hashes as an in-memory db
 sub load_gff_line {
   my $self = shift;
   my $feature_hash  = shift;
-  $feature_hash->{strand} = ''  if $feature_hash->{strand} && $feature_hash->{strand} eq '.';
-  $feature_hash->{phase}  = ''  if $feature_hash->{phase}  && $feature_hash->{phase} eq '.';
-  $feature_hash->{gclass} = 'Sequence' unless length $feature_hash->{gclass} > 0;
-  # sort by group please
-  push @{$self->{tmp}{$feature_hash->{gclass},$feature_hash->{gname}}},$feature_hash;
+  $feature_hash->{strand} = '' if $feature_hash->{strand} && $feature_hash->{strand} eq '.';
+  $feature_hash->{phase} = ''  if $feature_hash->{phase}  && $feature_hash->{phase} eq '.';
+  push @{$self->{data}},$feature_hash;
 }
 
 # given sequence name, return (reference,start,stop,strand)
@@ -188,9 +172,9 @@ sub get_abscoords {
   my ($name,$class,$refseq) = @_;
   my %refs;
   my $regexp;
-  
+
   if ($name =~ /[*?]/) {  # uh oh regexp time
-    $name = quotemeta($name);
+    $name =~ quotemeta($name);
     $name =~ s/\\\*/.*/g;
     $name =~ s/\\\?/.?/g;
     $regexp++;
@@ -202,13 +186,9 @@ sub get_abscoords {
 
     my $no_match_class_name;
     my $empty_class_name;
-    my $class_matches = !defined($feature->{gclass}) ||
-      length($feature->{gclass}) == 0 ||
-	$feature->{gclass} eq $class;
-
-    if (defined $feature->{gname}) {
-      my $matches = $class_matches
-	&& ($regexp ? $feature->{gname} =~ /$name/i : lc($feature->{gname}) eq lc($name));
+    if (defined $feature->{gname} and defined $feature->{gclass}){
+      my $matches = $feature->{gclass} eq $class
+	&& ($regexp ? $feature->{gname} =~ /$name/i : $feature->{gname} eq $name);
       $no_match_class_name = !$matches;  # to accomodate Shuly's interesting logic
     }
 
@@ -216,21 +196,25 @@ sub get_abscoords {
       $empty_class_name = 1;
     }
 
-    if ($no_match_class_name){
+    if ($no_match_class_name || $empty_class_name){
+
       my $feature_attributes = $feature->{attributes};
       my $attributes = {Alias => $name};
-      if (!$self->_matching_attributes($feature_attributes,$attributes)){
+      if (!_matching_attributes($feature_attributes,$attributes)){
 	next;
       }
     }
-
     push @{$refs{$feature->{ref}}},$feature;
   }
 
   # find out how many reference points we recovered
+
   if (! %refs) {
     $self->error("$name not found in database");
     return;
+  } elsif (keys %refs > 1) {
+    $self->error("$name has more than one reference sequence in database");
+    return;
   }
 
   # compute min and max
@@ -240,32 +224,26 @@ sub get_abscoords {
 
   my @found_segments;
   foreach my $ref (keys %refs) {
-    next if defined($refseq) and lc($ref) ne lc($refseq);
+    next if defined($refseq) and $ref ne $refseq;
     my @found = @{$refs{$ref}};
-    my ($strand,$start,$stop,$name);
+    my ($strand,$start,$stop);
     foreach (@found) {
       $strand ||= $_->{strand};
       $strand = '+' if $strand && $strand eq '.'; 
       $start  = $_->{start} if !defined($start) || $start > $_->{start};
       $stop   = $_->{stop}  if !defined($stop)  || $stop  < $_->{stop};
-      $name ||= $_->{gname};
     }
-    push @found_segments,[$ref,$class,$start,$stop,$strand,$name];
+    push @found_segments,[$ref,$class,$start,$stop,$strand];
 
   }
-  
   return \@found_segments;
 }
 
 sub search_notes {
   my $self = shift;
   my ($search_string,$limit) = @_;
-
-  $search_string =~ tr/*?//d;
-
   my @results;
   my @words = map {quotemeta($_)} $search_string =~ /(\w+)/g;
-  my $search = join '|',@words;
 
   for my $feature (@{$self->{data}}) {
     next unless defined $feature->{gclass} && defined $feature->{gname}; # ignore NULL objects
@@ -274,34 +252,20 @@ sub search_notes {
     my @values     = map {$_->[1]} @attributes;
     my $value      = "@values";
     my $matches    = 0;
+    my $note;
     for my $w (@words) {
-      my @hits = $value =~ /($w)/ig;
+      my @hits = $value =~ /($w)/g;
+      $note ||= $value if @hits;
       $matches += @hits;
     }
     next unless $matches;
 
     my $relevance = 10 * $matches;
     my $featname = Bio::DB::GFF::Featname->new($feature->{gclass}=>$feature->{gname});
-    my $note;
-    $note   = join ' ',map {$_->[1]} grep {$_->[0] eq 'Note'}                @{$feature->{attributes}};
-    $note  .= join ' ',grep /$search/,map {$_->[1]} grep {$_->[0] ne 'Note'} @{$feature->{attributes}};
     push @results,[$featname,$note,$relevance];
-    last if defined $limit && @results >= $limit;
+    last if @results >= $limit;
   }
-   
-  #added result filtering so that this method returns the expected results
-  #this section of code used to be in GBrowse's do_keyword_search method
-
-  my $match_sub = 'sub {';
-  foreach (split /\s+/,$search_string) {
-    $match_sub .= "return unless \$_[0] =~ /\Q$_\E/i; ";
-  }
-  $match_sub .= "};";
-  my $match = eval $match_sub;
-
-  my @matches = grep { $match->($_->[1]) } @results;
-
-  return @matches;
+  @results;
 }
 
 sub _delete_features {
@@ -317,7 +281,7 @@ sub _delete_features {
 
 sub _delete {
   my $self = shift;
-  my $delete_spec = shift;
+    my $delete_spec = shift;
   my $ranges      = $delete_spec->{segments} || [];
   my $types       = $delete_spec->{types}    || [];
   my $force       = $delete_spec->{force};
@@ -357,13 +321,11 @@ sub do_attributes{
   my ($feature_id,$tag) = @_;
   my $attr ;
 
-  #my $feature = ${$self->{data}}[$feature_id];
-  my $feature = $self->_basic_features_by_id($feature_id);
-
+  my $feature = ${$self->{data}}[$feature_id];
   my @result;
   for my $attr (@{$feature->{attributes}}) {
     my ($attr_name,$attr_value) = @$attr ;
-    if (defined($tag) && lc($attr_name) eq lc($tag)){push @result,$attr_value;}
+    if (defined($tag) && $attr_name eq $tag){push @result,$attr_value;}
     elsif (!defined($tag)) {push @result,($attr_name,$attr_value);}
   }
   return @result;
@@ -386,10 +348,23 @@ sub _feature_by_attribute{
       my ($attr_name,$attr_value) = @$attr ;
       #there could be more than one set of attributes......
       foreach (keys %$attributes) {
-	if (lc($_) eq lc($attr_name) && lc($attributes->{$_}) eq lc($attr_value)) {
-	  $callback->($self->_hash_to_array($feature));
-	  $count++;
-	}
+	if ($_ eq $attr_name && $attributes->{$_} eq $attr_value){
+           $callback->($feature->{ref},
+	        $feature->{start},
+	        $feature->{stop},
+	        $feature->{source},
+	        $feature->{method},
+	        $feature->{score},
+	        $feature->{strand},
+	        $feature->{phase},
+	        $feature->{gclass},
+	        $feature->{gname},
+		$feature->{tstart},
+		$feature->{tstop},
+	        $feature_id,
+		$feature_group_id);
+	   $count++;
+        }
       }
     }
   }
@@ -397,6 +372,7 @@ sub _feature_by_attribute{
 }
 
 
+
 # This is the low-level method that is called to retrieve GFF lines from
 # the database.  It is responsible for retrieving features that satisfy
 # range and feature type criteria, and passing the GFF fields to a
@@ -406,19 +382,20 @@ sub get_features{
   my $self = shift;
   my $count = 0;
   my ($search,$options,$callback) = @_;
+  my $data = \@{$self->{data}};
 
   my $found_features;
 
-  $found_features = $self->_get_features_by_search_options($search,$options);
+  $found_features = _get_features_by_search_options($data,$search,$options);
 
   # only true if the sort by group option was specified
-  @{$found_features} = sort {lc("$a->{gclass}:$a->{gname}") cmp lc("$b->{gclass}:$b->{gname}")}
+  @{$found_features} = sort {"$a->{gclass}:$a->{gname}" cmp "$b->{gclass}:$b->{gname}"} 
     @{$found_features} if $options->{sort_by_group} ;
 
   for my $feature (@{$found_features}) {  # only true if the sort by group option was specified
     $count++;
     $callback->(
-		$self->_hash_to_array($feature)
+		@{$feature}{qw(ref start stop source method score strand phase gclass gname tstart tstop feature_id feature_group_id)}
 	       );
   }
 
@@ -449,6 +426,7 @@ sub _feature_by_name {
   my ($class,$name,$location,$callback) = @_;
   $callback || $self->throw('must provide a callback argument');
   my $count = 0;
+  my $id    = -1;
   my $regexp;
 
   if ($name =~ /[*?]/) {  # uh oh regexp time
@@ -458,17 +436,32 @@ sub _feature_by_name {
     $regexp++;
   }
 
-  for my $feature (@{$self->{data}}) {
-    next unless ($regexp && $feature->{gname} =~ /$name/i) || lc($feature->{gname})  eq lc($name);
-    next if defined($feature->{gclass}) && length($feature->{gclass}) > 0 && $feature->{gclass} ne $class;
 
+  for my $feature (@{$self->{data}}) {
+    $id++;
+    next unless ($regexp && $feature->{gname} =~ /$name/i) || $feature->{gname}  eq $name;
+    next unless $feature->{gclass} eq $class;
     if ($location) {
       next if $location->[0] ne $feature->{ref};
       next if $location->[1] && $location->[1] > $feature->{stop};
       next if $location->[2] && $location->[2] < $feature->{start};
     }
     $count++;
-    $callback->($self->_hash_to_array($feature),0);
+    $callback->(@{$feature}{qw(
+			       ref
+			       start
+			       stop
+			       source
+			       method
+			       score
+			       strand
+			       phase
+			       gclass
+			       gname
+			       tstart
+			       tstop
+			      )},$id,0
+	       );
   }
   return $count;
 }
@@ -486,42 +479,48 @@ sub _feature_by_id{
   my $count = 0;
   if ($type eq 'feature'){
     for my $feature_id (@$ids){
-      my $feature = $self->_basic_features_by_id($feature_id);
-      $callback->($self->_hash_to_array($feature)) if $callback;
-      $count++;
+       my $feature = ${$self->{data}}[$feature_id];
+       
+       $callback->($feature->{ref},
+	        $feature->{start},
+	        $feature->{stop},
+	        $feature->{source},
+	        $feature->{method},
+	        $feature->{score},
+	        $feature->{strand},
+	        $feature->{phase},
+	        $feature->{gclass},
+	        $feature->{gname},
+		$feature->{tstart},
+		$feature->{tstop},
+	        $feature_id,
+		$feature_group_id);
+	   $count++;			
+    
     }
   }
 }
 
-sub _basic_features_by_id{
-  my $self = shift;
-  my ($ids) = @_;
-  
-  $ids = [$ids] unless ref $ids =~ /ARRAY/;
-
-  my @result;
-  for my $feature_id (@$ids){
-	  push @result, ${$self->{data}}[$feature_id];
-  }
-  return wantarray() ? @result : $result[0];
-}
 
 # This method is similar to get_features(), except that it returns an
-# iterator across the query.
-# See Bio::DB::GFF::Adaptor::memory::iterator.
+# iterator across the query.  
+# See Bio::DB::GFF::Adaptor::memory_iterator.
 
 sub get_features_iterator {
   my $self = shift;
   my ($search,$options,$callback) = @_;
   $callback || $self->throw('must provide a callback argument');
 
-  my $results = $self->_get_features_by_search_options($search,$options);
-  my $results_array = $self->_convert_feature_hash_to_array($results);
+  my $data = \@{$self->{data}};
+  my $results = _get_features_by_search_options($data,$search,$options);
+  my $results_array = _convert_feature_hash_to_array($results);
 
-  return Bio::DB::GFF::Adaptor::memory::iterator->new($results_array,$callback);
+  return Bio::DB::GFF::Adaptor::memory_iterator->new($results_array,$callback);
 }
 
 
+
+
 # This method is responsible for fetching the list of feature type names.
 # The query may be limited to a particular range, in
 # which case the range is indicated by a landmark sequence name and
@@ -536,7 +535,6 @@ sub get_features_iterator {
 sub get_types {
   my $self = shift;
   my ($srcseq,$class,$start,$stop,$want_count,$typelist) = @_;
-	  
   my(%result,%obj);
 
   for my $feature (@{$self->{data}}) {
@@ -546,24 +544,24 @@ sub get_types {
     my $feature_class = $feature->{class};
     my $feature_method = $feature->{method};
     my $feature_source = $feature->{source};
-
+   
     if (defined $srcseq){
-      next unless lc($feature_ref) eq lc($srcseq);
+      next unless $feature_ref eq $srcseq ;
     }
-
+    
     if (defined $class){ 
-      next unless defined $feature_class && $feature_class eq $class ;
+      next unless $feature_class eq $class ;
     }
-
+    
      # the requested range should OVERLAP the retrieved features
      if (defined $start or defined $stop) {
       $start = 1           unless defined $start;
       $stop  = MAX_SEGMENT unless defined $stop;
       next unless $feature_stop >= $start && $feature_start <= $stop;
     }
-
+    
     if (defined $typelist && @$typelist){
-      next unless $self->_matching_typelist($feature_method,$feature_source,$typelist);
+      next unless _matching_typelist($feature_method,$feature_source,$typelist);
     }
 
     my $type = Bio::DB::GFF::Typename->new($feature_method,$feature_source);
@@ -571,32 +569,25 @@ sub get_types {
     $obj{$type} = $type;
 
   }   #end features loop
-
+  
   return $want_count ? %result : values %obj;
+ 
 }
 
-sub classes {
-  my $self = shift;
-  my %classes;
-  for my $feature (@{$self->{data}}) {
-    $classes{$feature->{gclass}}++;
-  }
-  my @classes = sort keys %classes;
-  return @classes;
-}
+
+
 
 # Internal method that performs a search on the features array, 
 # sequentialy retrieves the features, and performs a check on each feature
 # according to the search options.
 sub _get_features_by_search_options{
   my $count = 0;
-  my ($self, $search,$options) = @_;
+  my ($data,$search,$options) = @_;
   my ($rangetype,$refseq,$class,$start,$stop,$types,$sparse,$order_by_group,$attributes) = 
     (@{$search}{qw(rangetype refseq refclass start stop types)},
-     @{$options}{qw(sparse sort_by_group ATTRIBUTES)}) ;
-
+    @{$options}{qw(sparse sort_by_group ATTRIBUTES)}) ;
+					       
   my @found_features;
-  my $data = $self->{data};
 
   my $feature_id = -1 ;
   my $feature_group_id = undef;
@@ -604,19 +595,19 @@ sub _get_features_by_search_options{
   for my $feature (@{$data}) {
 
     $feature_id++;
-
+    
     my $feature_start = $feature->{start};
     my $feature_stop  = $feature->{stop};
     my $feature_ref   = $feature->{ref};
-
+    
     if (defined $refseq){
-      next unless lc($feature_ref) eq lc($refseq);
+      next unless $feature_ref eq $refseq;
     }
 
      if (defined $start or defined $stop) {
       $start = 0               unless defined($start);
       $stop  = MAX_SEGMENT     unless defined($stop);
-
+    
       if ($rangetype eq 'overlaps') {
 	next unless $feature_stop >= $start && $feature_start <= $stop;
       } elsif ($rangetype eq 'contains') {
@@ -628,62 +619,100 @@ sub _get_features_by_search_options{
       }
 
     }
-
+    
     my $feature_source = $feature->{source};
     my $feature_method = $feature->{method};
 
     if (defined $types && @$types){
-      next unless $self->_matching_typelist($feature_method,$feature_source,$types);
-    }
+      next unless _matching_typelist($feature_method,$feature_source,$types);
+    } 
 
     my $feature_attributes = $feature->{attributes};
     if (defined $attributes){
-      next unless $self->_matching_attributes($feature_attributes,$attributes);
-    }
-
+      next unless _matching_attributes($feature_attributes,$attributes);
+    } 
+    
     # if we get here, then we have a feature that meets the criteria.
     # Then we just push onto an array
     # of found features and continue. 
-
+   
     my $found_feature = $feature ;
     $found_feature->{feature_id} = $feature_id;
     $found_feature->{group_id} = $feature_group_id;
     push @found_features,$found_feature;
+   
   }
 
   return \@found_features; 
 }
 
 
-sub _hash_to_array {
-  my ($self,$feature_hash) = @_;
-  my @array = @{$feature_hash}{@hash2array_map};
-  return wantarray ? @array : \@array;
-}
+
+
 
 # this subroutine is needed for convertion of the feature from hash to array in order to 
 # pass it to the callback subroutine
 sub _convert_feature_hash_to_array{
-  my ($self, $feature_hash_array) = @_;
-  my @features_array_array = map {scalar $self->_hash_to_array($_)} @$feature_hash_array;
+  my @features_hash_array = @_;
+
+  use constant FREF    => 0;
+  use constant FSTART  => 1;
+  use constant FSTOP   => 2;
+  use constant FSOURCE => 3;
+  use constant FMETHOD => 4;
+  use constant FSCORE  => 5;
+  use constant FSTRAND => 6;
+  use constant FPHASE  => 7;
+  use constant GCLASS  => 8;
+  use constant GNAME   => 9;
+  use constant TSTART  => 10;
+  use constant TSTOP   => 11;
+  use constant FID     => 12;
+  use constant GID     => 13;
+
+  my @features_array_array;
+  my $feature_count = 0;
+   
+  for my $feature_hash (@{$features_hash_array[0]}){
+    my @feature_array;
+
+    $feature_array[FREF]    = $feature_hash->{ref};
+    $feature_array[FSTART]  = $feature_hash->{start};
+    $feature_array[FSTOP]   = $feature_hash->{stop};  
+    $feature_array[FSOURCE] = $feature_hash->{source};
+    $feature_array[FMETHOD] = $feature_hash->{method};
+    $feature_array[FSCORE]  = $feature_hash->{score};
+    $feature_array[FSTRAND] = $feature_hash->{strand};  
+    $feature_array[FPHASE ] = $feature_hash->{phase};
+    $feature_array[GCLASS]  = $feature_hash->{gclass};  
+    $feature_array[GNAME]   = $feature_hash->{gname};
+    $feature_array[TSTART]  = $feature_hash->{tstart};
+    $feature_array[TSTOP]   = $feature_hash->{tstop};
+    $feature_array[FID]     = $feature_hash->{feature_id};  
+    $feature_array[GID]     = $feature_hash->{group_id};
+
+    $features_array_array[$feature_count] = \@feature_array;
+    $feature_count++;
+  }
   return \@features_array_array;
 }
 
 sub _matching_typelist{ 
-  my ($self, $feature_method,$feature_source,$typelist) = @_; 
+  my ($feature_method,$feature_source,$typelist) = @_; 
   foreach (@$typelist) {
 	 my ($search_method,$search_source) = @$_;
-	 next if lc($search_method) ne lc($feature_method);
-	 next if defined($search_source) && lc($search_source) ne lc($feature_source);
+	 next if $search_method ne $feature_method;
+	 next if defined($search_source) && $search_source ne $feature_source;
 	 return 1;
   }
   return 0;
 }
 
-sub _matching_attributes {
-  my ($self, $feature_attributes,$attributes) = @_ ;
+sub _matching_attributes{
+  my ($feature_attributes,$attributes) = @_ ;
   foreach (keys %$attributes) {
     return 0 if !_match_all_attr_in_feature($_,$attributes->{$_},$feature_attributes)
+   
   }
   return 1;
 }
@@ -700,6 +729,8 @@ sub _match_all_attr_in_feature{
 
 
 sub do_initialize { 1; }
+sub setup_load { }
+sub finish_load { 1; }
 sub get_feature_by_group_id{ 1; }
 
 1;
