@@ -73,15 +73,17 @@ use Bio::DB::GFF::Homol;
 use Bio::SeqFeatureI;
 use Bio::Root::Root;
 use Bio::LocationI;
+use Data::Dumper;
 
-use vars qw($VERSION @ISA $AUTOLOAD);
+use vars qw(@ISA $AUTOLOAD);
 @ISA = qw(Bio::DB::GFF::RelSegment Bio::SeqFeatureI 
 	  Bio::Root::Root);
 
-$VERSION = '0.61';
 #' 
 
 *segments = \&sub_SeqFeature;
+*get_SeqFeatures = \&sub_SeqFeature;
+
 my %CONSTANT_TAGS = (method=>1, source=>1, score=>1, phase=>1, notes=>1, id=>1, group=>1);
 
 =head2 new_from_parent
@@ -431,13 +433,55 @@ location of the alignment.
 
 =cut
 
+
 sub target {
-  my $self = shift;
-  my $group = $self->group or return;
-  return unless $group->can('start');
-  $group;
+    my $self = shift;
+    my $group = $self->group or return;
+    return unless $group->can('start');
+    $group;
 }
 
+=head2 flatten_target
+
+ Title   : flatten_target
+ Usage   : $target = $f->flatten_target($f->target)
+ Function: flatten a target object
+ Returns : a string (GFF2), an array [GFF2.5] or an array ref [GFF3]
+ Args    : a target object (required), GFF version (optional) 
+ Status  : Public
+
+This method flattens a target object into text for
+GFF dumping.  If a second argument is provided, version-specific
+vocabulary is used for the flattened target.
+
+=cut
+
+sub flatten_target {
+    my $self = shift;
+    my $t    = shift || return;
+    my $v    = shift;
+
+    return 0 unless $t->can('start');
+    my $class = $t->class;
+    my $name  = $t->name;
+    my $start = $t->start;
+    my $stop  = $t->stop;
+
+    $v ||=2;
+    if ( $v == 2.5 ) {
+	
+	print STDERR qq(Target "$class:$name"), "tstart $start", "tstop $stop\n";
+	return (qq(Target "$class:$name"), "tstart $start", "tstop $stop");
+    }
+    elsif ( $v == 3 ) {
+	return [Target=>"$name $start $stop"];
+    }
+    else {
+	return qq(Target "$class:$name" $start $stop);
+    }
+}
+
+
 =head2 hit
 
  Title   : hit
@@ -557,8 +601,9 @@ feature.  For those features that contain heterogeneous subfeatures,
 you can retrieve a subset of the subfeatures by providing a method
 name to filter on.
 
-For AcePerl compatibility, this method may also be called as
-segments().
+This method may also be called as segments() or get_SeqFeatures().
+
+
 
 =cut
 
@@ -599,6 +644,22 @@ sub add_subfeature {
   push @{$subfeat},$feature;
 }
 
+=head2 attach_seq
+
+ Title   : attach_seq
+ Usage   : $sf->attach_seq($seq)
+ Function: Attaches a Bio::Seq object to this feature. This
+           Bio::Seq object is for the *entire* sequence: ie
+           from 1 to 10000
+ Example :
+ Returns : TRUE on success
+ Args    : a Bio::PrimarySeqI compliant object
+
+=cut
+
+sub attach_seq { }
+
+
 =head2 location
 
  Title   : location
@@ -688,9 +749,10 @@ sub merged_segments {
   for my $s (@segs) {
     my $previous = $merged[-1] if @merged;
     my ($pscore,$score) = (eval{$previous->score}||0,eval{$s->score}||0);
-    if (defined($previous) 
+    if (defined($previous)
 	&& $previous->stop+1 >= $s->start
-	&& $previous->score == $s->score
+	&& (!defined($s->score) || $previous->score  == $s->score)
+	&& $previous->method eq $s->method
        ) {
       if ($self->absolute && $self->strand < 0) {
 	$previous->{start} = $s->{start};
@@ -703,8 +765,8 @@ sub merged_segments {
 	my $cg = $s->{group};
 	$g->{stop} = $cg->{stop};
       }
-    } elsif (defined($previous) 
-	     && $previous->start == $s->start 
+    } elsif (defined($previous)
+	     && $previous->start == $s->start
 	     && $previous->stop == $s->stop) {
       next;
     } else {
@@ -837,7 +899,7 @@ is equivalent to this call:
 
 The following Bio::SeqFeatureI methods are implemented:
 
-primary_tag(), source_tag(), all_tags(), has_tag(), each_tag_value().
+primary_tag(), source_tag(), all_tags(), has_tag(), each_tag_value() [renamed get_tag_values()].
 
 =cut
 
@@ -845,11 +907,14 @@ primary_tag(), source_tag(), all_tags(), has_tag(), each_tag_value().
 *source_tag  = \&source;
 sub all_tags {
   my $self = shift;
-  my @tags = keys %CONSTANT_TAGS;
+  my %atts = $self->attributes;
+  my @tags = keys %atts;
+
   # autogenerated methods
-  if (my $subfeat = $self->{subfeatures}) {
-    push @tags,keys %$subfeat;
-  }
+  #if (my $subfeat = $self->{subfeatures}) {
+  #  push @tags,keys %$subfeat;
+  #}
+
   @tags;
 }
 *get_all_tags = \&all_tags;
@@ -857,13 +922,22 @@ sub all_tags {
 sub has_tag {
   my $self = shift;
   my $tag  = shift;
-  my %tags = map {$_=>1} $self->all_tags;
+  my %att  = $self->attributes;
+  my %tags = map {$_=>1} ( $self->all_tags );
+  print map { $_, '=>', $tags{$_}, "\n" } keys %tags;
   return $tags{$tag};
 }
-sub each_tag_value {
+
+*each_tag_value = \&get_tag_values;
+
+sub get_tag_values {
   my $self = shift;
   my $tag  = shift;
   return $self->$tag() if $CONSTANT_TAGS{$tag};
+  
+  my $atts = $self->attributes;
+  return @{$atts->{$tag}} if $atts && $atts->{$tag};
+
   $tag = ucfirst $tag;
   return $self->$tag();  # try autogenerated tag
 }
@@ -912,12 +986,13 @@ sub adjust_bounds {
 	# fix up our bounds to hold largest subfeature
 	my($start,$stop,$strand) = $feat->adjust_bounds;
 	$self->{fstrand} = $strand unless defined $self->{fstrand};
-	if ($start <= $stop) {
-	  $self->{start} = $start if !defined($self->{start}) || $start < $self->{start};
-	  $self->{stop}  = $stop  if !defined($self->{stop})  || $stop  > $self->{stop};
+	my ($low,$high)  = $start < $stop ? ($start,$stop) : ($stop,$start);
+	if ($self->{fstrand} ne '-') {
+	  $self->{start} = $low   if !defined($self->{start}) || $low < $self->{start};
+	  $self->{stop}  = $high  if !defined($self->{stop})  || $high  > $self->{stop};
 	} else {
-	  $self->{start} = $start if !defined($self->{start}) || $start > $self->{start};
-	  $self->{stop}  = $stop  if !defined($self->{stop})  || $stop  < $self->{stop};
+	  $self->{start} = $high  if !defined($self->{start}) || $high > $self->{start};
+	  $self->{stop}  = $low   if !defined($self->{stop})  || $low  < $self->{stop};
 	}
 
 	# fix up endpoints of targets too (for homologies only)
@@ -925,7 +1000,7 @@ sub adjust_bounds {
 	next unless $h && $h->isa('Bio::DB::GFF::Homol');
 	next unless $g && $g->isa('Bio::DB::GFF::Homol');
 	($start,$stop) = ($h->{start},$h->{stop});
-	if ($h->strand >= 0) {
+	if ($start <= $stop) {
 	  $g->{start} = $start if !defined($g->{start}) || $start < $g->{start};
 	  $g->{stop}  = $stop  if !defined($g->{stop})  || $stop  > $g->{stop};
 	} else {
@@ -1005,8 +1080,24 @@ sub name {
   return $self->group || $self->SUPER::name;
 }
 
+=head2 gff_string
+
+ Title   : gff_string
+ Usage   : $string = $feature->gff_string
+ Function: return GFF2 of GFF2.5 representation of feature
+ Returns : a string
+ Args    : none
+ Status  : Public
+
+=cut
+
 sub gff_string {
   my $self = shift;
+  my $version = $self->version;
+
+  # gff3_string and gff_string are synonymous if the version is set to 3
+  return $self->gff3_string(@_) if $version == 3;
+
   my ($start,$stop) = ($self->start,$self->stop);
 
   # the defined() tests prevent uninitialized variable warnings, when dealing with clone objects
@@ -1014,31 +1105,123 @@ sub gff_string {
   ($start,$stop) = ($stop,$start) if defined($start) && defined($stop) && $start > $stop;
 
   my ($class,$name) = ('','');
-  my @group;
-  if (my $t = $self->target) {
-    my $class = $t->class;
-    my $name  = $t->name;
-    my $start = $t->start;
-    my $stop  = $t->stop;
-    push @group,qq(Target "$class:$name" $start $stop);
-  }
+  my $strand = ('-','.','+')[$self->strand+1];
 
+  my @group;
+
+  if (my $t = $self->target) {
+    push @group, $version == 2.5 ? $self->flatten_target($t,2.5) 
+                                 : $self->flatten_target($t);
+  }
   elsif (my $g = $self->group) {
     $class = $g->class || '';
     $name  = $g->name  || '';
-    push @group,"$class $name";
+    ($name =~ /\S\s\S/)?(push @group, "$class '$name'"):(push @group,"$class $name");
+  }
+
+  # add exhaustive list of attributes
+  my $att = $self->attributes;
+  for ( keys %$att ) {
+      for my $v ( @{$att->{$_}} ) {     
+	  $v = qq("$v") if $v=~ /\S\s+\S/;
+	  push @group, qq($_ $v);
+      }
   }
-  push @group,map {qq(Note "$_")} $self->notes;
 
   my $group_field = join ' ; ',@group;
+  my $ref = $self->refseq;
+  my $n   = ref($ref) ? $ref->name : $ref;
+  my $phase = $self->phase;
+  $phase = '.' unless defined $phase;
+  return join("\t",$n,$self->source,$self->method,$start||'.',$stop||'.',
+                   $self->score||'.',$strand||'.',$phase,$group_field);
+}
+
+=head2 gff3_string
+
+ Title   : gff3_string
+ Usage   : $string = $feature->gff3_string([$recurse])
+ Function: return GFF3 representation of feature
+ Returns : a string
+ Args    : An optional flag, which if true, will cause the feature to recurse over
+           subfeatures.
+ Status  : Public
+
+=cut
+
+sub gff3_string {
+  my $self = shift;
+  my ($recurse,$parent) = @_;
+  my ($start,$stop) = ($self->start,$self->stop);
+
+  # the defined() tests prevent uninitialized variable warnings, when dealing with clone objects
+  # whose endpoints may be undefined
+  ($start,$stop) = ($stop,$start) if defined($start) && defined($stop) && $start > $stop;
+
   my $strand = ('-','.','+')[$self->strand+1];
   my $ref = $self->refseq;
   my $n   = ref($ref) ? $ref->name : $ref;
   my $phase = $self->phase;
   $phase = '.' unless defined $phase;
-  return join("\t",$n,$self->source,$self->method,$start||'.',$stop||'.',$self->score||'.',$strand||'.',$phase,$group_field);
+
+  my ($class,$name) = ('','');
+  my @group;
+  if (my $t = $self->target) {
+    $strand = '-' if $t->stop < $t->start;
+    push @group, $self->flatten_target($t,3);
+  }
+
+  elsif (my $g = $self->group) {
+    $class = $g->class || '';
+    $name  = $g->name  || '';
+    $name  = "$class:$name" if defined $class;
+    push @group,[ID =>  $name] if !defined($parent) || $name ne $parent;
+  }
+
+  push @group,[Parent => $parent] if defined $parent && $parent ne '';
+
+  my @attributes = $self->attributes;
+  while (@attributes) {
+    push @group,[shift(@attributes),shift(@attributes)]
+  }
+  my $group_field = join ';',map {join '=',_escape($_->[0]),_escape($_->[1])} @group;
+  my $string = join("\t",$n,$self->source,$self->method,$start||'.',$stop||'.',
+                         $self->score||'.',$strand||'.',$phase,$group_field);
+  $string .= "\n";
+  if ($recurse) {
+    foreach ($self->sub_SeqFeature) {
+      $string .= $_->gff3_string(1,$name);
+    }
+  }
+  $string;
 }
 
+=head2 version
+
+ Title   : version
+ Usage   : $feature->version()
+ Function: get/set the GFF version to be returned by gff_string
+ Returns : the GFF version (default is 2)
+ Args    : the GFF version (2, 2.5 of 3)
+ Status  : Public
+
+=cut
+
+sub version {
+  my ($self, $version) = @_;
+  $self->{version} = $version if $version;
+  return $self->{version} || 2;
+}
+
+
+sub _escape {
+  my $toencode = shift;
+  $toencode    =~ s/([^a-zA-Z0-9_. :?^*\(\)\[\]@!-])/uc sprintf("%%%02x",ord($1))/eg;
+  $toencode    =~ tr/ /+/;
+  $toencode;
+}
+
+
 =head1 A Note About Similarities
 
 The current default aggregator for GFF "similarity" features creates a
