@@ -28,9 +28,8 @@ use Bio::RangeI;
 use Bio::Das::SegmentI;
 use Bio::SeqI;
 
-use vars qw($VERSION @ISA);
+use vars qw(@ISA);
 @ISA = qw(Bio::Root::Root Bio::RangeI Bio::SeqI Bio::Das::SegmentI);
-$VERSION = '0.31';
 
 use overload 
   '""'     => 'asString',
@@ -103,6 +102,7 @@ to create the segment.
 sub factory { shift->{factory} }
 
 # start, stop, length
+
 =head2 start
 
  Title   : start
@@ -319,6 +319,41 @@ sub seq {
 
 *protein = *dna = \&seq;
 
+
+=head2 primary_seq
+
+ Title   : primary_seq
+ Usage   : $s->primary_seq
+ Function: returns a Bio::PrimarySeqI compatible object
+ Returns : a Bio::PrimarySeqI object
+ Args    : none
+ Status  : Public
+
+This is for compatibility with BioPerl's separation of SeqI
+from PrimarySeqI.  It just returns itself.
+
+=cut
+
+#'
+
+sub primary_seq { shift }
+
+=head2 type
+
+ Title   : type
+ Usage   : $s->type
+ Function: return the string "feature"
+ Returns : the string "feature"
+ Args    : none
+ Status  : Public
+
+This is for future sequence ontology-compatibility and
+represents the default type of a feature on the genome
+
+=cut
+
+sub type { "feature" }
+
 =head2 equals
 
  Title   : equals
@@ -527,6 +562,7 @@ An alias for refseq() but only allows reading.
 =cut
 
 sub seq_id { shift->refseq }
+*seqname = \&seq_id;
 
 =head2 truncated
 
@@ -648,10 +684,10 @@ sub primary_id {
 }
 
 
-=head2 display_id
+=head2 display_name
 
- Title   : display_id
- Usage   : $id = $obj->display_id or $obj->display_id($newid);
+ Title   : display_name
+ Usage   : $id = $obj->display_name or $obj->display_name($newid);
  Function: Gets or sets the display id, also known as the common name of
            the Seq object.
 
@@ -671,10 +707,13 @@ sub primary_id {
  Returns : A string
  Args    : None or a new id
 
+Note, this used to be called display_id(), and this name is preserved for
+backward compatibility.  The default is to return the seq_id().
 
 =cut
 
-sub display_id { shift->seq_id }
+sub display_name { shift->seq_id }
+*display_id = \&display_name;
 
 =head2 accession_number
 
@@ -736,6 +775,8 @@ sub alphabet{
 
 sub desc { shift->asString }
 
+*description = \&desc;
+
 =head2 species
 
  Title   : species
@@ -781,6 +822,20 @@ sub annotation {
 
 }
 
+=head2 is_circular
+
+ Title   : is_circular
+ Usage   : if( $obj->is_circular) { /Do Something/ }
+ Function: Returns true if the molecule is circular
+ Returns : Boolean value
+ Args    : none
+
+=cut
+
+sub is_circular{
+    return 0;
+}
+
 
 1;
 __END__
