@@ -1,4 +1,4 @@
-# $Id: NCBIHelper.pm,v 1.24 2002/12/09 15:39:44 jason Exp $
+# $Id: NCBIHelper.pm,v 1.35 2003/12/08 16:06:34 bosborne Exp $
 #
 # BioPerl module for Bio::DB::NCBIHelper
 #
@@ -19,11 +19,12 @@ NCBI databases.
 
 =head1 SYNOPSIS
 
- Do not use this module directly.
+ #Do not use this module directly.
+
  # get a Bio::DB::NCBIHelper object somehow
  my $seqio = $db->get_Stream_by_acc(['MUSIGHBA1']);
  foreach my $seq ( $seqio->next_seq ) {
-  # process seq
+     # process seq
  }
 
 =head1 DESCRIPTION
@@ -33,9 +34,8 @@ web databases.  This module just centralizes the methods for
 constructing a URL for querying NCBI GenBank and NCBI GenPept and the
 common HTML stripping done in L<postprocess_data>().
 
-The NCBI query URLs used are http://www.ncbi.nlm.nih.gov as the base URL,
-/cgi-bin/Entrez/qserver.cgi as the query interface for batch mode, and 
-/entrez/utils/qmap.cgi for single-query mode.
+The base NCBI query URL used is 
+http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi.
 
 =head1 FEEDBACK
 
@@ -77,7 +77,7 @@ preceded with a _
 package Bio::DB::NCBIHelper;
 use strict;
 use vars qw(@ISA $HOSTBASE %CGILOCATION %FORMATMAP 
-	    $DEFAULTFORMAT $MAX_ENTRIES $VERSION);
+	    $DEFAULTFORMAT $MAX_ENTRIES);
 
 use Bio::DB::WebDBSeqI;
 use Bio::DB::Query::GenBank;
@@ -88,11 +88,10 @@ use Bio::DB::RefSeq;
 use Bio::Root::Root;
 
 @ISA = qw(Bio::DB::WebDBSeqI Bio::Root::Root);
-$VERSION = '0.8';
 
 BEGIN {
     $MAX_ENTRIES = 19000;
-    $HOSTBASE = 'http://www.ncbi.nih.gov';
+    $HOSTBASE = 'http://eutils.ncbi.nlm.nih.gov';
     %CGILOCATION = (
 		    'batch'  => ['post' => '/entrez/eutils/efetch.fcgi'],
 		    'query'  => ['get'  => '/entrez/eutils/efetch.fcgi'],
@@ -160,11 +159,12 @@ sub default_format {
 
 sub get_request {
     my ($self, @qualifiers) = @_;
-    my ($mode, $uids, $format, $query) = $self->_rearrange([qw(MODE UIDS FORMAT QUERY)],
+    my ($mode, $uids, $format, $query) = $self->_rearrange([qw(MODE UIDS 
+							       FORMAT QUERY)],
 							   @qualifiers);
 
     $mode = lc $mode;
-    ($format) = $self->request_format() if( !defined $format);
+    ($format) = $self->request_format() unless ( defined $format);
     if( !defined $mode || $mode eq '' ) { $mode = 'single'; }
     my %params = $self->get_params($mode);
     if( ! %params ) {
@@ -311,8 +311,8 @@ sub postprocess_data {
 	my $ct = 0;
 	while( my $seq = $stream->next_seq() ) {	    
 	    if( $seq->accession_number !~ /$unique_accessions[$ct]/ ) {
-		printf STDERR "warning, %s does not match %s\n",
-		$seq->accession_number, $unique_accessions[$ct];
+		$self->warn( sprintf("warning, %s does not match %s\n",
+		$seq->accession_number, $unique_accessions[$ct]));
 	    }
 	    $accessions{$unique_accessions[$ct]}->{'seq'} = $seq;
 	    $ct++;
@@ -424,6 +424,7 @@ sub get_Stream_by_acc {
   Function: 
   Returns : A Bio::DB::RefSeq reference or throws
   Args    : $id(s), $string
+
 =cut
 
 sub _check_id {
@@ -437,10 +438,10 @@ sub _check_id {
     # Asking for a RefSeq from EMBL/GenBank
 
     if ($ids =~ /N._/) {
-	$self->warn("[$ids] is not a normal sequence database but a RefSeq entry.".
+	$self->warn("[$ids] is not a normal sequence entry but a RefSeq entry.".
 		   " Redirecting the request.\n")
 	    if $self->verbose >= 0;
-	return  new Bio::DB::RefSeq;
+	return $self->refseq_db;
     }
 }
 
@@ -462,5 +463,29 @@ sub delay_policy {
   return 3;
 }
 
+=head2 refseq_db
+
+ Title   : refseq_db
+ Usage   : $obj->refseq_db($newval)
+ Function: 
+ Example : 
+ Returns : value of refseq_db (a scalar)
+ Args    : on set, new value (a scalar or undef, optional)
+
+
+=cut
+
+sub refseq_db{
+    my $self = shift;
+    if( @_ ) {
+	return $self->{'refseq_db'} = shift;
+    } elsif( ! defined $self->{'refseq_db'} ) {
+	$self->{'refseq_db'} = Bio::DB::RefSeq->new
+	    (-retrievaltype => $self->retrieval_type,
+	     -verbose        => $self->verbose);
+    }
+    return $self->{'refseq_db'};
+}
+
 1;
 __END__
