@@ -1,4 +1,4 @@
-# $Id: FTLocationFactory.pm,v 1.9 2002/10/22 07:45:14 lapp Exp $
+# $Id: FTLocationFactory.pm,v 1.12 2003/09/14 19:19:23 jason Exp $
 #
 # BioPerl module for Bio::Factory::FTLocationFactory
 #
@@ -26,7 +26,7 @@
 
 =head1 NAME
 
-Bio::Factory::FTLocationFactory - DESCRIPTION of Object
+Bio::Factory::FTLocationFactory - A FeatureTable Location Parser
 
 =head1 SYNOPSIS
 
@@ -103,14 +103,6 @@ use Bio::Location::Fuzzy;
 
 =cut
 
-sub new {
-  my($class,@args) = @_;
-
-  my $self = $class->SUPER::new(@args);
-
-  return $self;
-}
-
 =head2 from_string
 
  Title   : from_string
@@ -146,13 +138,14 @@ sub from_string{
 	    # parse the argument recursively, then set the strand to -1
 	    $loc = $self->from_string($oparg, 1);
 	    $loc->strand(-1);
-	} elsif(($op eq "join") || ($op eq "order")) {
+	} elsif(($op eq "join") || ($op eq "order") || ($op eq "bond")) {
 	    # This is a split location. Split into components and parse each
 	    # one recursively, then gather into a SplitLocationI instance.
 	    #
 	    # Note: The following code will /not/ work with nested
 	    # joins (you want to have grammar-based parsing for that).
-	    $loc = Bio::Location::Split->new(-splittype => $op);
+	    $loc = Bio::Location::Split->new(-verbose   => $self->verbose,
+					     -splittype => $op);
 	    foreach my $substr (split(/,/, $oparg)) {
 		$loc->add_sub_Location($self->from_string($substr, 1));
 	    }
@@ -212,7 +205,7 @@ sub _parse_location {
 	    $end = $3;
 	    $loctype = $2;
 	    $locclass = "Bio::Location::Fuzzy"
-		unless (($end-1) == $start) && ($loctype eq "^");
+		unless (abs($end-$start) <= 1) && ($loctype eq "^");
 	} else {
 	    $end = $start;
 	}
@@ -222,8 +215,9 @@ sub _parse_location {
     } 
 
     # instantiate location and initialize
-    $loc = $locclass->new(-start => $start, -end  => $end, -strand => 1,
-			  -location_type => $loctype);
+    $loc = $locclass->new(-verbose => $self->verbose,
+			  -start => $start, -end  => $end, 
+			  -strand => 1, -location_type => $loctype);
     # set remote ID if remote location
     if($seqid) {
 	$loc->is_remote(1);
