@@ -1,6 +1,6 @@
 package Bio::Graphics::FeatureFile;
 
-# $Id: FeatureFile.pm,v 1.20 2002/12/14 00:37:17 lstein Exp $
+# $Id: FeatureFile.pm,v 1.41 2003/12/13 17:17:50 lstein Exp $
 # This package parses and renders a simple tab-delimited format for features.
 # It is simpler than GFF, but still has a lot of expressive power.
 # See __END__ for the file format
@@ -39,7 +39,9 @@ Bio::Graphics::FeatureFile -- A set of Bio::Graphics features, stored in a file
  $data->add_type(EST => {fgcolor=>'blue',height=>12});
 
  # add a feature
- my $feature = Bio::Graphics::Feature->new(....); # or some other SeqI
+ my $feature = Bio::Graphics::Feature->new(
+                                             # params
+                                          ); # or some other SeqI
  $data->add_feature($feature=>'EST');
 
 =head1 DESCRIPTION
@@ -114,12 +116,11 @@ belong to the same group named yk53c10.
 
 use strict;
 use Bio::Graphics::Feature;
-use Bio::Graphics::Panel;
+use Bio::DB::GFF::Util::Rearrange;
 use Carp;
+use Bio::DB::GFF;
 use IO::File;
 use Text::Shellwords;
-use vars '$VERSION';
-$VERSION = '1.04';
 
 # default colors for unconfigured features
 my @COLORS = qw(cyan blue red yellow green wheat turquoise orange);
@@ -151,7 +152,7 @@ object.  Arguments are -name=E<gt>value pairs:
 
    -safe           Indicates that the contents of this file is trusted.
                    Any option value that begins with the string "sub {"
-                   will be evaluated as a code reference.
+                   or \&subname will be evaluated as a code reference.
 
 The -file and -text arguments are mutually exclusive, and -file will
 supersede the other if both are present.
@@ -168,6 +169,11 @@ The -smart_features flag is used by the generic genome browser to
 provide features with a way to access the link-generation code.  See
 gbrowse for how this works.
 
+If the file is trusted, and there is an option named "init_code" in
+the [GENERAL] section of the file, it will be evaluated as perl code
+immediately after parsing.  You can use this to declare global
+variables and subroutines for use in option values.
+
 =back
 
 =cut
@@ -185,7 +191,6 @@ sub new {
   my $self = bless {
 		    config   => {},
 		    features => {},
-		    groups   => {},
 		    seenit   => {},
 		    types    => [],
 		    max      => undef,
@@ -239,6 +244,8 @@ for you.
 
 =cut
 
+#"
+
 sub render {
   my $self = shift;
   my $panel = shift;
@@ -251,7 +258,7 @@ sub render {
   my $color;
   my %types = map {$_=>1} $self->configured_types;
 
-  my @configured_types   = grep {exists $self->features->{$_}} $self->configured_types;
+  my @configured_types   = grep {exists $self->{features}{$_}} $self->configured_types;
   my @unconfigured_types = sort grep {!exists $types{$_}}      $self->types;
 
   my @base_config = $self->style('general');
@@ -360,7 +367,7 @@ sub parse_file {
   $self->init_parse;
   while (<$fh>) {
     chomp;
-    $self->parse_line($_);
+    $self->parse_line($_) || last;
   }
   $self->finish_parse;
 }
@@ -370,7 +377,7 @@ sub parse_text {
   my $text = shift;
 
   $self->init_parse;
-  foreach (split /\r?\n|\r\n?/,$text) {
+  foreach (split /\015?\012|\015\012?/,$text) {
     $self->parse_line($_);
   }
   $self->finish_parse;
@@ -380,25 +387,37 @@ sub parse_line {
   my $self = shift;
   local $_ = shift;
 
-  s/\r//g;  # get rid of carriage returns left over by MS-DOS/Windows systems
+  s/\015//g;  # get rid of carriage returns left over by MS-DOS/Windows systems
 
-  return if /^\s*[\#]/;
+  # capture GFF header
+  if (/^\#\#gff-version\s+(\d+)/) {
+    $self->{gff_version} = $1;
+    require Bio::DB::GFF;
+    return 1;
+  }
+
+  # skip on blank lines and comments
+  return 1 if /^\s*[\#]/;
+
+  # abort if we see a >FASTA line
+  return 0 if /^>/;
 
   if (/^\s+(.+)/ && $self->{current_tag}) { # continuation line
-      my $value = $1;
-      my $cc = $self->{current_config} ||= 'general';       # in case no configuration named
-      $self->{config}{$cc}{$self->{current_tag}} .= ' ' . $value;
-      # respect newlines in code subs
-      $self->{config}{$cc}{$self->{current_tag}} .= "\n" if $self->{config}{$cc}{$self->{current_tag}}=~ /^sub\s*{/;
-      return;
+    my $value = $1;
+    my $cc = $self->{current_config} ||= 'general';       # in case no configuration named
+    $self->{config}{$cc}{$self->{current_tag}} .= ' ' . $value;
+    # respect newlines in code subs
+    $self->{config}{$cc}{$self->{current_tag}} .= "\n"
+      if $self->{config}{$cc}{$self->{current_tag}}=~ /^sub\s*\{/;
+    return 1;
   }
 
   if (/^\s*\[([^\]]+)\]/) {  # beginning of a configuration section
-     my $label = $1;
-     my $cc = $label =~ /^(general|default)$/i ? 'general' : $label;  # normalize
-     push @{$self->{types}},$cc unless $cc eq 'general';
-     $self->{current_config} = $cc;
-     return;
+    my $label = $1;
+    my $cc = $label =~ /^(general|default)$/i ? 'general' : $label;  # normalize
+    push @{$self->{types}},$cc unless $cc eq 'general';
+    $self->{current_config} = $cc;
+    return 1;
   }
 
   if (/^([\w: -]+?)\s*=\s*(.*)/) {   # key value pair within a configuration section
@@ -407,13 +426,13 @@ sub parse_line {
     my $value = defined $2 ? $2 : '';
     $self->{config}{$cc}{$tag} = $value;
     $self->{current_tag} = $tag;
-    return;
+    return 1;
   }
 
 
   if (/^$/) { # empty line
     undef $self->{current_tag};
-    return;
+    return 1;
   }
 
   # parse data lines
@@ -421,31 +440,44 @@ sub parse_line {
   unshift @tokens,'' if /^\s+/;
 
   # close any open group
-  undef $self->{grouptype} if length $tokens[0] > 0;
-
-  if (@tokens < 3) {      # short line; assume a group identifier
-    $self->{grouptype}     = shift @tokens;
-    $self->{groupname}     = shift @tokens;
-    return;
+  if (length $tokens[0] > 0 && $self->{group}) {
+    push @{$self->{features}{$self->{grouptype}}},$self->{group};
+    undef $self->{group};
+    undef $self->{grouptype};
   }
 
-  my($ref,$type,$name,$strand,$bounds,$description,$url);
+  if (@tokens < 3) {      # short line; assume a group identifier
+    my $type               = shift @tokens;
+    my $name               = shift @tokens;
+    $self->{group}         = Bio::Graphics::Feature->new(-name => $name,
+							 -type => 'group');
+    $self->{grouptype}     = $type;
+    return 1;
+  }
+
+  my($ref,$type,$name,$strand,$bounds,$description,$url,$score,%attributes);
 
   if (@tokens >= 8) { # conventional GFF file
-    my ($r,$source,$method,$start,$stop,$score,$s,$phase,@rest) = @tokens;
+    my ($r,$source,$method,$start,$stop,$scor,$s,$phase,@rest) = @tokens;
     my $group = join ' ',@rest;
-    $type   = join(':',$method,$source);
+    $type   = defined $source && $source ne '.' ? join(':',$method,$source) : $method;
     $bounds = join '..',$start,$stop;
     $strand = $s;
     if ($group) {
       my ($notes,@notes);
-      (undef,$self->{groupname},undef,undef,$notes) = split_group($group);
+      (undef,$name,undef,undef,$notes) = $self->split_group($group);
       foreach (@$notes) {
-	if (m!^(http|ftp)://!) { $url = $_ } else { push @notes,$_ }
+	my ($key,$value) = @$_;
+	if ($value =~ m!^(http|ftp)://!) { 
+	  $url = $_ 
+	} elsif ($key=~/note/i) { 
+	  push @notes,$value;
+	}
       }
       $description = join '; ',@notes if @notes;
+      $score       = $scor if defined $scor && $scor ne '.';
     }
-    $name ||= $self->{groupname};
+    $name ||= $self->{group}->display_id if $self->{group};
     $ref = $r;
   }
 
@@ -477,34 +509,85 @@ sub parse_line {
 
   if ($self->{coordinate_mapper} && $ref) {
     ($ref,@parts) = $self->{coordinate_mapper}->($ref,@parts);
-    return unless $ref;
+    return 1 unless $ref;
   }
 
   $type = '' unless defined $type;
   $name = '' unless defined $name;
 
+  # attribute handling
+  if (defined $description && $description =~ /\w+=\w+/) { # attribute line
+    my @attributes = split /;\s*/,$description;
+    foreach (@attributes) {
+      my ($name,$value) = split /=/,$_,2;
+      Bio::Root::Root->throw(qq("$_" is not a valid attribute=value pair)) unless defined $value;
+      _unescape($name);
+      my @values = split /,/,$value;
+      _unescape(@values);
+      if ($name =~ /^(note|description)/) {
+	$description = "@values";
+      } elsif ($name eq 'url') {
+	$url = $value;
+      } elsif ($name eq 'score') {
+	$score = $value;
+      } else {
+	push @{$attributes{$name}},@values;
+      }
+    }
+  }
+
   # either create a new feature or add a segment to it
   if (my $feature = $self->{seenit}{$type,$name}) {
-    $feature->add_segment(@parts);
-  } else {
-    $feature = $self->{seenit}{$type,$name} = 
-      Bio::Graphics::Feature->new(-name     => $name,
-				  -type     => $type,
-				  $strand ? (-strand   => make_strand($strand)) : (),
-				  -segments => \@parts,
-				  -desc     => $description,
-				  -ref      => $ref,
-				  defined($url) ? (-url      => $url) : (),
-				 );
+
+    # create a new segment to hold the parts
+    if (!$feature->segments) {
+      my $new_segment  = bless {%$feature},ref $feature;
+      $feature->add_segment($new_segment);
+    }
+    # add the segments
+    $feature->add_segment(map {
+      _make_feature($name,$type,$strand,$description,$ref,\%attributes,$url,$score,[$_])
+    }  @parts);
+  }
+
+  else {
+    $feature = $self->{seenit}{$type,$name} = _make_feature($name,$type,$strand,
+							    $description,$ref,
+							    \%attributes,$url,$score,\@parts);
     $feature->configurator($self) if $self->smart_features;
-    if ($self->{grouptype}) {
-      push @{$self->{groups}{$self->{grouptype}}{$self->{groupname}}},$feature;
+    if ($self->{group}) {
+      $self->{group}->add_segment($feature);
     } else {
       push @{$self->{features}{$type}},$feature;  # for speed; should use add_feature() instead
     }
   }
+
+  return 1;
 }
 
+sub _unescape {
+  foreach (@_) {
+    tr/+/ /;       # pluses become spaces
+    s/%([0-9a-fA-F]{2})/chr hex($1)/g;
+  }
+  @_;
+}
+
+sub _make_feature {
+  my ($name,$type,$strand,$description,$ref,$attributes,$url,$score,$parts) = @_;
+  my @coordinates = @$parts > 1 ? (-segments => $parts) : (-start=>$parts->[0][0],-end=>$parts->[0][1]);
+  Bio::Graphics::Feature->new(-name       => $name,
+			      -type       => $type,
+			      -subtype    => "${type}_part",
+			      $strand ? (-strand   => make_strand($strand)) : (),
+			      -desc       => $description,
+			      -ref        => $ref,
+			      -attributes => $attributes,
+			      defined $url   ? (-url  => $url) : (),
+			      defined $score ? (-score=>$score) : (),
+			      @coordinates,
+			     );
+}
 
 =over 4
 
@@ -590,12 +673,12 @@ sub set {
 }
 
 # break circular references
-sub destroy {
+sub finished {
   my $self = shift;
   delete $self->{features};
 }
 
-sub DESTROY { shift->destroy(@_) }
+sub DESTROY { shift->finished(@_) }
 
 =over 4
 
@@ -662,11 +745,23 @@ sub code_setting {
   my $setting = $self->_setting($section=>$option);
   return unless defined $setting;
   return $setting if ref($setting) eq 'CODE';
-  return $setting unless $setting =~ /^sub\s*\{/;
-  my $coderef = eval $setting;
-  warn $@ if $@;
-  $self->set($section,$option,$coderef);
-  return $coderef;
+  if ($setting =~ /^\\&(\w+)/) {  # coderef in string form
+    my $subroutine_name = $1;
+    my $package         = $self->base2package;
+    my $codestring      = "\\&${package}\:\:${subroutine_name}";
+    my $coderef         = eval $codestring;
+    warn $@ if $@;
+    $self->set($section,$option,$coderef);
+    return $coderef;
+  }
+  elsif ($setting =~ /^sub\s*\{/) {
+    my $coderef   = eval $setting;
+    warn $@ if $@;
+    $self->set($section,$option,$coderef);
+    return $coderef;
+  } else {
+    return $setting;
+  }
 }
 
 =over 4
@@ -710,7 +805,11 @@ sub style {
   my $type = shift;
 
   my $config  = $self->{config}  or return;
-  my $hashref = $config->{$type} or return;
+  my $hashref = $config->{$type};
+  unless ($hashref) {
+    $type =~ s/:.+$//;
+    $hashref = $config->{$type} or return;
+  }
 
   return map {("-$_" => $hashref->{$_})} keys %$hashref;
 }
@@ -774,6 +873,55 @@ sub types {
   return keys %{$features};
 }
 
+=over 4
+
+=item $features = $features-E<gt>features($type)
+
+Return a list of all the feature types of type "$type".  If the
+featurefile object was created by parsing a file or text scalar, then
+the features will be of type Bio::Graphics::Feature (which follow the
+Bio::FeatureI interface).  Otherwise the list will contain objects of
+whatever type you added with calls to add_feature().
+
+Two APIs:
+
+  1) original API:
+
+      # Reference to an array of all features of type "$type"
+      $features = $features-E<gt>features($type)
+
+      # Reference to an array of all features of all types
+      $features = $features-E<gt>features()
+
+      # A list when called in a list context
+      @features = $features-E<gt>features()
+
+   2) Bio::Das::SegmentI API:
+
+       @features = $features-E<gt>features(-type=>['list','of','types']);
+
+       # variants
+       $features = $features-E<gt>features(-type=>['list','of','types']);
+       $features = $features-E<gt>features(-type=>'a type');
+       $iterator = $features-E<gt>features(-type=>'a type',-iterator=>1);
+
+=back
+
+=cut
+
+# return features
+sub features {
+  my $self = shift;
+  my ($types,$iterator,@rest) = defined($_[0] && $_[0]=~/^-/) ? rearrange([['TYPE','TYPES']],@_) : (\@_);
+  $types = [$types] if $types && !ref($types);
+  my @types = ($types && @$types) ? @$types : $self->types;
+  my @features = map {@{$self->{features}{$_}}} @types;
+  if ($iterator) {
+    require Bio::Graphics::FeatureFile::Iterator;
+    return Bio::Graphics::FeatureFile::Iterator->new(\@features);
+  }
+  return wantarray ? @features : \@features;
+}
 
 =over 4
 
@@ -789,15 +937,6 @@ whatever type you added with calls to add_feature().
 
 =cut
 
-# return features
-sub features {
-  my $self = shift;
-  return $self->{features}{shift()} if @_;
-  return $self->{features};
-}
-
-
-
 sub make_strand {
   local $^W = 0;
   return +1 if $_[0] =~ /^\+/ || $_[0] > 0;
@@ -805,6 +944,43 @@ sub make_strand {
   return 0;
 }
 
+=head2 get_seq_stream
+
+ Title   : get_seq_stream
+ Usage   : $stream = $s->get_seq_stream(@args)
+ Function: get a stream of features that overlap this segment
+ Returns : a Bio::SeqIO::Stream-compliant stream
+ Args    : see below
+ Status  : Public
+
+This is the same as feature_stream(), and is provided for Bioperl
+compatibility.  Use like this:
+
+ $stream = $s->get_seq_stream('exon');
+ while (my $exon = $stream->next_seq) {
+    print $exon->start,"\n";
+ }
+
+=cut
+
+sub get_seq_stream {
+  my $self = shift;
+  local $^W = 0;
+  my @args = $_[0] =~ /^-/ ? (@_,-iterator=>1) : (-types=>\@_,-iterator=>1);
+  $self->features(@args);
+}
+
+=head2 get_feature_stream(), top_SeqFeatures(), all_SeqFeatures()
+
+Provided for compatibility with older BioPerl and/or Bio::DB::GFF
+APIs.
+
+=cut
+
+*get_feature_stream = \&get_seq_stream;
+*top_SeqFeatures    = *all_SeqFeatures = \&features;
+
+
 =over 4
 
 =item @refs = $features-E<gt>refs
@@ -849,33 +1025,23 @@ sub init_parse {
   my $s = shift;
 
   $s->{seenit} = {}; 
-  $s->{max}      = $s->{min} = undef;
-  $s->{types}    = [];
-  $s->{groups}   = {};
-  $s->{features} = {};
-  $s->{config}   = {}
+  $s->{max}         = $s->{min} = undef;
+  $s->{types}       = [];
+  $s->{features}    = {};
+  $s->{config}      = {};
+  $s->{gff_version} = 0;
 }
 
 sub finish_parse {
   my $s = shift;
-  $s->consolidate_groups;
   $s->evaluate_coderefs if $s->safe;
   $s->{seenit} = {};
-  $s->{groups} = {};
-}
-
-sub consolidate_groups {
-  my $self = shift;
-  my $groups = $self->{groups} or return;
-
-  for my $type (keys %$groups) {
-    my @groups = values %{$groups->{$type}};
-    push @{$self->{features}{$type}},@groups;
-  }
+  delete $s->{gff_version};
 }
 
 sub evaluate_coderefs {
   my $self = shift;
+  $self->initialize_code();
   for my $s ($self->_setting) {
     for my $o ($self->_setting($s)) {
       $self->code_setting($s,$o);
@@ -883,55 +1049,33 @@ sub evaluate_coderefs {
   }
 }
 
+sub initialize_code {
+  my $self       = shift;
+  my $package = $self->base2package;
+  my $init_code = $self->_setting(general => 'init_code') or return;
+  my $code = "package $package; $init_code; 1;";
+  eval $code;
+  warn $@ if $@;
+}
+
+sub base2package {
+  my $self = shift;
+  (my $package = overload::StrVal($self)) =~ s/[^a-z0-9A-Z_]/_/g;
+  $package     =~ s/^[^a-zA-Z_]/_/g;
+  $package;
+}
+
 sub split_group {
-  my $group = shift;
-
-  $group =~ s/\\;/$;/g;  # protect embedded semicolons in the group
-  $group =~ s/( \"[^\"]*);([^\"]*\")/$1$;$2/g;
-  my @groups = split(/\s*;\s*/,$group);
-  foreach (@groups) { s/$;/;/g }
-
-  my ($gclass,$gname,$tstart,$tstop,@notes);
-
-  foreach (@groups) {
-
-    my ($tag,$value) = /^(\S+)\s*(.*)/;
-    $value =~ s/\\t/\t/g;
-    $value =~ s/\\r/\r/g;
-    $value =~ s/^"//;
-    $value =~ s/"$//;
-
-    # if the tag is "Note", then we add this to the
-    # notes array
-   if ($tag eq 'Note') {  # just a note, not a group!
-     push @notes,$value;
-   }
-
-    # if the tag eq 'Target' then the class name is embedded in the ID
-    # (the GFF format is obviously screwed up here)
-    elsif ($tag eq 'Target' && $value =~ /([^:\"]+):([^\"]+)/) {
-      ($gclass,$gname) = ($1,$2);
-      ($tstart,$tstop) = /(\d+) (\d+)/;
-    }
-
-    elsif (!$value) {
-      push @notes,$tag;  # e.g. "Confirmed_by_EST"
-    }
-
-    # otherwise, the tag and value correspond to the
-    # group class and name
-    else {
-      ($gclass,$gname) = ($tag,$value);
-    }
-  }
-
-  return ($gclass,$gname,$tstart,$tstop,\@notes);
+  my $self = shift;
+  return Bio::DB::GFF->split_group(shift, $self->{gff_version} > 2);
 }
 
 # create a panel if needed
 sub new_panel {
   my $self = shift;
 
+  require Bio::Graphics::Panel;
+
   # general configuration of the image here
   my $width         = $self->setting(general => 'pixels')
                       || $self->setting(general => 'width')
@@ -1001,8 +1145,10 @@ sub feature2label {
   my $feature = shift;
   my $type  = eval {$feature->type} || $feature->primary_tag or return;
   (my $basetype = $type) =~ s/:.+$//;
-  my $label = $self->type2label($type) || $self->type2label($basetype) || $type;
-  $label;
+  my @labels = $self->type2label($type);
+  @labels = $self->type2label($basetype) unless @labels;
+  @labels = ($type) unless @labels;;
+  wantarray ? @labels : $labels[0];
 }
 
 =over 4
@@ -1020,38 +1166,48 @@ convenience for the generic genome browser.
 sub make_link {
   my $self     = shift;
   my $feature  = shift;
-  my $label    = $self->feature2label($feature) or return;
-  my $link     = $self->setting($label,'link');
-  $link        = $self->setting(general=>'link') unless defined $link;
-  return unless $link;
-  return $self->link_pattern($link,$feature);
+  for my $label ($self->feature2label($feature)) {
+    my $link     = $self->setting($label,'link');
+    $link        = $self->setting(general=>'link') unless defined $link;
+    next unless $link;
+    return $self->link_pattern($link,$feature);
+  }
+  return;
 }
 
 sub link_pattern {
   my $self = shift;
   my ($pattern,$feature,$panel) = @_;
+  require CGI unless defined &CGI::escape;
+  my $n;
   $pattern =~ s/\$(\w+)/
-    $1 eq 'ref'           ? $feature->location->seq_id
-      : $1 eq 'name'      ? $feature->display_name
-      : $1 eq 'class'     ? $feature->class
-      : $1 eq 'type'      ? $feature->method
-      : $1 eq 'method'    ? $feature->method
-      : $1 eq 'source'    ? $feature->source
-      : $1 eq 'start'     ? $feature->start
-      : $1 eq 'end'       ? $feature->end
-      : $1 eq 'segstart'  ? $panel->start
-      : $1 eq 'segend'    ? $panel->end
+    CGI::escape(
+    $1 eq 'ref'              ? ($n = $feature->location->seq_id) && "$n"
+      : $1 eq 'name'         ? ($n = $feature->display_name) && "$n"  # workaround broken CGI.pm
+      : $1 eq 'class'        ? eval {$feature->class}  || ''
+      : $1 eq 'type'         ? eval {$feature->method} || $feature->primary_tag
+      : $1 eq 'method'       ? eval {$feature->method} || $feature->primary_tag
+      : $1 eq 'source'       ? eval {$feature->source} || $feature->source_tag
+      : $1 eq 'start'        ? $feature->start
+      : $1 eq 'end'          ? $feature->end
+      : $1 eq 'stop'         ? $feature->end
+      : $1 eq 'segstart'     ? $panel->start
+      : $1 eq 'segend'       ? $panel->end
+      : $1 eq 'description'  ? eval {join '',$feature->notes} || ''
+      : $1 eq 'id'           ? $feature->feature_id
       : $1
+	       )
        /exg;
   return $pattern;
 }
 
-# given a feature type, return its label
+# given a feature type, return its label(s)
 sub type2label {
   my $self = shift;
   my $type = shift;
   $self->{_type2label} ||= $self->invert_types;
-  $self->{_type2label}{$type};
+  my @labels = keys %{$self->{_type2label}{$type}};
+  wantarray ? @labels : $labels[0]
 }
 
 sub invert_types {
@@ -1061,7 +1217,7 @@ sub invert_types {
   for my $label (keys %{$config}) {
     my $feature = $config->{$label}{feature} or next;
     foreach (shellwords($feature||'')) {
-      $inverted{$_} = $label;
+      $inverted{$_}{$label}++;
     }
   }
   \%inverted;
