@@ -1,8 +1,10 @@
 package Bio::Graphics::Glyph::redgreen_segment;
-#$Id: redgreen_segment.pm,v 1.3.10.1 2006/10/02 23:10:20 sendu Exp $
+#$Id: redgreen_segment.pm,v 1.3 2003/06/04 08:36:40 heikki Exp $
 
 use strict;
-use base qw(Bio::Graphics::Glyph::graded_segments);
+use Bio::Graphics::Glyph::graded_segments;
+use vars '@ISA';
+@ISA = 'Bio::Graphics::Glyph::graded_segments';
 
 sub calculate_color {
   my $self = shift;
