@@ -1,5 +1,5 @@
 package Bio::Graphics::Glyph::transcript;
-# $Id: transcript.pm,v 1.12 2002/10/26 18:07:21 lstein Exp $
+# $Id: transcript.pm,v 1.16 2003/09/17 17:11:30 lstein Exp $
 
 use strict;
 use Bio::Graphics::Glyph::segments;
@@ -45,14 +45,26 @@ sub draw_connectors {
     @parts = $self;
   }
 
-  if ($self->feature->strand >= 0) {
-    my($x1,$y1,$x2,$y2) = $parts[-1]->bounds(@_);
+  # flip argument makes this confusing
+  # certainly there's a simpler way to express this idea
+  my $strand    = $self->feature->strand;
+  my ($first,$last) = ($parts[0],$parts[-1]);
+  ($first,$last) = ($last,$first) if exists $self->{flip};
+
+  if ($strand >= 0) {
+    my($x1,$y1,$x2,$y2) = $last->bounds(@_);
     my $center = ($y2+$y1)/2;
-    $self->arrow($gd,$x2,$x2+$self->arrow_length,$center);
+    $self->{flip} ?
+	$self->arrow($gd,$x1,$x1-$self->arrow_length,$center)
+      :
+	$self->arrow($gd,$x2,$x2+$self->arrow_length,$center);
   } else {
-    my($x1,$y1,$x2,$y2) = $parts[0]->bounds(@_);
+    my($x1,$y1,$x2,$y2) = $first->bounds(@_);
     my $center = ($y2+$y1)/2;
-    $self->arrow($gd,$x1,$x1 - $self->arrow_length,$center);
+    $self->{flip } ?
+	$self->arrow($gd,$x2,$x2+$self->arrow_length,$center)
+      :
+	$self->arrow($gd,$x1,$x1 - $self->arrow_length,$center);
   }
 }
 
@@ -65,7 +77,7 @@ sub arrow_length {
 sub connector {
   my $self = shift;
   return $self->SUPER::connector(@_) if $self->all_callbacks;
-  return 'hat';
+  return ($self->option('connector') || 'hat');
 }
 
 
@@ -119,6 +131,8 @@ L<Bio::Graphics::Glyph> for a full explanation.
 
   -description  Whether to draw a description  0 (false)
 
+  -hilite       Highlight color                undef (no color)
+
 In addition, the alignment glyph recognizes the following
 glyph-specific options:
 
