@@ -6,12 +6,26 @@ use Bio::Graphics::Util qw(frame_and_offset);
 use vars '@ISA';
 @ISA = qw(Bio::Graphics::Glyph::generic);
 
+my %default_colors = qw(
+			frame0f  cadetblue
+			frame1f  blue
+			frame2f  darkblue
+			frame0r  darkred
+			frame1r  red
+			frame2r  crimson
+		       );
+
 # turn off description
 sub description { 0 }
 
 # turn off label
 # sub label { 1 }
 
+sub default_color {
+  my ($self,$key) = @_;
+  return $self->factory->translate_color($default_colors{$key});
+}
+
 sub height {
   my $self = shift;
   my $font = $self->font;
@@ -38,8 +52,16 @@ sub gridcolor {
   $self->factory->translate_color($color);
 }
 
+sub show_sequence {
+  my $self = shift;
+  my $show_sequence = $self->option('show_sequence');
+  return 1 unless defined $show_sequence;  # default to true
+  return $show_sequence;
+}
+
 sub protein_fits {
   my $self = shift;
+  return unless $self->show_sequence;
 
   my $pixels_per_base = $self->pixels_per_residue;
   my $font            = $self->font;
@@ -100,6 +122,7 @@ sub draw_component {
 sub draw_frame {
   my $self = shift;
   my ($feature,$strand,$base_offset,$phase,$gd,$x1,$y1,$x2,$y2) = @_;
+  return unless $feature->seq;  # no sequence, arggh.
   my ($seq,$pos) = $strand < 0 ? ($feature->revcom,$feature->end)
                                : ($feature,$feature->start);
   my ($frame,$offset) = frame_and_offset($pos,$strand,$phase);
@@ -116,8 +139,13 @@ sub draw_frame {
 
   $y2 = $y1;
 
-  my $protein = $seq->translate(undef,undef,$base_offset)->seq;
-  my $color   = $self->color("frame$frame") || $self->fgcolor;
+  my $codon_table = $self->option('codontable') || 1;
+  my $protein = $seq->translate(undef,undef,$base_offset,$codon_table)->seq;
+
+  my $k       = $strand>=0 ? 'f' : 'r';
+  my $color   = $self->color("frame$frame$k") ||
+                $self->color("frame$frame") ||
+                $self->default_color("frame$frame$k") || $self->fgcolor;
   if ($self->protein_fits) {
     $self->draw_protein(\$protein,$strand,$color,$gd,$x1,$y1,$x2,$y2);
   } else {
@@ -267,6 +295,8 @@ L<Bio::Graphics::Glyph> for a full explanation.
 
   -description  Whether to draw a description  0 (false)
 
+  -hilite       Highlight color                undef (no color)
+
 In addition to the common options, the following glyph-specific
 options are recognized:
 
@@ -298,6 +328,11 @@ options are recognized:
   -arrow_height Height of the start codon    1
                 arrowheads
 
+  -show_sequence Show the amino acid sequence 1 (true)
+                if there's room.
+
+  -codontable   Codon table to use           1 (see Bio::Tools::CodonTable)
+
 =head1 SUGGESTED STANZA FOR GENOME BROWSER
 
 This produces a nice gbrowse display in which the DNA/GC Content glyph
