@@ -3,12 +3,9 @@ package Bio::Graphics::Panel;
 use strict;
 use Bio::Graphics::Glyph::Factory;
 use Bio::Graphics::Feature;
-use GD;
-use vars '$VERSION';
 
-$VERSION = 1.04;
-
-use constant KEYLABELFONT => gdMediumBoldFont;
+# KEYLABELFONT must be treated as string until image_class is established
+use constant KEYLABELFONT => 'gdMediumBoldFont';
 use constant KEYSPACING   => 5; # extra space between key columns
 use constant KEYPADTOP    => 5;  # extra padding before the key starts
 use constant KEYCOLOR     => 'wheat';
@@ -39,11 +36,16 @@ sub new {
   my $allcallbacks = $options{-all_callbacks} || 0;
   my $gridcolor    = $options{-gridcolor} || GRIDCOLOR;
   my $grid         = $options{-grid}       || 0;
+  my $flip         = $options{-flip}       || 0;
   my $empty_track_style   = $options{-empty_tracks} || 'key';
   my $truecolor    = $options{-truecolor}  || 0;
-
-  $offset   ||= $options{-segment}->start-1 if $options{-segment};
-  $length   ||= $options{-segment}->length  if $options{-segment};
+  my $image_class  = $options{-image_class} || 'GD';
+  $options{-stop}||= $options{-end};  # damn damn damn
+  
+  if (my $seg = $options{-segment}) {
+    $offset = eval {$seg->start-1} || 0;
+    $length = $seg->length;
+  }
 
   $offset   ||= $options{-start}-1 if defined $options{-start};
   $length   ||= $options{-stop}-$options{-start}+1 
@@ -70,7 +72,11 @@ sub new {
 		key_align => $keyalign,
 		all_callbacks => $allcallbacks,
 		truecolor     => $truecolor,
-		empty_track_style    => $empty_track_style,
+		flip          => $flip,
+		empty_track_style  => $empty_track_style,
+		image_class  => $image_class,
+		image_package => $image_class . '::Image',     # Accessors
+		polygon_package => $image_class . '::Polygon',
 	       },$class;
 }
 
@@ -99,6 +105,13 @@ sub pad_bottom {
   $g;
 }
 
+sub flip {
+  my $self = shift;
+  my $g = $self->{flip};
+  $self->{flip} = shift if @_;
+  $g;
+}
+
 # values of empty_track_style are:
 #    "suppress" -- suppress empty tracks entirely (default)
 #    "key"      -- show just the key in "between" mode
@@ -118,6 +131,15 @@ sub key_style {
   $g;
 }
 
+# public routine for mapping from a base pair
+# location to pixel coordinates
+sub location2pixel {
+  my $self   = shift;
+  my $end    = $self->end + 1;
+  my @coords = $self->{flip} ? map { $end-$_ } @_ : @_;
+  $self->map_pt(@coords);
+}
+
 # numerous direct calls into array used here for performance considerations
 sub map_pt {
   my $self   = shift;
@@ -125,9 +147,11 @@ sub map_pt {
   my $scale  = $self->{scale} || $self->scale;
   my $pl     = $self->{pad_left};
   my $pr     = $self->{width} - $self->{pad_right};
+  my $flip   = $self->{flip};
+  my $length = $self->{length};
   my @result;
   foreach (@_) {
-    my $val = int (0.5 + $pl + ($_-$offset-1) * $scale);
+    my $val = $flip ? int (0.5 + $pr - ($length - ($_- 1)) * $scale) : int (0.5 + $pl + ($_-$offset-1) * $scale);
     $val = $pl-1 if $val < $pl;
     $val = $pr+1 if $val > $pr;
     push @result,$val;
@@ -139,10 +163,14 @@ sub map_no_trunc {
   my $self   = shift;
   my $offset = $self->{offset};
   my $scale  = $self->scale;
-  my $pl = $self->{pad_left};
+  my $pl     = $self->{pad_left};
+  my $pr     = $self->{width} - $self->{pad_right};
+  my $flip   = $self->{flip};
+  my $length = $self->{length};
+  my $end    = $offset+$length;
   my @result;
   foreach (@_) {
-    my $val = int (0.5 + $pl + ($_-$offset-1) * $scale);
+    my $val = $flip ? int (0.5 + $pl + ($end - ($_- 1)) * $scale) : int (0.5 + $pl + ($_-$offset-1) * $scale);
     push @result,$val;
   }
   @result;
@@ -150,8 +178,7 @@ sub map_no_trunc {
 
 sub scale {
   my $self = shift;
-  $self->{scale} ||= ($self->{width}-$self->pad_left-$self->pad_right-1)/($self->length-1);  # wrong!
-#  $self->{scale} ||= ($self->{width}-$self->pad_left-$self->pad_right-1)/($self->length);   # right, but I don't want to fix regression tests!
+  $self->{scale} ||= ($self->{width}-$self->pad_left-$self->pad_right)/($self->length);
 }
 
 sub start { shift->{offset}+1}
@@ -256,23 +283,25 @@ sub _do_add_track {
   $glyph_name = $map if defined $map;
   $glyph_name ||= 'generic';
 
+  local $^W = 0;  # uninitialized variable warnings under 5.00503
+
   my $panel_map =
     ref($map) eq 'CODE' ?  sub {
       my $feature = shift;
-      return 'track' if eval { $feature->primary_tag  eq 'track' };
-      return 'group' if eval { $feature->primary_tag  eq 'group' };
+      return 'track' if eval { defined $feature->primary_tag && $feature->primary_tag  eq 'track' };
+      return 'group' if eval { defined $feature->primary_tag && $feature->primary_tag  eq 'group' };
       return $map->($feature);
     }
    : ref($map) eq 'HASH' ? sub {
      my $feature = shift;
-     return 'track' if eval { $feature->primary_tag  eq 'track' };
-     return 'group' if eval { $feature->primary_tag  eq 'group' };
+     return 'track' if eval { defined $feature->primary_tag && $feature->primary_tag  eq 'track' };
+     return 'group' if eval { defined $feature->primary_tag && $feature->primary_tag  eq 'group' };
      return eval {$map->{$feature->primary_tag}} || 'generic';
    }
    : sub {
      my $feature = shift;
-     return 'track' if eval { $feature->primary_tag  eq 'track' };
-     return 'group' if eval { $feature->primary_tag  eq 'group' };
+     return 'track' if eval { defined $feature->primary_tag && $feature->primary_tag  eq 'track' };
+     return 'group' if eval { defined $feature->primary_tag && $feature->primary_tag  eq 'group' };
      return $glyph_name;
    };
 
@@ -310,8 +339,10 @@ sub _add_track {
   return $track;
 }
 
+
 sub height {
   my $self = shift;
+  $self->setup_fonts;
   my $spacing           = $self->spacing;
   my $key_height        = $self->format_key;
   my $empty_track_style = $self->empty_track_style;
@@ -336,6 +367,18 @@ sub height {
   return $height + $key_height + $self->pad_top + $self->pad_bottom;
 }
 
+sub setup_fonts {
+  my $self = shift;
+  return if ref $self->{key_font};
+
+  my $image_class = $self->image_class;
+  eval "use $image_class; 1" or die $@;
+
+  my $keyfont = $self->{key_font};
+  my $font_obj = $image_class->$keyfont;
+  $self->{key_font} = $font_obj;
+}
+
 sub gd {
   my $self        = shift;
   my $existing_gd = shift;
@@ -344,12 +387,25 @@ sub gd {
 
   return $self->{gd} if $self->{gd};
 
+  $self->setup_fonts;
+
+  unless ($existing_gd) {
+    # Encapsulating this eval within a BEGIN block
+    # adds nothing since $image_class is undefined at compile time.
+    # gd exported functions should all use ();
+    # BEGIN {
+    my $image_class = $self->image_class;
+    eval "use $image_class; 1" or die $@;
+    # }
+  }
+
   my $width  = $self->width;
   my $height = $self->height;
 
-  my $gd = $existing_gd || GD::Image->new($width,$height,
-					  ($self->{truecolor} && GD::Image->can('isTrueColor') ? 1 : ())
-					 );
+  my $pkg = $self->image_package;
+  my $gd  = $existing_gd || $pkg->new($width,$height,
+				      ($self->{truecolor} && $pkg->can('isTrueColor') ? 1 : ())
+				     );
 
   my %translation_table;
   for my $name ('white','black',keys %COLORS) {
@@ -420,15 +476,23 @@ sub gd {
 
 
   $self->draw_bottom_key($gd,$pl,$offset) if $self->{key_style} eq 'bottom';
-
   return $self->{gd} = $gd;
 }
 
+
+# Package accessors
+# GD (and GD::SVG)'s new() resides in GD::Image
+sub image_class     { return shift->{image_class}; }
+sub image_package   { return shift->{image_package}; }
+sub polygon_package { return shift->{polygon_package}; }
+
 sub boxes {
   my $self = shift;
   my @boxes;
   my $offset = 0;
 
+  $self->setup_fonts;
+
   my $pl = $self->pad_left;
   my $pt = $self->pad_top;
   my $between_key       = $self->{key_style} eq 'between';
@@ -466,7 +530,10 @@ sub draw_between_key {
   my $x =   $self->{key_align} eq 'center' ? $self->width - (CORE::length($key) * $self->{key_font}->width)/2
           : $self->{key_align} eq 'right'  ? $self->width - CORE::length($key)
           : $self->pad_left;
-  $gd->string($self->{key_font},$x,$offset,$key,1);
+
+  # Key color hard-coded. Should be configurable for the control freaks.
+  my $color = $self->translate_color('black');
+  $gd->string($self->{key_font},$x,$offset,$key,$color);
   return $self->{key_font}->height;
 }
 
@@ -477,7 +544,8 @@ sub draw_side_key {
   my $key = $track->option('key') or return;
   my $pos = $side eq 'left' ? $self->pad_left - $self->{key_font}->width * CORE::length($key)-3
                             : $self->width - $self->pad_right+3;
-  $gd->string($self->{key_font},$pos,$offset,$key,1);
+  my $color = $self->translate_color('black');
+  $gd->string($self->{key_font},$pos,$offset,$key,$color);
 }
 
 # draw the keys -- bottom
@@ -488,7 +556,8 @@ sub draw_bottom_key {
 
   my $color = $self->translate_color($self->{key_color});
   $gd->filledRectangle($left,$top,$self->width - $self->pad_right,$self->height-$self->pad_bottom,$color);
-  $gd->string($self->{key_font},$left,KEYPADTOP+$top,"KEY:",1);
+  my $text_color = $self->translate_color('black');
+  $gd->string($self->{key_font},$left,KEYPADTOP+$top,"KEY:",$text_color);
   $top += $self->{key_font}->height + KEYPADTOP;
 
   $_->draw($gd,$left,$top) foreach @$key_glyphs;
@@ -597,8 +666,8 @@ sub draw_empty {
   my $right = $self->width-$self->pad_right;
   my $color = $self->translate_color(MISSING_TRACK_COLOR);
   if ($style eq 'dashed') {
-    $gd->setStyle($color,$color,gdTransparent,gdTransparent);
-    $gd->line($left,$offset,$right,$offset,gdStyled);
+    $gd->setStyle($color,$color,gdTransparent(),gdTransparent());
+    $gd->line($left,$offset,$right,$offset,gdStyled());
   } else {
     $gd->line($left,$offset,$right,$offset,$color);
   }
@@ -624,6 +693,7 @@ sub draw_grid {
   my $pl = $self->pad_left;
   my $pt = $self->pad_top;
   my $pb = $self->height - $self->pad_bottom;
+  local $self->{flip} = 0;
   for my $tick (@positions) {
     my ($pos) = $self->map_pt($tick);
     $gd->line($pos,$pt,$pos,$pb,$gridcolor);
@@ -635,8 +705,9 @@ sub ticks {
   my $self = shift;
   my ($length,$minwidth) = @_;
 
+  my $img = $self->image_class;
   $length   = $self->{length}       unless defined $length;
-  $minwidth = gdSmallFont->width*7  unless defined $minwidth;
+  $minwidth = $img->gdSmallFont->width*7  unless defined $minwidth;
 
   my ($major,$minor);
 
@@ -652,6 +723,11 @@ sub ticks {
     last if $pixels >= $minwidth;
     $interval *= 10;
   }
+
+  # to make sure a major tick shows up somewhere in the first half
+  #
+  $interval *= .5 if ($interval > 0.5*$length);
+
   return ($interval,$interval/10);
 }
 
@@ -706,14 +782,16 @@ sub set_pen {
   my $self = shift;
   my ($linewidth,$color) = @_;
   return $self->{pens}{$linewidth,$color} if $self->{pens}{$linewidth,$color};
-
-  my $pen = $self->{pens}{$linewidth} = GD::Image->new($linewidth,$linewidth);
+  my $gd = $self->{gd};
+  my $pkg = $self->image_package;
+  my $pen = $self->{pens}{$linewidth} = $pkg->new($linewidth,$linewidth);
   my @rgb = $self->rgb($color);
   my $bg = $pen->colorAllocate(255,255,255);
   my $fg = $pen->colorAllocate(@rgb);
   $pen->fill(0,0,$fg);
-  $self->{gd}->setBrush($pen);
-  return gdBrushed;
+  #  $self->{gd}->setBrush($pen);
+  $gd->setBrush($pen);
+  return gdBrushed();
 }
 
 sub png {
@@ -721,6 +799,11 @@ sub png {
   $gd->png;
 }
 
+sub svg {
+  my $gd = shift->gd;
+  $gd->svg;
+}
+
 sub read_colors {
   my $class = shift;
   while (<DATA>) {
@@ -746,6 +829,12 @@ sub color_names {
     return wantarray ? keys %COLORS : [keys %COLORS];
 }
 
+sub finished {
+  for my $track (@{shift->{tracks}}) {
+    $track->finished();
+  }
+}
+
 1;
 
 __DATA__
@@ -889,6 +978,56 @@ wheat                F5           DE            B3
 whitesmoke           F5           F5            F5
 yellow               FF           FF            00
 yellowgreen          9A           CD            32
+gradient1	00 ff 00
+gradient2	0a ff 00
+gradient3	14 ff 00
+gradient4	1e ff 00
+gradient5	28 ff 00
+gradient6	32 ff 00
+gradient7	3d ff 00
+gradient8	47 ff 00
+gradient9	51 ff 00
+gradient10	5b ff 00
+gradient11	65 ff 00
+gradient12	70 ff 00
+gradient13	7a ff 00
+gradient14	84 ff 00
+gradient15	8e ff 00
+gradient16	99 ff 00
+gradient17	a3 ff 00
+gradient18	ad ff 00
+gradient19	b7 ff 00
+gradient20	c1 ff 00
+gradient21	cc ff 00
+gradient22	d6 ff 00
+gradient23	e0 ff 00
+gradient24	ea ff 00
+gradient25	f4 ff 00
+gradient26	ff ff 00
+gradient27	ff f4 00
+gradient28	ff ea 00
+gradient29	ff e0 00
+gradient30	ff d6 00
+gradient31	ff cc 00
+gradient32	ff c1 00
+gradient33	ff b7 00
+gradient34	ff ad 00
+gradient35	ff a3 00
+gradient36	ff 99 00
+gradient37	ff 8e 00
+gradient38	ff 84 00
+gradient39	ff 7a 00
+gradient40	ff 70 00
+gradient41	ff 65 00
+gradient42	ff 5b 00
+gradient43	ff 51 00
+gradient44	ff 47 00
+gradient45	ff 3d 00
+gradient46	ff 32 00
+gradient47	ff 28 00
+gradient48	ff 1e 00
+gradient49	ff 14 00
+gradient50	ff 0a 00
 __END__
 
 =head1 NAME
@@ -906,8 +1045,8 @@ Bio::Graphics::Panel - Generate GD images of Bio::Seq objects
  use Bio::SeqIO;
 
  my $file = shift                       or die "provide a sequence file as the argument";
- my $io = Bio::SeqIO->new(-file=>$file) or die "couldn't create Bio::SeqIO";
- my $seq = $io->next_seq                or die "couldn't find a sequence in the file";
+ my $io = Bio::SeqIO->new(-file=>$file) or die "could not create Bio::SeqIO";
+ my $seq = $io->next_seq                or die "could not find a sequence in the file";
 
  my @features = $seq->all_SeqFeatures;
 
@@ -919,19 +1058,19 @@ Bio::Graphics::Panel - Generate GD images of Bio::Seq objects
  }
 
  my $panel = Bio::Graphics::Panel->new(
- 				      -segment   => $seq,
+                                      -length    => $seq->length,
  				      -key_style => 'between',
  				      -width     => 800,
  				      -pad_left  => 10,
  				      -pad_right => 10,
  				      );
- $panel->add_track($seq,
- 		  -glyph => 'arrow',
+ $panel->add_track( arrow => Bio::SeqFeature::Generic->new(-start=>1,
+                                                           -end=>$seq->length),
  		  -bump => 0,
  		  -double=>1,
  		  -tick => 2);
-
- $panel->add_track($seq,
+ $panel->add_track(generic => Bio::SeqFeature::Generic->new(-start=>1,
+							  -end=>$seq->length),
  		  -glyph  => 'generic',
  		  -bgcolor => 'blue',
  		  -label  => 1,
@@ -956,6 +1095,8 @@ Bio::Graphics::Panel - Generate GD images of Bio::Seq objects
  }
 
  print $panel->png;
+ $panel->finished;
+
  exit 0;
 
 =head1 DESCRIPTION
@@ -986,7 +1127,8 @@ You can add a key to the generated image using either of two key
 styles.  One style places the key captions at the top of each track.
 The other style generates a graphical key at the bottom of the image.
 
-Note that this modules depends on GD.
+Note that this module depends on GD. The optional SVG output depends
+on GD::SVG and SVG.
 
 =head1 METHODS
 
@@ -1031,6 +1173,8 @@ a set of tag/value pairs as follows:
   -stop       Stop of range, in 1-based              none
 	      coordinates.
 
+  -end        Same as -stop.
+
   -segment    A Bio::SeqI or Das::Segment            none
               object, used to derive sequence
 	      range if not otherwise specified.
@@ -1078,6 +1222,12 @@ a set of tag/value pairs as follows:
               to draw a thin grey line ("line"),
               or to draw a dashed line ("dashed").
 
+  -flip       flip the drawing coordinates left     false
+              to right, so that lower coordinates
+              are to the right.  This can be
+              useful for drawing (-) strand
+              features.
+
   -all_callbacks Whether to invoke callbacks on      false
                the automatic "track" and "group"
                glyphs.
@@ -1093,6 +1243,11 @@ a set of tag/value pairs as follows:
 
   -gridcolor   Color of the grid                     lightcyan
 
+  -image_class To create output in scalable vector
+               graphics (SVG), optionally pass the image
+               class parameter 'GD::SVG'. Defaults to
+               using vanilla GD. See the corresponding
+               image_class() method below for details.
 
 Typically you will pass new() an object that implements the
 Bio::RangeI interface, providing a length() method, from which the
@@ -1113,6 +1268,11 @@ and then to compute its width using the formula:
 
   $width = gdMediumBoldFont->width * length($longest_key) +3;
 
+In order to obtain scalable vector graphics (SVG) output, you should
+pass new() the -image_class=E<gt>'GD::SVG' parameter. This will cause
+Bio::Graphics::Panel to load the optional GD::SVG module. See the gd()
+and svg() methods below for additional information.
+
 =back
 
 =head2 OBJECT METHODS
@@ -1153,6 +1313,11 @@ Currently, the following glyphs are available:
   Name        Description
   ----        -----------
 
+  anchored_arrow
+              a span with vertical bases |---------|.  If one or
+              the other end of the feature is off-screen, the base
+              will be replaced by an arrow.
+
   arrow	      An arrow; can be unidirectional or bidirectional.
 	      It is also capable of displaying a scale with
 	      major and minor tickmarks, and can be oriented
@@ -1198,9 +1363,16 @@ Currently, the following glyphs are available:
   pinsertion  A triangle designed to look like an insertion location
               (e.g. a transposon insertion).
 
+  processed_transcript  multi-purpose representation of a spliced mRNA, including
+			positions of UTRs
+
   primers     Two inward pointing arrows connected by a line.
 	      Used for STSs.
 
+  redgreen_box A box that changes from green->yellow->red as the score
+              of the feature increases from 0.0 to 1.0.  Useful for
+              representing microarray results.
+
   rndrect     A round-cornered rectangle.
 
   segments    A set of filled rectangles connected by solid lines.
@@ -1230,6 +1402,8 @@ Currently, the following glyphs are available:
 
   triangle    A triangle whose width and orientation can be altered.
 
+  xyplot      Histograms and other graphs plotted against the genome.
+
 If the glyph name is omitted from add_track(), the "generic" glyph
 will be used by default.  To get more information about a glyph, run
 perldoc on "Bio::Graphics::Glyph::glyphname", replacing "glyphname"
@@ -1308,6 +1482,13 @@ Typical usage is:
 unshift_track() works like add_track(), except that the new track is
 added to the top of the image rather than the bottom.
 
+=item $track = $panel-E<gt>insert_track($position,$glyph,$features,@options)
+
+This works like add_track(), but the track is inserted into the
+indicated position.  The track will be inserted B<before> the
+indicated position; thus specify a track of 0 to insert the new track
+at the beginning.
+
 =item $gd = $panel-E<gt>gd([$gd])
 
 The gd() method lays out the image and returns a GD::Image object
@@ -1319,11 +1500,52 @@ wish to draw on top of.  If you do so, you should call the width() and
 height() methods first to ensure that the image has sufficient
 dimensions.
 
+If you passed new() the -image_class=E<gt>'GD::SVG' parameter, the gd() method
+returns a GD::SVG::Image object. This object overrides GD::Image
+methods in order to generate SVG output. It behaves exactly as
+described for GD::Image objects with one exception: it implements and
+svg() method instead of the png() or jpeg() methods. Currently there
+is no direct access to underlying SVG calls but this is subject to
+change in the future.
+
 =item $png = $panel-E<gt>png
 
 The png() method returns the image as a PNG-format drawing, without
 the intermediate step of returning a GD::Image object.
 
+=item $svg = $panel-E<gt>svg
+
+The svg() method returns the image in an XML-ified SVG format.
+
+=item $panel-E<gt>finished
+
+Bio::Graphics creates memory cycles.  When you are finished with the
+panel, you should call its finished() method.  Otherwise you will have
+memory leaks.  This is only an issue if you're going to create several
+panels in a single program.
+
+=item $image_class = $panel-E<gt>image_class
+
+The image_class() method returns the current drawing package being
+used, currently one of GD or GD::SVG.  This is primarily used
+internally to ensure that calls to GD's exported methods are called in
+an object-oriented manner to avoid compile time undefined string
+errors.  This is usually not needed for external use.
+
+=item $image_package = $panel-E<gt>image_package
+
+This accessor method, like image_class() above is provided as a
+convenience.  It returns the current image package in use, currently
+one of GD::Image or GD::SVG::Image.  This is not normally used
+externally.
+
+=item $polygon_package = $panel-E<gt>polygon_package
+
+This accessor method, like image_package() above is provided as a
+convenience.  It returns the current polygon package in use, currently
+one of GD::Polygon or GD::SVG::Polygon.  This is not normally used
+externally except in the design of glyphs.
+
 =item $boxes = $panel-E<gt>boxes
 
 =item @boxes = $panel-E<gt>boxes
@@ -1334,13 +1556,14 @@ array ref.  In an list context, the method returns the array directly.
 
 Each member of the list is an anonymous array of the following format:
 
-  [ $feature, $x1, $y1, $x2, $y2 ]
+  [ $feature, $x1, $y1, $x2, $y2, $track ]
 
 The first element is the feature object; either an
 Ace::Sequence::Feature, a Das::Segment::Feature, or another Bioperl
 Bio::SeqFeatureI object.  The coordinates are the topleft and
 bottomright corners of the glyph, including any space allocated for
-labels.
+labels. The track is the Bio::Graphics::Glyph object corresponding
+to the track that the feature is rendered inside.
 
 =item $position = $panel-E<gt>track_position($track)
 
@@ -1349,6 +1572,11 @@ coordinate of a track by calling track_position() with the value
 returned by add_track() or unshift_track().  This will return undef if
 called before gd() or boxes() or with an invalid track.
 
+=item @pixel_coords = $panel-E<gt>location2pixel(@feature_coords)
+
+Public routine to map feature coordinates (in base pairs) into pixel
+coordinates relative to the left-hand edge of the picture.
+
 =back
 
 =head1 GLYPH OPTIONS
@@ -1386,11 +1614,16 @@ some are shared by all glyphs:
   -bump_limit Maximum number of levels     undef (unlimited)
               to bump
 
+  -strand_arrow Whether to indicate        undef (false)
+                 strandedness
+
+  -stranded    Synonym for -strand_arrow   undef (false)
+
   -connector  Type of connector to         none
 	      use to connect related
 	      features.  Options are
-	      "hat", "dashed" and
-	      "none".
+	      "solid," "hat", "dashed", 
+              "quill" and "none".
 
   -key        Description of track for     undef
 	      use in key.
@@ -1464,7 +1697,17 @@ subglyphs, and the "transcript" and "transcript2" glyphs draw
 hat-shaped lines between their subglyphs.  All other glyphs do not
 connect their components.  You can override this behavior by providing 
 a -connector option, to explicitly set the type of connector.  Valid
-options are "dashed", "solid", "hat" and "none".
+options are:
+
+
+   "hat"     an upward-angling conector
+   "solid"   a straight horizontal connector
+   "quill"   a decorated line with small arrows indicating strandedness
+             (like the UCSC Genome Browser uses)
+   "dashed"  a horizontal dashed line.
+
+The B<-connector_color> option controls the color of the connector, if
+any.
 
 B<Collision control:> The -bump argument controls what happens when
 glyphs collide.  By default, they will simply overlap (value 0).  A
@@ -1485,6 +1728,12 @@ surround the top level features.  If you wish for the rectangles to
 surround subpieces of the glyph, such as the exons in a transcript,
 set box_subparts to a true value.
 
+B<strand_arrow:> If set to true, some glyphs will indicate their
+strandedness, usually by drawing an arrow.  For this to work, the
+Bio::SeqFeature must have a strand of +1 or -1.  The glyph will ignore
+this directive if the underlying feature has a strand of zero or
+undef.
+
 B<sort_order>: By default, features are drawn with a layout based only on the
 position of the feature, assuring a maximal "packing" of the glyphs
 when bumped.  In some cases, however, it makes sense to display the
@@ -1504,21 +1753,37 @@ In all cases, the "left" position will be used to break any ties.  To
 break ties using another field, options may be strung together using a
 "|" character; e.g. "strand|low_score|right" would cause the features
 to be sorted first by strand, then score (lowest to highest), then by
-"right" position in the sequence.  Finally, a subroutine coderef can
-be provided, which should expect to receive two feature objects (via
-the special sort variables $a and $b), and should return -1, 0 or 1
-(see Perl's sort() function for more information); this subroutine
-will be used without further modification for sorting.  For example,
-to sort a set of database search hits by bits (stored in the features'
-"score" fields), scaled by the log of the alignment length (with
-"left" position breaking any ties):
+"right" position in the sequence.
 
-  sort_order = sub { ( $b->score/log($b->length)
-                                      <=>
-                       $a->score/log($a->length) )
-                                      ||
-                     ( $a->start <=> $b->start )
-                   }
+Finally, a subroutine coderef with a $$ prototype can be provided.  It
+will receive two B<glyph> as arguments and should return -1, 0 or 1
+(see Perl's sort() function for more information).  For example, to
+sort a set of database search hits by bits (stored in the features'
+"score" fields), scaled by the log of the alignment length (with
+"start" position breaking any ties):
+
+  sort_order = sub ($$) {
+    my ($glyph1,$glyph2) = @_;
+    my $a = $glyph1->feature;
+    my $b = $glyph2->feature;
+    ( $b->score/log($b->length)
+          <=>
+      $a->score/log($a->length) )
+          ||
+    ( $a->start <=> $b->start )
+  }
+
+It is important to remember to use the $$ prototype as shown in the
+example.  Otherwise Bio::Graphics will quit with an exception. The
+arguments are subclasses of Bio::Graphics::Glyph, not the features
+themselves.  While glyphs implement some, but not all, of the feature
+methods, to be safe call the two glyphs' feature() methods in order to
+convert them into the actual features.
+
+The '-always_sort' option, if true, will sort features even if bumping
+is turned off.  This is useful if you would like overlapping features
+to stack in a particular order.  Features towards the end of the list
+will overlay those towards the beginning of the sort order.
 
 B<bump_limit>: When bumping is chosen, colliding features will
 ordinarily move upward or downward without limit.  When many features
@@ -1603,6 +1868,7 @@ and then caches the result.
    spacing()	      Get/set spacing between tracks
    key_spacing()      Get/set spacing between keys
    length()	      Get/set length of segment (bp)
+   flip()             Get/set coordinate flipping
    pad_top()	      Get/set top padding
    pad_left()	      Get/set left padding
    pad_bottom()	      Get/set bottom padding
@@ -1676,16 +1942,19 @@ L<Bio::Graphics::Glyph::pinsertion>,
 L<Bio::Graphics::Glyph::primers>,
 L<Bio::Graphics::Glyph::rndrect>,
 L<Bio::Graphics::Glyph::segments>,
+L<Bio::Graphics::Glyph::redgreen_box>,
 L<Bio::Graphics::Glyph::ruler_arrow>,
 L<Bio::Graphics::Glyph::toomany>,
 L<Bio::Graphics::Glyph::transcript>,
 L<Bio::Graphics::Glyph::transcript2>,
 L<Bio::Graphics::Glyph::translation>,
 L<Bio::Graphics::Glyph::triangle>,
+L<Bio::Graphics::Glyph::xyplot>,
 L<Bio::SeqI>,
 L<Bio::SeqFeatureI>,
 L<Bio::Das>,
 L<GD>
+L<GD::SVG>
 
 =head1 AUTHOR
 
