@@ -1,4 +1,4 @@
-# $Id: ChainI.pm,v 1.16 2006/07/04 22:23:17 mauricio Exp $
+# $Id: ChainI.pm,v 1.11 2003/06/04 08:36:40 heikki Exp $
 #
 # bioperl module for Bio::LiveSeq::ChainI
 #
@@ -36,20 +36,26 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org             - General discussion
+  http://bio.perl.org/MailList.html - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via the
-web:
+the bugs and their resolution.  Bug reports can be submitted via email
+or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bio.perl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Joseph A.L. Insana
 
 Email:  Insana@ebi.ac.uk, jinsana@gmx.net
+Address: 
+
+     EMBL Outstation, European Bioinformatics Institute
+     Wellcome Trust Genome Campus, Hinxton
+     Cambs. CB10 1SD, United Kingdom 
 
 =head1 APPENDIX
 
