@@ -1,4 +1,4 @@
-# $Id: Repeat_Unit.pm,v 1.9.4.1 2006/10/02 23:10:21 sendu Exp $
+# $Id: Repeat_Unit.pm,v 1.8 2003/06/04 08:36:40 heikki Exp $
 #
 # bioperl module for Bio::LiveSeq::Repeat_Unit
 #
@@ -28,6 +28,12 @@ The strand could be 1 (forward strand, default), -1 (reverse strand).
 
 Email:  Insana@ebi.ac.uk, jinsana@gmx.net
 
+Address: 
+
+     EMBL Outstation, European Bioinformatics Institute
+     Wellcome Trust Genome Campus, Hinxton
+     Cambs. CB10 1SD, United Kingdom 
+
 =head1 APPENDIX
 
 The rest of the documentation details each of the object
@@ -40,7 +46,9 @@ methods. Internal methods are usually preceded with a _
 package Bio::LiveSeq::Repeat_Unit;
 
 use strict;
-use base qw(Bio::LiveSeq::Repeat_Region);
+use vars qw(@ISA);
+use Bio::LiveSeq::Repeat_Region; # uses Repeat_Region, inherits from it
+@ISA=qw(Bio::LiveSeq::Repeat_Region);
 
 =head2 new
 
