@@ -1,4 +1,4 @@
-# $Id: LocatableSeq.pm,v 1.22 2002/10/22 07:38:24 lapp Exp $
+# $Id: LocatableSeq.pm,v 1.31 2003/11/21 15:48:00 heikki Exp $
 #
 # BioPerl module for Bio::LocatableSeq
 #
@@ -12,7 +12,7 @@
 
 =head1 NAME
 
-Bio::LocatableSeq - A Sequence object with start/end points on it 
+Bio::LocatableSeq - A Sequence object with start/end points on it
   that can be projected into a MSA or have coordinates relative to another seq.
 
 =head1 SYNOPSIS
@@ -52,7 +52,7 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
   http://bio.perl.org/MailList.html             - About the mailing lists
 
 
-The locatable sequence object was developed mainly because the 
+The locatable sequence object was developed mainly because the
 SimpleAlign object requires this functionality, and in the rewrite
 of the Sequence object we had to decide what to do with this.
 
@@ -95,20 +95,18 @@ use Bio::Location::Fuzzy;
 
 @ISA = qw(Bio::PrimarySeq Bio::RangeI);
 
-# new() is inherited from Bio::Root::Root
-
 sub new {
     my ($class, @args) = @_;
     my $self = $class->SUPER::new(@args);
 
-    my ($start,$end,$strand) = 
+    my ($start,$end,$strand) =
 	$self->_rearrange( [qw(START END STRAND)],
 			   @args);
-    
+
     defined $start && $self->start($start);
     defined $end   && $self->end($end);
     defined $strand && $self->strand($strand);
-    
+
     return $self; # success - we hope!
 }
 
@@ -116,7 +114,7 @@ sub new {
 
  Title   : start
  Usage   : $obj->start($newval)
- Function: 
+ Function:
  Returns : value of start
  Args    : newvalue (optional)
 
@@ -127,16 +125,17 @@ sub start{
    if( @_ ) {
       my $value = shift;
       $self->{'start'} = $value;
-    }
-    return $self->{'start'};
-
+  }
+   return $self->{'start'} if defined $self->{'start'};
+   return 1                if $self->seq;
+   return;
 }
 
 =head2 end
 
  Title   : end
  Usage   : $obj->end($newval)
- Function: 
+ Function:
  Returns : value of end
  Args    : newvalue (optional)
 
@@ -147,28 +146,32 @@ sub end {
    if( @_ ) {
       my $value = shift;
       my $string = $self->seq;
-      if ($string and $self->start) {
-	  my $s2 = $string;
-	  $string =~ s/[.-]+//g;
-	  my $len = CORE::length $string;
-	  my $new_end = $self->start + $len - 1 ;
-	  my $id = $self->id; 
-	  $self->warn("In sequence $id residue count gives value $len.
-Overriding value [$value] with value $new_end for Bio::LocatableSeq::end().")
-	      and $value = $new_end if $new_end != $value and $self->verbose > 0;
+      if ($self->seq) {
+          my $len = $self->_ungapped_len;
+	  my $id = $self->id;
+	  $self->warn("In sequence $id residue count gives end value $len.
+Overriding value [$value] with value $len for Bio::LocatableSeq::end().")
+	      and $value = $len if $len != $value and $self->verbose > 0;
       }
-      
+
       $self->{'end'} = $value;
     }
-    return $self->{'end'};
 
+   return $self->{'end'} || $self->_ungapped_len;
+}
+
+sub _ungapped_len {
+    my $self = shift;
+    my $string = $self->seq || '';
+    $string =~ s/[.-]+//g;
+    $self->seq ? (return $self->start + CORE::length($string) - 1 ) : undef;
 }
 
 =head2 strand
 
  Title   : strand
  Usage   : $obj->strand($newval)
- Function: 
+ Function:
  Returns : value of strand
  Args    : newvalue (optional)
 
@@ -187,42 +190,42 @@ sub strand{
 
  Title   : get_nse
  Usage   :
- Function: read-only name of form id/start-end 
+ Function: read-only name of form id/start-end
  Example :
- Returns : 
+ Returns :
  Args    :
 
 =cut
 
 sub get_nse{
    my ($self,$char1,$char2) = @_;
-  
+
    $char1 ||= "/";
    $char2 ||= "-";
 
-   $self->throw("Attribute id not set") unless $self->id();
-   $self->throw("Attribute start not set") unless $self->start();
-   $self->throw("Attribute end not set") unless $self->end();
+   $self->throw("Attribute id not set") unless defined($self->id());
+   $self->throw("Attribute start not set") unless defined($self->start());
+   $self->throw("Attribute end not set") unless defined($self->end());
 
    return $self->id() . $char1 . $self->start . $char2 . $self->end ;
 
 }
 
 
-=head2 no_gaps
+=head2 no_gap
 
  Title   : no_gaps
  Usage   :$self->no_gaps('.')
- Function: 
+ Function:
 
            Gets number of gaps in the sequence. The count excludes
            leading or trailing gap characters.
 
            Valid bioperl sequence characters are [A-Za-z\-\.\*]. Of
            these, '.' and '-' are counted as gap characters unless an
-           optional argument specifies one of them. 
+           optional argument specifies one of them.
 
- Returns : number of internal gaps in the sequnce. 
+ Returns : number of internal gaps in the sequnce.
  Args    : a gap character (optional)
 
 =cut
@@ -277,20 +280,29 @@ sub no_gaps {
 sub column_from_residue_number {
     my ($self, $resnumber) = @_;
 
-    $self->throw("Residue number has to be a positive integer, not [$resnumber]") 
+    $self->throw("Residue number has to be a positive integer, not [$resnumber]")
 	unless $resnumber =~ /^\d+$/ and $resnumber > 0;
 
     if ($resnumber >= $self->start() and $resnumber <= $self->end()) {
 	my @residues = split //, $self->seq;
 	my $count = $self->start();
 	my $i;
-	for ($i=0; $i < @residues; $i++) {
+	my ($start,$end,$inc,$test);
+        my $strand = $self->strand || 0;
+	# the following bit of "magic" allows the main loop logic to be the
+	# same regardless of the strand of the sequence
+	($start,$end,$inc,$test)= ($strand == -1)?
+            (scalar(@residues-1),0,-1,sub{$i >= $end}) :
+                (0,scalar(@residues-1),1,sub{$i <= $end});
+
+	for ($i=$start; $test->(); $i+= $inc) {
 	    if ($residues[$i] ne '.' and $residues[$i] ne '-') {
 		$count == $resnumber and last;
 		$count++;
-	    }		    
+	    }
 	}
-	# $i now holds the index of the column. The actual column number is this index + 1
+	# $i now holds the index of the column.
+        # The actual column number is this index + 1
 	
 	return $i+1;
     }
@@ -303,50 +315,48 @@ sub column_from_residue_number {
 =head2 location_from_column
 
  Title   : location_from_column
- Usage   : $loc = $ali->location_from_column( $seq_number, $column_number)
+ Usage   : $loc = $ali->location_from_column($column_number)
  Function:
 
-           This function gives the residue number in the sequence with
-           the given name for a given position in the alignment
-           (i.e. column number) of the given. Gaps complicate this
-           process and force the output to be a L<Bio::Range> where
-           values can be undefined. For example, for the alignment
+           This function gives the residue number for a given position
+           in the alignment (i.e. column number) of the given. Gaps
+           complicate this process and force the output to be a
+           L<Bio::Range> where values can be undefined. For example,
+           for the sequence:
 
-  	     Seq1/91-97 AC..DEF.G.
-  	     Seq2/1-9   .CYHDEFKGK
+  	     Seq/91-97 .AC..DEF.G.
 
-           location_from_column( Seq1/91-97, 3 ) position 93
-           location_from_column( Seq1/91-97, 2 ) position 92^93
-           location_from_column( Seq1/91-97, 10) position 97^98
-           location_from_column( Seq2/1-9, 1 )   position undef
+           location_from_column( 3 ) position 93
+           location_from_column( 2 ) position 92^93
+           location_from_column(10 ) position 97^98
+           location_from_column( 1 ) position undef
 
-           An exact position returns a Bio::Location::Simple
-           object, a position between bases returns a
-           Bio::Location::Fuzzy object where loc_type() returns
-           'BETWEEN'. Column before the first residue returns
-           undef. Note that if the position is after the last residue
-           in the alignment, that there is no guarantee that the
-           original sequence has residues after that position.
+           An exact position returns a Bio::Location::Simple object
+           where where location_type() returns 'EXACT', if a position
+           is between bases location_type() returns 'IN-BETWEEN'.
+           Column before the first residue returns undef. Note that if
+           the position is after the last residue in the alignment,
+           that there is no guarantee that the original sequence has
+           residues after that position.
 
            An exception is thrown if the column number is not within
            the sequence.
 
- Returns : Bio::Location::Simple or Bio::Location::Fuzzy object or
-           undef
+ Returns : Bio::Location::Simple or undef
  Args    : A column number
  Throws  : If column is not within the sequence
 
-See L<Bio::Location::Simple>, L<Bio::Location::Fuzzy> for more.
+See L<Bio::Location::Simple> for more.
 
 =cut
 
 sub location_from_column {
     my ($self, $column) = @_;
 
-    $self->throw("Column number has to be a positive integer, not [$column]") 
+    $self->throw("Column number has to be a positive integer, not [$column]")
 	unless $column =~ /^\d+$/ and $column > 0;
-    $self->throw("Column number [column] is larger than".
-		 " sequence length [". $self->length. "]") 
+    $self->throw("Column number [$column] is larger than".
+		 " sequence length [". $self->length. "]")
 	unless $column <= $self->length;
 
     my ($loc);
@@ -354,18 +364,27 @@ sub location_from_column {
     $s =~ s/\W//g;
     my $pos = CORE::length $s;
 
+    my $start = $self->start || 0 ;
+    my $strand = $self->strand() || 1;
+    my $relative_pos = ($strand == -1)
+        ? ($self->end - $pos + 1)
+            : ($pos + $start - 1);
     if ($self->subseq($column, $column) =~ /[a-zA-Z]/ ) {
-	$loc = new Bio::Location::Simple 
-	    (-start => $pos + $self->start - 1,
-	     -end => $pos + $self->start - 1,
-	     -strand => 1
+	$loc = new Bio::Location::Simple
+	    (-start => $relative_pos,
+	     -end => $relative_pos,
+	     -strand => 1,
 	     );
     }
     elsif ($pos == 0 and $self->start == 1) {
     } else {
-	$loc = new Bio::Location::Simple 
-	    (-start => $pos + $self->start - 1,
-	     -end => $pos +1 + $self->start - 1,
+      my ($start,$end) = ($relative_pos, $relative_pos + $strand);
+      if ($strand == -1) {
+	($start,$end) = ($end,$start);
+      }
+	$loc = new Bio::Location::Simple
+	    (-start => $start,
+	     -end => $end,
 	     -strand => 1,
 	     -location_type => 'IN-BETWEEN'
 	     );
@@ -373,20 +392,56 @@ sub location_from_column {
     return $loc;
 }
 
-sub trunc {
+=head2 revcom
 
+ Title   : revcom
+ Usage   : $rev = $seq->revcom()
+ Function: Produces a new Bio::LocatableSeq object which
+           has the reversed complement of the sequence. For protein
+           sequences this throws an exception of "Sequence is a
+           protein. Cannot revcom"
+
+ Returns : A new Bio::LocatableSeq object
+ Args    : none
+
+=cut
+
+sub revcom {
+    my ($self) = @_;
+
+    my $new = $self->SUPER::revcom;
+    $new->strand($self->strand * -1);
+    $new->start($self->start) if $self->start;
+    $new->end($self->end) if $self->end;
+    return $new;
+}
+
+
+=head2 trunc
+
+ Title   : trunc
+ Usage   : $subseq = $myseq->trunc(10,100);
+ Function: Provides a truncation of a sequence,
+
+ Example :
+ Returns : a fresh Bio::PrimarySeqI implementing object
+ Args    : Two integers denoting first and last columns of the
+           sequence to be included into sub-sequence.
+
+
+=cut
+
+sub trunc {
     my ($self, $start, $end) = @_;
     my $new = $self->SUPER::trunc($start, $end);
+    $new->strand($self->strand);
+
+    # end will be automatically calculated
+    $start = $end if $self->strand == -1;
 
     $start = $self->location_from_column($start);
-    $start = $start->start if $start;
-
-    $end = $self->location_from_column($end);
-    $end = $end->end if $end;
-
-    $new->strand($self->strand);
+    $start ? ($start = $start->end) : ($start = 1);
     $new->start($start) if $start;
-    $new->end($end) if $end;
 
     return $new;
 }
