@@ -1,4 +1,4 @@
-# $Id: Path.pm,v 1.7.4.1 2006/10/02 23:10:22 sendu Exp $
+# $Id: Path.pm,v 1.2 2003/03/26 09:08:55 lapp Exp $
 #
 # BioPerl module for Path
 #
@@ -22,7 +22,7 @@
 
 =head1 NAME
 
-Bio::Ontology::Path - a path for an ontology term graph
+Path - a path for an ontology term graph
 
 =head1 SYNOPSIS
 
@@ -47,16 +47,17 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to the 
 Bioperl mailing lists  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org                         - General discussion
+  http://bio.perl.org/MailList.html             - About the mailing lists
 
 =head2 Reporting Bugs
 
-Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via
-the web:
+report bugs to the Bioperl bug tracking system to help us keep track
+ the bugs and their resolution.  Bug reports can be submitted via
+ email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bio.perl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR
 
@@ -74,9 +75,13 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::Ontology::Path;
+use vars qw( @ISA );
 use strict;
+use Bio::Ontology::PathI;
+use Bio::Ontology::Relationship;
 
-use base qw(Bio::Ontology::Relationship Bio::Ontology::PathI);
+@ISA = qw( Bio::Ontology::Relationship
+           Bio::Ontology::PathI );
 
 
 
@@ -152,6 +157,7 @@ sub init {
  Returns : value of distance (a scalar)
  Args    : on set, new value (a scalar or undef, optional)
 
+
 =cut
 
 sub distance{
