@@ -1,4 +1,4 @@
-# $Id: RelationshipType.pm,v 1.5 2002/12/12 18:27:00 czmasek Exp $
+# $Id: RelationshipType.pm,v 1.11 2003/06/20 18:31:44 allenday Exp $
 #
 # BioPerl module for Bio::Ontology::RelationshipType  
 #
@@ -33,7 +33,9 @@ RelationshipType - a relationship type for an ontology
 This class can be used to model various types of relationships
 (such as "IS_A", "PART_OF", "CONTAINS", "FOUND_IN").
 
-This class extends Bio::Ontology::Term.
+This class extends L<Bio::Ontology::Term>, so it essentially is-a
+L<Bio::Ontology::TermI>. In addition, all methods are overridden such
+as to make the object immutable.
 
 =head1 FEEDBACK
 
@@ -53,7 +55,7 @@ report bugs to the Bioperl bug tracking system to help us keep track
  email or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR
 
@@ -80,7 +82,7 @@ methods. Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::Ontology::RelationshipType;
-use vars qw( @ISA $IS_A $PART_OF $CONTAINS $FOUND_IN );
+use vars qw( @ISA );
 use strict;
 use Bio::Ontology::Term;
 
@@ -94,11 +96,10 @@ use constant FOUND_IN => "FOUND_IN";
 @ISA = qw( Bio::Ontology::Term );
 
 
-$IS_A     = Bio::Ontology::RelationshipType->get_instance( IS_A );
-$PART_OF  = Bio::Ontology::RelationshipType->get_instance( PART_OF );
-$CONTAINS = Bio::Ontology::RelationshipType->get_instance( CONTAINS );
-$FOUND_IN = Bio::Ontology::RelationshipType->get_instance( FOUND_IN );
-
+#
+# cache for terms
+#
+my %term_name_map = ();
 
 
 =head2 get_instance
@@ -111,43 +112,43 @@ $FOUND_IN = Bio::Ontology::RelationshipType->get_instance( FOUND_IN );
  Function: Factory method to create instances of RelationshipType
  Returns : [Bio::Ontology::RelationshipType]
  Args    : "IS_A" or "PART_OF" or "CONTAINS" or "FOUND_IN" [scalar]
+           the ontology [Bio::Ontology::OntologyI] (optional)
 
 =cut
 
 sub get_instance {
-    my ( $class, $value ) = @_;
+    my ( $class, $name, $ont ) = @_;
 
-    my $instance = $class->new();
-    $instance->category( "relationship type" );
+    $class->throw("must provide predicate name") unless $name;
 
-    if ( $value eq IS_A ) {
-        $instance->identifier( IS_A );
-        $instance->name( IS_A );
-        $instance->definition( IS_A . " relationship type" );
-    }
-    elsif ( $value eq PART_OF ) {
-        $instance->identifier( PART_OF );
-        $instance->name( PART_OF );
-        $instance->definition( PART_OF . " relationship type" );
-    }
-    elsif ( $value eq CONTAINS ) {
-        $instance->identifier( CONTAINS );
-        $instance->name( CONTAINS );
-        $instance->definition( CONTAINS . " relationship type" );
-    }
-    elsif ( $value eq FOUND_IN ) {
-        $instance->identifier( FOUND_IN );
-        $instance->name( FOUND_IN );
-        $instance->definition( FOUND_IN . " relationship type" );
-    }
-    else {
-        my $msg = "Found unknown type of relationship: [" . $value . "]\n";
-        $msg .= "Known types are: [" . IS_A . "], [" . PART_OF . "], [" . CONTAINS . "], [" . FOUND_IN . "]";
-        $class->throw( $msg );
-    }
+    # is one in the cache?
+    my $reltype = $term_name_map{$name};
 
-    return $instance;
+    if($reltype &&
+       # check whether ontologies match
+       (($ont && $reltype->ontology() &&
+	 ($ont->name() eq $reltype->ontology->name())) ||
+	(! ($reltype->ontology() || $ont)))) {
+	# we're done, return cached type
+	return $reltype;
+    }
+    # valid relationship type?
 
+#
+#see the cell ontology.  this code is too strict, even for dag-edit files. -allen
+#
+#    if ( ! (($name eq IS_A) || ($name eq PART_OF) ||
+#	    ($name eq CONTAINS) || ( $name eq FOUND_IN ))) {
+#        my $msg = "Found unknown type of relationship: [" . $name . "]\n";
+#        $msg .= "Known types are: [" . IS_A . "], [" . PART_OF . "], [" . CONTAINS . "], [" . FOUND_IN . "]";
+#        $class->throw( $msg );
+#    }
+    # if we get here we need to create the rel.type
+    $reltype = $class->new(-name     => $name,
+			   -ontology => $ont);
+    # cache it (FIXME possibly overrides one from another ontology)
+    $term_name_map{$name} = $reltype;
+    return $reltype;
 } # get_instance
 
 
@@ -163,16 +164,11 @@ sub get_instance {
 =cut
 
 sub init {
-   my( $self ) = @_;
+    my $self = shift;
 
-    $self->{ "_identifier" }  = undef;
-    $self->{ "_name" }        = undef;
-    $self->{ "_definition" }  = undef;
-    $self->{ "_version" }     = undef;
-    $self->{ "_is_obsolete" } = undef;
-    $self->{ "_comment" }     = undef;
-    $self->remove_synonyms();
+    $self->SUPER::init();
 
+    # at this point we don't really need to do anything special for us
 } # init
 
 
@@ -182,7 +178,8 @@ sub init {
  Title   : equals
  Usage   : if ( $type->equals( $other_type ) ) { ...
  Function: Compares this type to another one, based on string "eq" of
-           the "identifier" field
+           the "identifier" field, if at least one of the two types has
+           the identifier set, or string eq of the name otherwise.
  Returns : true or false
  Args    : [Bio::Ontology::RelationshipType]
 
@@ -193,12 +190,16 @@ sub equals {
 
     $self->_check_class( $type, "Bio::Ontology::RelationshipType" );
 
-    unless ( $self->identifier() && $type->identifier() ) {
-        $self->throw( "Cannot compare RelationshipType with a undef identifier" );
+    if ( $self->identifier() xor $type->identifier() ) {
+        $self->warn("comparing relationship types when only ".
+		    "one has an identifier will always return false" );
     }
 
-    return( $self->identifier() eq $type->identifier() );
-
+    return
+ 	($self->identifier() || $type->identifier()) ?
+	$self->identifier() eq $type->identifier() :
+	$self->name() eq $type->name();
+	
 } # equals
 
 
@@ -216,17 +217,14 @@ sub equals {
 =cut
 
 sub identifier {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        if ( defined $self->{ "_identifier" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        $self->{ "_identifier" } = $value;
+    my $self = shift;
+    my $ret = $self->SUPER::identifier();
+    if(@_) {
+	$self->throw($self->veto_change("identifier",$ret,$_[0]))
+	    if $ret && ($ret ne $_[0]);
+	$ret = $self->SUPER::identifier(@_);
     }
-
-    return $self->{ "_identifier" };
-
+    return $ret;
 } # identifier
 
 
@@ -245,17 +243,14 @@ sub identifier {
 =cut
 
 sub name {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        if ( defined $self->{ "_name" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        $self->{ "_name" } = $value;
+    my $self = shift;
+    my $ret = $self->SUPER::name();
+    if(@_) {
+	$self->throw($self->veto_change("name",$ret,$_[0]))
+	    if $ret && ($ret ne $_[0]);
+	$ret = $self->SUPER::name(@_);
     }
-
-    return $self->{ "_name" };
-
+    return $ret;
 } # name
 
 
@@ -275,58 +270,45 @@ sub name {
 =cut
 
 sub definition {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        if ( defined $self->{ "_definition" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        $self->{ "_definition" } = $value;
+    my $self = shift;
+    my $ret = $self->SUPER::definition();
+    if(@_) {
+	$self->veto_change("definition",$ret,$_[0]) 
+	    if $ret && ($ret ne $_[0]);
+	$ret = $self->SUPER::definition(@_);
     }
-
-    return $self->{ "_definition" };
-
+    # let's be nice and return something readable here
+    return $ret if $ret;
+    return $self->name()." relationship predicate (type)" if $self->name();
 } # definition
 
 
 
-=head2 category
+=head2 ontology
 
- Title   : category
- Usage   : $term->category( $top );
+ Title   : ontology
+ Usage   : $term->ontology( $top );
            or
-           $top = $term->category();
- Function: Set/get for a immutable  relationship between this Type and
-           another Term (e.g. the top level of the ontology).
- Returns : The category [TermI].
- Args    : The category [TermI or scalar -- which
-           becomes the name of the catagory term] (optional).
+           $top = $term->ontology();
+ Function: Set/get for the ontology this relationship type lives in.
+ Returns : The ontology [Bio::Ontology::OntologyI].
+ Args    : On set, the ontology [Bio::Ontology::OntologyI] (optional).
 
 =cut
 
-sub category {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        if ( defined $self->{ "_category" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        if ( ! ref( $value ) ) {
-            my $term = $self->new();
-            $term->name( $value );
-            $self->{ "_category" } = $term;
-        }
-        elsif ( $value->isa( "Bio::Ontology::TermI" ) ) {
-            $self->{ "_category" } = $value;
-        }
-        else {
-            $self->throw( "Found [". ref( $value )
-            . "] where [Bio::Ontology::TermI] or [scalar] expected" );
-        }
+sub ontology {
+    my $self = shift;
+    my $ret = $self->SUPER::ontology();
+    if(@_) {
+	my $ont = shift;
+	if($ret) {
+	    $self->throw($self->veto_change("ontology",$ret->name,
+					    $ont ? $ont->name : $ont))
+		unless $ont && ($ont->name() eq $ret->name());
+	}
+	$ret = $self->SUPER::ontology($ont,@_);
     }
-
-    return $self->{ "_category" };
-
+    return $ret;
 } # category
 
 
@@ -344,17 +326,14 @@ sub category {
 =cut
 
 sub version {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        if ( defined $self->{ "_version" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        $self->{ "_version" } = $value;
+    my $self = shift;
+    my $ret = $self->SUPER::version();
+    if(@_) {
+	$self->throw($self->veto_change("version",$ret,$_[0]))
+	    if $ret && ($ret ne $_[0]);
+	$ret = $self->SUPER::version(@_);
     }
-
-    return $self->{ "_version" };
-
+    return $ret;
 } # version
 
 
@@ -372,24 +351,17 @@ sub version {
 =cut
 
 sub is_obsolete {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-            if ( defined $self->{ "_is_obsolete" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        $self->_is_true_or_false( $value );
-        $self->{ "_is_obsolete" } = $value;
+    my $self = shift;
+    my $ret = $self->SUPER::is_obsolete();
+    if(@_) {
+	$self->throw($self->veto_change("is_obsolete",$ret,$_[0]))
+	    if $ret && ($ret != $_[0]);
+	$ret = $self->SUPER::is_obsolete(@_);
     }
-
-    return $self->{ "_is_obsolete" };
-
+    return $ret;
 } # is_obsolete
 
 
-
-
-
 =head2 comment
 
  Title   : comment
@@ -403,20 +375,22 @@ sub is_obsolete {
 =cut
 
 sub comment {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        if ( defined $self->{ "_comment" } ) {
-            $self->throw( "Attempted to change field in immutable object" );
-        }
-        $self->{ "_comment" } = $value;
+    my $self = shift;
+    my $ret = $self->SUPER::comment();
+    if(@_) {
+	$self->throw($self->veto_change("comment",$ret,$_[0]))
+	    if $ret && ($ret ne $_[0]);
+	$ret = $self->SUPER::comment(@_);
     }
-
-    return $self->{ "_comment" };
-
+    return $ret;
 } # comment
 
+=head1 Private methods 
 
+May be overridden in a derived class, but should never be called from
+outside.
+
+=cut
 
 sub _check_class {
     my ( $self, $value, $expected_class ) = @_;
@@ -433,5 +407,33 @@ sub _check_class {
 
 } # _check_type
 
+=head2 veto_change
+
+ Title   : veto_change
+ Usage   :
+ Function: Called if an attribute is changed. Setting an attribute is
+           considered a change if it had a value before and the attempt
+           to set it would change the value.
+
+           This method returns the message to be printed in the exception.
+
+ Example :
+ Returns : A string
+ Args    : The name of the attribute that was attempted to change.
+           Optionally, the old value and the new value for reporting
+           purposes only.
+
+
+=cut
+
+sub veto_change{
+    my ($self,$attr,$old,$new) = @_;
+
+    my $changetype = $old ? ($new ? "change" : "unset") : "change";
+    my $msg = "attempt to $changetype attribute $attr in ".ref($self).
+    ", which is immutable";
+    $msg .= " (\"$old\" to \"$new\")" if $old && $new;
+    return $msg;
+}
 
 1;
