@@ -1,4 +1,4 @@
-# $Id: SimpleGOEngine.pm,v 1.3 2002/12/12 18:27:00 czmasek Exp $
+# $Id: SimpleGOEngine.pm,v 1.24 2003/10/23 01:52:42 allenday Exp $
 #
 # BioPerl module for Bio::Ontology::SimpleGOEngine
 #
@@ -26,13 +26,13 @@ SimpleGOEngine - a Ontology Engine for GO implementing OntologyEngineI
 
 =head1 SYNOPSIS
 
-  use Bio::Ontology::simpleGOparser;
+  use Bio::Ontology::SimpleGOEngine;
 
-  my $parser = Bio::Ontology::simpleGOparser->new
-	( -go_defs_file_name    => "/home/czmasek/GO/GO.defs",
-	  -components_file_name => "/home/czmasek/GO/component.ontology",
-	  -functions_file_name  => "/home/czmasek/GO/function.ontology",
-	  -processes_file_name  => "/home/czmasek/GO/process.ontology" );
+  my $parser = Bio::Ontology::SimpleGOEngine->new
+	( -defs_file => "/home/czmasek/GO/GO.defs",
+	  -files     => ["/home/czmasek/GO/component.ontology",
+	                 "/home/czmasek/GO/function.ontology",
+	                 "/home/czmasek/GO/process.ontology"] );
 
   my $engine = $parser->parse();
 
@@ -98,17 +98,20 @@ use vars qw( @ISA );
 use strict;
 use Bio::Root::Root;
 use Bio::Ontology::RelationshipType;
-use Bio::Ontology::Relationship;
+use Bio::Ontology::RelationshipFactory;
 use Bio::Ontology::OntologyEngineI;
+use Data::Dumper;
 
-use constant TRUE    => 1;
-use constant FALSE   => 0;
-use constant IS_A    => "IS_A";
-use constant PART_OF => "PART_OF";
-use constant TERM    => "TERM";
-use constant TYPE    => "TYPE";
+use constant TRUE     => 1;
+use constant FALSE    => 0;
+use constant IS_A     => "IS_A";
+use constant PART_OF  => "PART_OF";
+use constant TERM     => "TERM";
+use constant TYPE     => "TYPE";
+use constant ONTOLOGY => "ONTOLOGY";
 
-@ISA = qw( Bio::Ontology::OntologyEngineI );
+@ISA = qw( Bio::Root::Root
+           Bio::Ontology::OntologyEngineI );
 
 
 
@@ -152,6 +155,9 @@ sub init {
 
     $self->graph( Graph::Directed->new() );
 
+    # set defaults for the factories
+    $self->relationship_factory(Bio::Ontology::RelationshipFactory->new(
+				     -type => "Bio::Ontology::Relationship"));
 
 } # init
 
@@ -207,10 +213,11 @@ sub part_of_relationship {
 =head2 add_term
 
  Title   : add_term
- Usage   : $engine->add_term( $GOterm_obj );
- Function: Adds a Bio::Ontology::GOterm to this engine
- Returns : true
- Args    : Bio::Ontology::GOterm
+ Usage   : $engine->add_term( $term_obj );
+ Function: Adds a Bio::Ontology::TermI to this engine
+ Returns : true if the term was added and false otherwise (e.g., if the
+           term already existed in the ontology engine)
+ Args    : Bio::Ontology::TermI
 
 
 =cut
@@ -218,13 +225,9 @@ sub part_of_relationship {
 sub add_term {
     my ( $self, $term ) = @_;
 
-    $self->_check_class( $term, "Bio::Ontology::GOterm" );
+    return FALSE if $self->has_term( $term );
 
-    my $goid = $term->GO_id();
-
-    if ( $self->graph()->has_vertex( $term ) ) {
-        $self->throw( "Ontology already contains a GO term with an identifier of \"$goid\"" );
-    }
+    my $goid = $self->_get_id($term);
 
     $self->graph()->add_vertex( $goid );
     $self->graph()->set_attribute( TERM, $goid, $term );
@@ -239,20 +242,18 @@ sub add_term {
 
  Title   : has_term
  Usage   : $engine->has_term( $term );
- Function: Checks whether this engine contains a particular GO term
+ Function: Checks whether this engine contains a particular term
  Returns : true or false
- Args    : Bio::Ontology::GOterm
+ Args    : Bio::Ontology::TermI
            or
-           GO term identifier (e.g. "GO:0012345")
+           Term identifier (e.g. "GO:0012345")
 
 
 =cut
 
 sub has_term {
     my ( $self, $term ) = @_;
-
     $term = $self->_get_id( $term );
-
     if ( $self->graph()->has_vertex( $term ) ) {
         return TRUE;
     }
@@ -263,18 +264,58 @@ sub has_term {
 } # has_term
 
 
+=head2 add_relationship_type
+
+ Title   : add_relationship_type
+ Usage   : $engine->add_relationship_type( $type_name, $ont );
+ Function: Adds a new relationship type to the engine.  User get_relationship_type($type_name) to retrieve.
+ Returns : true if successfully added, false otherwise
+ Args    : relationship type name to add (scalar)
+
+
+=cut
+
+sub add_relationship_type{
+   my ($self,@args) = @_;
+
+   if(scalar(@_) == 3){
+	 my $type_name = $args[0];
+	 my $ont = $args[1];
+	 $self->{ "_extra_relationship_types" }{$type_name} = Bio::Ontology::RelationshipType->get_instance($type_name,$ont);
+#warn Dumper($self->{"_extra_relationship_types"}{$type_name});
+	 return 1;
+   }
+   return 0;
+}
+
+
+=head2 get_relationship_type
+
+ Title   : get_relationship_type
+ Usage   : $engine->get_relationship_type( $type_name );
+ Function: Gets a Bio::Ontology::RelationshipI object corresponding to $type_name
+ Returns : a Bio::Ontology::RelationshipI object
+ Args    :
+
+
+=cut
+
+sub get_relationship_type{
+   my ($self,$type_name) = @_;
+   return $self->{ "_extra_relationship_types" }{$type_name};
+}
 
 =head2 add_relationship
 
  Title   : add_relationship
  Usage   : $engine->add_relationship( $relationship );
-           $engine->add_relatioship( $parent_obj, $child_obj, $relationship_type );
-           $engine->add_relatioship( $parent_id, $child_id, $relationship_type);
+           $engine->add_relatioship( $subject_term, $predicate_term, $object_term, $ontology );
+           $engine->add_relatioship( $subject_id, $predicate_id, $object_id, $ontology);
  Function: Adds a relationship to this engine
  Returns : true if successfully added, false otherwise
- Args    : GO id, GO id, Bio::Ontology::RelationshipType
+ Args    : term id, Bio::Ontology::TermI (rel.type), term id, ontology 
            or
-           Bio::Ontology::GOterm, Bio::Ontology::GOterm, Bio::Ontology::RelationshipType
+           Bio::Ontology::TermI, Bio::Ontology::TermI (rel.type), Bio::Ontology::TermI, ontology
            or
            Bio::Ontology::RelationshipI
 
@@ -282,35 +323,35 @@ sub has_term {
 
 # term objs or term ids
 sub add_relationship {
-    my ( $self, $parent, $child, $type ) = @_;
+    my ( $self, $child, $type, $parent, $ont ) = @_;
 
     if ( scalar( @_ ) == 2 ) {
-        $self->_check_class( $parent, "Bio::Ontology::RelationshipI" );
-        $child = $parent->child_term();
-        $type = $parent->relationship_type();
-        $parent = $parent->parent_term();
+        $self->_check_class( $child, "Bio::Ontology::RelationshipI" );
+        $type   = $child->predicate_term();
+        $parent = $child->object_term();
+	$ont    = $child->ontology();
+        $child  = $child->subject_term();
     }
 
 
-    $self->_check_class( $type, "Bio::Ontology::RelationshipType" );
+    $self->_check_class( $type, "Bio::Ontology::TermI" );
 
-    $parent = $self->_get_id( $parent );
-    $child = $self->_get_id( $child );
+    my $parentid = $self->_get_id( $parent );
+    my $childid = $self->_get_id( $child );
 
-    if ( ! $self->graph()->has_vertex( $child ) ) {
-        $self->throw( "Ontology does not contain a GO term with an identifier of \"$child\"" );
-    }
-    if ( ! $self->graph()->has_vertex( $parent ) ) {
-        $self->throw( "Ontology does not contain a GO term with an identifier of \"$parent\"" );
-    }
+    my $g = $self->graph();
+
+    $self->add_term($child) unless $g->has_vertex( $childid );
+    $self->add_term($parent) unless $g->has_vertex( $parentid );
 
     # This prevents multi graphs.
-    if ( $self->graph()->has_edge( $parent, $child ) ) {
+    if ( $g->has_edge( $parentid, $childid ) ) {
         return FALSE;
     }
 
-    $self->graph()->add_edge( $parent, $child );
-    $self->graph()->set_attribute( TYPE, $parent, $child, $type );
+    $g->add_edge( $parentid, $childid );
+    $g->set_attribute( TYPE, $parentid, $childid, $type );
+    $g->set_attribute( ONTOLOGY, $parentid, $childid, $ont );
 
     return TRUE;
 
@@ -324,53 +365,73 @@ sub add_relationship {
 
  Title   : get_relationships
  Usage   : $engine->get_relationships( $term );
- Function: Returns all relationships of a term
+ Function: Returns all relationships of a term, or all relationships in
+           the graph if no term is specified.
  Returns : Relationship[]
- Args    : GO id
+ Args    : term id
            or
-           Bio::Ontology::GOterm
+           Bio::Ontology::TermI
 
 =cut
 
 sub get_relationships {
     my ( $self, $term ) = @_;
 
-    $term = $self->_get_id( $term );
+    my $g = $self->graph();
 
-    if ( ! $self->graph()->has_vertex( $term ) ) {
-        $self->throw( "Ontology does not contain a GO term with an identifier of \"$term\"" );
+    # obtain the ID if term provided
+    my $termid;
+    if($term) {
+	$termid = $self->_get_id( $term );
+	# check for presence in the graph
+	if ( ! $g->has_vertex( $termid ) ) {
+	    $self->throw( "no term with identifier \"$termid\" in ontology" );
+	}
     }
 
-    my @childs  = $self->get_child_terms( $term );
-    my @parents = $self->get_parent_terms( $term );
-
+    # now build the relationships
+    my $relfact = $self->relationship_factory();
+    # we'll build the relationships from edges
     my @rels = ();
-
-    foreach my $child ( @childs ) {
-        my $rel = Bio::Ontology::Relationship->new();
-        $rel->parent_term( $self->get_terms( $term ) );
-        $rel->child_term( $child );
-        $rel->relationship_type( $self->graph()->get_attribute( TYPE, $term, $child->GO_id() ) );
+    my @edges = $g->edges($termid);
+    while(@edges) {
+	my $startid = shift(@edges);
+	my $endid = shift(@edges);
+	my $rel = $relfact->create_object(
+		    -subject_term   => $self->get_terms($endid),
+                    -object_term    => $self->get_terms($startid),
+                    -predicate_term => $g->get_attribute(TYPE, 
+							 $startid, $endid),
+	            -ontology       => $g->get_attribute(ONTOLOGY, 
+							 $startid, $endid));
         push( @rels, $rel );
     }
-    foreach my $parent ( @parents ) {
-        my $rel = Bio::Ontology::Relationship->new();
-        $rel->parent_term( $parent );
-        $rel->child_term( $self->get_terms( $term ) );
-        $rel->relationship_type( $self->graph()->get_attribute( TYPE, $parent->GO_id(), $term ) );
-        push( @rels, $rel );
-    }
-
+    
     return @rels;
 
 } # get_relationships
 
+=head2 get_all_relationships
 
 
-=head2 get_relationship_types
+ Title   : get_all_relationships
+ Usage   : @rels = $engine->get_all_relationships();
+ Function: Returns all relationships in the graph.
+ Returns : Relationship[]
+ Args    : 
 
- Title   : get_relationship_types
- Usage   : $engine->get_relationship_types();
+=cut
+
+sub get_all_relationships {
+    return shift->get_relationships(@_);
+} # get_all_relationships
+
+
+
+=head2 get_predicate_terms
+
+ Title   : get_predicate_terms
+ Usage   : $engine->get_predicate_terms();
  Function: Returns the types of relationships this engine contains
  Returns : Bio::Ontology::RelationshipType[]
  Args    :
@@ -378,14 +439,18 @@ sub get_relationships {
 
 =cut
 
-sub get_relationship_types {
+sub get_predicate_terms {
     my ( $self ) = @_;
 
     my @a = ( $self->is_a_relationship(),
               $self->part_of_relationship() );
 
+	foreach my $termname (keys %{$self->{ "_extra_relationship_types" }}){
+	  push @a, $self->{ "_extra_relationship_types" }{ $termname };
+	}
+
     return @a;
-} # get_relationship_types
+} # get_predicate_terms
 
 
 
@@ -396,10 +461,10 @@ sub get_relationship_types {
  Usage   : $engine->get_child_terms( $term_obj, @rel_types );
            $engine->get_child_terms( $term_id, @rel_types );
  Function: Returns the children of this term
- Returns : Bio::Ontology::GOterm[]
- Args    : Bio::Ontology::GOterm, Bio::Ontology::RelationshipType[]
+ Returns : Bio::Ontology::TermI[]
+ Args    : Bio::Ontology::TermI, Bio::Ontology::RelationshipType[]
            or
-           GO id, Bio::Ontology::RelationshipType[]
+           term id, Bio::Ontology::RelationshipType[]
 
            if NO Bio::Ontology::RelationshipType[] is indicated: children
            of ALL types are returned
@@ -414,19 +479,16 @@ sub get_child_terms {
 } # get_child_terms
 
 
-
-
-
 =head2 get_descendant_terms
 
  Title   : get_descendant_terms
  Usage   : $engine->get_descendant_terms( $term_obj, @rel_types );
            $engine->get_descendant_terms( $term_id, @rel_types );
  Function: Returns the descendants of this term
- Returns : Bio::Ontology::GOterm[]
- Args    : Bio::Ontology::GOterm, Bio::Ontology::RelationshipType[]
+ Returns : Bio::Ontology::TermI[]
+ Args    : Bio::Ontology::TermI, Bio::Ontology::RelationshipType[]
            or
-           GO id, Bio::Ontology::RelationshipType[]
+           term id, Bio::Ontology::RelationshipType[]
 
            if NO Bio::Ontology::RelationshipType[] is indicated: descendants
            of ALL types are returned
@@ -442,7 +504,7 @@ sub get_descendant_terms {
     $term = $self->_get_id( $term );
 
     if ( ! $self->graph()->has_vertex( $term ) ) {
-        $self->throw( "Ontology does not contain a GO term with an identifier of \"$term\"" );
+        $self->throw( "Ontology does not contain a term with an identifier of \"$term\"" );
     }
 
     $self->_get_descendant_terms_helper( $term, \%ids, \@types );
@@ -464,10 +526,10 @@ sub get_descendant_terms {
  Usage   : $engine->get_parent_terms( $term_obj, @rel_types );
            $engine->get_parent_terms( $term_id, @rel_types );
  Function: Returns the parents of this term
- Returns : Bio::Ontology::GOterm[]
- Args    : Bio::Ontology::GOterm, Bio::Ontology::RelationshipType[]
+ Returns : Bio::Ontology::TermI[]
+ Args    : Bio::Ontology::TermI, Bio::Ontology::RelationshipType[]
            or
-           GO id, Bio::Ontology::RelationshipType[]
+           term id, Bio::Ontology::RelationshipType[]
 
            if NO Bio::Ontology::RelationshipType[] is indicated: parents
            of ALL types are returned
@@ -489,10 +551,10 @@ sub get_parent_terms {
  Usage   : $engine->get_ancestor_terms( $term_obj, @rel_types );
            $engine->get_ancestor_terms( $term_id, @rel_types );
  Function: Returns the ancestors of this term
- Returns : Bio::Ontology::GOterm[]
- Args    : Bio::Ontology::GOterm, Bio::Ontology::RelationshipType[]
+ Returns : Bio::Ontology::TermI[]
+ Args    : Bio::Ontology::TermI, Bio::Ontology::RelationshipType[]
            or
-           GO id, Bio::Ontology::RelationshipType[]
+           term id, Bio::Ontology::RelationshipType[]
 
            if NO Bio::Ontology::RelationshipType[] is indicated: ancestors
            of ALL types are returned
@@ -508,7 +570,7 @@ sub get_ancestor_terms {
     $term = $self->_get_id( $term );
 
     if ( ! $self->graph()->has_vertex( $term ) ) {
-        $self->throw( "Ontology does not contain a GO term with an identifier of \"$term\"" );
+        $self->throw( "Ontology does not contain a term with an identifier of \"$term\"" );
     }
 
     $self->_get_ancestor_terms_helper( $term, \%ids, \@types );
@@ -530,7 +592,7 @@ sub get_ancestor_terms {
  Title   : get_leaf_terms
  Usage   : $engine->get_leaf_terms();
  Function: Returns the leaf terms
- Returns : Bio::Ontology::GOterm[]
+ Returns : Bio::Ontology::TermI[]
  Args    :
 
 =cut
@@ -551,7 +613,7 @@ sub get_leaf_terms {
  Title   : get_root_terms
  Usage   : $engine->get_root_terms();
  Function: Returns the root terms
- Returns : Bio::Ontology::GOterm[]
+ Returns : Bio::Ontology::TermI[]
  Args    :
 
 =cut
@@ -567,36 +629,14 @@ sub get_root_terms {
 }
 
 
-
-=head2 get_term
-
- Title   : get_term
- Usage   : $engine->get_term( "GO:1234567" );
- Function: Returns a GO term with a given identifier
- Returns : Bio::Ontology::GOterm is present, false otherwise
- Args    : GO id
-
-
-=cut
-
-sub get_term {
-    my ( $self, $id ) = @_;
-    if ( $self->graph()->has_vertex( $id ) ) {
-        return( $self->graph()->get_attribute( TERM, $id ) );
-    }
-
-} # get_term
-
-
-
-
 =head2 get_terms
 
  Title   : get_terms
- Usage   : $engine->get_term( "GO:1234567", "GO:2234567" );
- Function: Returns a GO terms with given identifiers
- Returns : Bio::Ontology::GOterm[]
- Args    : GO id[]
+ Usage   : @terms = $engine->get_terms( "GO:1234567", "GO:2234567" );
+ Function: Returns term objects with given identifiers
+ Returns : Bio::Ontology::TermI[], or the term corresponding to the
+           first identifier if called in scalar context
+ Args    : term ids[]
 
 
 =cut
@@ -612,32 +652,117 @@ sub get_terms {
         }
     }
 
-    return @terms;
+    return wantarray ? @terms : shift(@terms);
 
 } # get_terms
 
 
+=head2 get_all_terms
 
-
-=head2 each_term
-
- Title   : each_term
- Usage   : $engine->each_term();
+ Title   : get_all_terms
+ Usage   : $engine->get_all_terms();
  Function: Returns all terms in this engine
- Returns : Bio::Ontology::GOterm[]
+ Returns : Bio::Ontology::TermI[]
  Args    :
 
 =cut
 
-sub each_term {
+sub get_all_terms {
     my ( $self ) = @_;
 
     return( $self->get_terms( $self->graph()->vertices() ) );
 
-} # each_term
+} # get_all_terms
 
 
+=head2 find_terms
 
+ Title   : find_terms
+ Usage   : ($term) = $oe->find_terms(-identifier => "SO:0000263");
+ Function: Find term instances matching queries for their attributes.
+
+           This implementation can efficiently resolve queries by
+           identifier.
+
+ Example :
+ Returns : an array of zero or more Bio::Ontology::TermI objects
+ Args    : Named parameters. The following parameters should be recognized
+           by any implementations:
+
+              -identifier    query by the given identifier
+              -name          query by the given name
+
+
+=cut
+
+sub find_terms{
+    my ($self,@args) = @_;
+    my @terms;
+
+    my ($id,$name) = $self->_rearrange([qw(IDENTIFIER NAME)],@args);
+
+    if(defined($id)) {
+	@terms = $self->get_terms($id);
+    } else {
+	@terms = $self->get_all_terms();
+    }
+    if(defined($name)) {
+	@terms = grep { $_->name() eq $name; } @terms;
+    }
+    return @terms;
+}
+
+=head2 relationship_factory
+
+ Title   : relationship_factory
+ Usage   : $fact = $obj->relationship_factory()
+ Function: Get/set the object factory to be used when relationship
+           objects are created by the implementation on-the-fly.
+
+ Example : 
+ Returns : value of relationship_factory (a Bio::Factory::ObjectFactoryI
+           compliant object)
+ Args    : on set, a Bio::Factory::ObjectFactoryI compliant object
+
+
+=cut
+
+sub relationship_factory{
+    my $self = shift;
+
+    return $self->{'relationship_factory'} = shift if @_;
+    return $self->{'relationship_factory'};
+}
+
+=head2 term_factory
+
+ Title   : term_factory
+ Usage   : $fact = $obj->term_factory()
+ Function: Get/set the object factory to be used when term objects are
+           created by the implementation on-the-fly.
+
+           Note that this ontology engine implementation does not
+           create term objects on the fly, and therefore setting this
+           attribute is meaningless.
+
+ Example : 
+ Returns : value of term_factory (a Bio::Factory::ObjectFactoryI
+           compliant object)
+ Args    : on set, a Bio::Factory::ObjectFactoryI compliant object
+
+
+=cut
+
+sub term_factory{
+    my $self = shift;
+
+    if(@_) {
+	$self->warn("setting term factory, but ".ref($self).
+		    " does not create terms on-the-fly");
+	return $self->{'term_factory'} = shift;
+    }
+    return $self->{'term_factory'};
+}
 
 =head2 graph
 
@@ -666,29 +791,40 @@ sub graph {
 # ----------------
 
 
-# Checks the correct format of a GO id
-# Gets the GO out of a GOterm
+# Checks the correct format of a GOBO-formatted id
+# Gets the id out of a term or id string
 sub _get_id {
     my ( $self, $term ) = @_;
-    if ( ref( $term ) ) {
-        if ( $term->isa( "Bio::Ontology::GOterm" ) ) {
-            return $term->GO_id();
-        }
-        else {
-            $self->throw( "Found [" . ref( $term ) . "] where [Bio::Ontology::GOterm] expected" );
-        }
-    }
-    else {
-        if ( $term =~ /^GO:\d{7}$/ ) {
-            return $term;
-        }
-        elsif ( $term =~ /^\d{7}$/ ) {
-            return "GO:" . $term;
-        }
-        else {
-            $self->throw( "Illegal format for GO identifier [$term]" );
-        }
+    my $id = $term;
+
+    if(ref($term)) {
+	# use TermI standard API
+	$self->throw("Object doesn't implement Bio::Ontology::TermI. ".
+		     "Bummer.")
+	    unless $term->isa("Bio::Ontology::TermI");
+	$id = $term->identifier();
+	# if there is no ID, we need to fake one from ontology name and name
+	# in order to achieve uniqueness
+	if(!$id) {
+	    $id = $term->ontology->name() if $term->ontology();
+	    $id = $id ? $id.'|' : '';
+	    $id .= $term->name();
+	}
     }
+    # don't fuss if it looks remotely standard, and we trust GO terms
+    return $id
+#	if $term->isa("Bio::Ontology::GOterm")||($id =~ /^[A-Z_]{1,8}:\d{1,}$/);
+	if $term->isa("Bio::Ontology::GOterm")||($id =~ /^\w+:\w+$/);
+    # prefix with something if only numbers
+    if($id =~ /^\d+$/) {
+	$self->warn(ref($self).": identifier [$id] is only numbers - ".
+		    "prefixing with 'GO:'");
+	return "GO:" . $id;
+    } 
+    # we shouldn't have gotten here if it's at least a remotely decent ID
+    $self->throw(ref($self).": non-standard identifier '$id'\n")
+	unless $id =~ /\|/;
+    return $id;
 } # _get_id
 
 
@@ -697,14 +833,14 @@ sub _get_child_parent_terms_helper {
     my ( $self, $term, $do_get_child_terms, @types ) = @_;
 
     foreach my $type ( @types ) {
-        $self->_check_class( $type, "Bio::Ontology::RelationshipType" );
+        $self->_check_class( $type, "Bio::Ontology::TermI" );
     }
 
     my @relative_terms = ();
 
     $term = $self->_get_id( $term );
     if ( ! $self->graph()->has_vertex( $term ) ) {
-        $self->throw( "Ontology does not contain a GO term with an identifier of \"$term\"" );
+        $self->throw( "Ontology does not contain a term with an identifier of \"$term\"" );
     }
 
     my @all_relative_terms = ();
@@ -751,7 +887,7 @@ sub _get_descendant_terms_helper {
     }
 
     foreach my $child_term ( @child_terms ) {
-        my $child_term_id = $child_term->GO_id();
+        my $child_term_id = $self->_get_id($child_term->identifier());
         $ids_ref->{ $child_term_id } = 0;
         $self->_get_descendant_terms_helper( $child_term_id, $ids_ref, $types_ref );
     }
@@ -770,7 +906,7 @@ sub _get_ancestor_terms_helper {
     }
 
     foreach my $parent_term ( @parent_terms ) {
-        my $parent_term_id = $parent_term->GO_id();
+        my $parent_term_id = $self->_get_id($parent_term->identifier());
         $ids_ref->{ $parent_term_id } = 0;
         $self->_get_ancestor_terms_helper( $parent_term_id, $ids_ref, $types_ref );
     }
@@ -795,4 +931,11 @@ sub _check_class {
 } # _check_class
 
 
+#################################################################
+# aliases
+#################################################################
+
+*get_relationship_types = \&get_predicate_terms;
+
+
 1;
