@@ -1,4 +1,4 @@
-# $Id: MiniMIMentry.pm,v 1.9.4.1 2006/10/02 23:10:22 sendu Exp $
+# $Id: MiniMIMentry.pm,v 1.5 2002/12/12 18:27:01 czmasek Exp $
 #
 # BioPerl module for Bio::Phenotype::OMIM::MiniMIMentry
 #
@@ -20,9 +20,10 @@
 
 # POD documentation - main docs before the code
 
+
 =head1 NAME
 
-Bio::Phenotype::OMIM::MiniMIMentry - Representation of a Mini MIM entry
+MiniMIMentry - Representation of a Mini MIM entry
 
 =head1 SYNOPSIS
 
@@ -48,16 +49,17 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org             - General discussion
+  http://bio.perl.org/MailList.html - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via the
-web:
+the bugs and their resolution.  Bug reports can be submitted via email
+or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bio.perl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR
 
@@ -84,9 +86,14 @@ methods.
 # Let the code begin...
 
 package Bio::Phenotype::OMIM::MiniMIMentry;
+use vars qw( @ISA );
 use strict;
+use Bio::Root::Object;
+
+@ISA = qw( Bio::Root::Root );
+
+
 
-use base qw(Bio::Root::Root);
 
 =head2 new
 
