@@ -1,4 +1,4 @@
-# $Id$
+# $Id: OMIMentry.pm,v 1.10 2003/11/05 03:27:17 juguang Exp $
 #
 # BioPerl module for Bio::Phenotype::OMIM::OMIMentry
 #
@@ -180,7 +180,7 @@ sub new {
     $edited                         && $self->edited( $edited );    
     $contributors                   && $self->contributors( $contributors );
     $additional_references          && $self->additional_references( $additional_references );     
-    $clinical_symptoms              && $self->clinical_symptoms( $clinical_symptoms );
+    $clinical_symptoms              && $self->clinical_symptoms_raw( $clinical_symptoms );
     $miniMIM                        && $self->miniMIM( $miniMIM );
                                                     
     return $self;
@@ -219,7 +219,7 @@ sub init {
     $self->edited( "" );
     $self->contributors( "" );
     $self->additional_references( "" );
-    $self->clinical_symptoms( "" );
+    $self->clinical_symptoms( {} );
     $self->remove_Correlates();
     $self->remove_References();
     $self->remove_AllelicVariants();
@@ -337,9 +337,9 @@ sub MIM_number {
 =cut
 
 sub title {
-    my ( $self, $value ) = @_;
-
-    $self->name( $value );
+    my $self = shift;
+    
+    $self->name(@_);
     
 } # title
 
@@ -451,19 +451,11 @@ sub is_separate {
 =cut
 
 sub mapping_method {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        $self->{ "_mapping_method" } = $value;
-    }
-
+    my $self = shift;
+    return $self->{ "_mapping_method" } = shift if(@_);
     return $self->{ "_mapping_method" };
-
 } # mapping_method
 
-
-
-
 =head2 gene_status
 
  Title   : gene_status
@@ -516,17 +508,13 @@ sub gene_status {
 } # gene_status
 
 
-
-
 =head2 clinical_symptoms
 
  Title   : clinical_symptoms
- Usage   : $omim->clinical_symptoms( "Patients with ..." );
-           or
-           print $omim->clinical_symptoms();
+ Usage   : $omim->clinical_symptoms({});
  Function: Set/get for the clinical symptoms of this OMIM entry.
- Returns : The clinical symptoms [scalar].
- Args    : The clinical symptoms [scalar] (optional).
+ Returns : [hash reference].
+ Args    : [hash reference]. Suggested not to assign alone. Parser will do.
 
 =cut
 
@@ -534,6 +522,9 @@ sub clinical_symptoms {
     my ( $self, $value ) = @_;
 
     if ( defined $value ) {
+        unless(ref($value) eq 'HASH'){
+            $self->throw('a hash referenced needed');
+        }
         $self->{ "_clinical_symptoms" } = $value;
     }
 
@@ -541,8 +532,55 @@ sub clinical_symptoms {
 
 } # clinical_symptoms
 
+=head2 clinical_symptoms_raw
 
+  Title     : clinical_symptoms_raw
+  Usage     : $omim->clinical_symptoms( "Patients with ..." );
+              print $omim->clinical_symptoms();
+  Functions : Get/set for text information of clinical symptoms
+  Returns   : The clinical symptoms [scalar].
+  Args      : The clinical symptoms [scalar] (optional).
 
+=cut 
+
+sub clinical_symptoms_raw {
+    my $self = shift;
+    return $self->{_clinical_symptoms_raw} = shift if @_;
+    return $self->{_clinical_symptoms_raw};
+}
+
+=head2 add_clinical_symptoms
+
+  Title     : add_clinical_symptoms
+  Usage     : $entry->add_clinical_symptoms('Ears', 'Floppy ears', 'Lop-ears');
+  Function  : add one or more symptoms on one part of body.
+  Returns   : [none]
+  Args      : ($part, @symptoms)
+              $part, the text name of part/organism of human
+              @symptoms, an array of text description
+
+=cut
+
+sub add_clinical_symptoms {
+    my ($self, $part, @symptoms) = @_;
+    unless(defined $part){
+        $self->throw('a part/organism must be assigned');
+    }
+    $self->{_clinical_symptoms} = {} unless $self->{_clinical_symptoms};
+    $self->{_clinical_symptoms}->{$part} = [] 
+        unless $self->{_clinical_symptoms}->{$part};
+    push @{$self->{_clinical_symptoms}->{$part}}, @symptoms;
+}
+
+=head2 get_clinical_symptoms
+
+  Title     : get_clinical_symptoms
+  Usage     : @symptoms = $self->get_clinical_symptoms('Ears');
+  Function  : get all symptoms specific to one part/organism.
+  Returns   : an array of text
+  Args      : $part
+
+=cut
 
 
 =head2 created
@@ -558,12 +596,8 @@ sub clinical_symptoms {
 =cut
 
 sub created {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        $self->{ "_created" } = $value;
-    }
-
+    my $self = shift;
+    return $self->{ "_created" } = shift if(@_);
     return $self->{ "_created" };
 
 } # created
@@ -584,12 +618,8 @@ sub created {
 =cut
 
 sub contributors {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        $self->{ "_contributors" } = $value;
-    }
-
+    my  $self = shift;
+    $self->{ "_contributors" } = shift if(@_);
     return $self->{ "_contributors" };
 
 } # contributors
@@ -610,12 +640,8 @@ sub contributors {
 =cut
 
 sub edited {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        $self->{ "_edited" } = $value;
-    }
-
+    my $self = shift;
+    return $self->{ "_edited" } = shift if(@_);
     return $self->{ "_edited" };
 
 } # edited
@@ -637,19 +663,12 @@ sub edited {
 =cut
 
 sub additional_references {
-    my ( $self, $value ) = @_;
-
-    if ( defined $value ) {
-        $self->{ "_additional_references" } = $value;
-    }
-
+    my $self = shift;
+    return $self->{ "_additional_references" } = shift if(@_);
     return $self->{ "_additional_references" };
 
 } # additional_references
 
-
-
-
 =head2 miniMIM
 
  Title   : miniMIM
@@ -675,9 +694,6 @@ sub miniMIM {
     return $self->{ "_mini_mim" };
 }
 
-
-
-
 =head2 each_AllelicVariant
 
  Title   : each_AllelicVariant()
@@ -750,9 +766,6 @@ sub remove_AllelicVariants {
 } # remove_AllelicVariants
 
 
-
-
-
 # Title   : _array_to_string         
 # Function:
 # Returns : 
