@@ -1,4 +1,4 @@
-# $Id: Statistics.pm,v 1.34.4.1 2006/10/02 23:10:23 sendu Exp $
+# $Id: Statistics.pm,v 1.16 2003/12/06 18:08:46 jason Exp $
 #
 # BioPerl module for Bio::PopGen::Statistics
 #
@@ -25,12 +25,12 @@ Bio::PopGen::Statistics - Population Genetics statistical tests
 
   my $tree = $sim->next_tree;
 
-  $sim->add_Mutations($tree,20);
+  $factory->add_Mutations($tree,20);
 
   my $stats = new Bio::PopGen::Statistics();
   my $individuals = [ $tree->get_leaf_nodes];
   my $pi = $stats->pi($individuals);
-  my $D  = $stats->tajima_D($individuals);
+  my $D  = $stats->tajima_d($individuals);
 
   # Alternatively to do this on input data from
   # See the tests in t/PopGen.t for more examples
@@ -40,26 +40,15 @@ Bio::PopGen::Statistics - Population Genetics statistical tests
   # Note that you can also call the stats as a class method if you like
   # the only reason to instantiate it (as above) is if you want
   # to set the verbosity for debugging
-  $pi     = Bio::PopGen::Statistics->pi($pop);
+  $pi     = Bio::PopGen::Statistics->pi($pop);   
   $theta  = Bio::PopGen::Statistics->theta($pop);
 
   # Pi and Theta also take additional arguments,
-  # see the documentation for more information
+  # see the documentation for more information  
 
-  use Bio::PopGen::Utilities;
-  use Bio::AlignIO;
-
-  my $in = new Bio::AlignIO(-file   => 't/data/t7.aln',
-                            -format => 'clustalw');
-  my $aln = $in->next_aln;
-  # get a population, each sequence is an individual and 
-  # for the default case, every site which is not monomorphic
-  # is a 'marker'.  Each individual will have a 'genotype' for the
-  # site which will be the specific base in the alignment at that
-  # site
-
-  my $pop = Bio::PopGen::Utilities->aln_to_population(-alignment => $aln);
 
+  # To come -- examples for creating pops/individuals from
+  # Aligned sequence data
 
 =head1 DESCRIPTION
 
@@ -74,16 +63,13 @@ aligned sequence data given that you can calculate alleles, number of
 segregating sites.
 
 Currently implemented:
- Fu and Li's D    (fu_and_li_D)
- Fu and Li's D*   (fu_and_li_D_star)
- Fu and Li's F    (fu_and_li_F)
- Fu and Li's F*   (fu_and_li_F_star)
- Tajima's D       (tajima_D)
- Watterson's theta (theta)
- pi               (pi) - number of pairwise differences
- composite_LD     (composite_LD)
-
-Count based methods also exist in case you have already calculated the key statistics (seg sites, num individuals, etc) and just want to compute the statistic.
+ Fu and Li's D  (fu_and_li_D)
+ Fu and Li's D* (fu_and_li_D_star)
+ Fu and Li's F  (fu_and_li_F)
+ Tajima's D     (tajima_D)
+ theta          (theta)
+ pi             (pi) - number of pairwise differences
+ composite_LD   (composite_LD)
 
 In all cases where a the method expects an arrayref of
 L<Bio::PopGen::IndividualI> objects and L<Bio::PopGen::PopulationI>
@@ -100,15 +86,6 @@ from a Population." Genetics 143:557-570.
 Tajima F. (1989) "Statistical method for testing the neutral mutation
 hypothesis by DNA polymorphism." Genetics 123:585-595.
 
-=head2 CITING THIS WORK
-
-Please see this reference for use of this implementation.
-
-Stajich JE and Hahn MW "Disentangling the Effects of Demography and Selection in Human History." (2005) Mol Biol Evol 22(1):63-73. 
-
-If you use these Bio::PopGen modules please cite the Bioperl
-publication (see FAQ) and the above reference.
-
 
 =head1 FEEDBACK
 
@@ -118,8 +95,8 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org              - General discussion
+  http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
@@ -127,12 +104,16 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 the web:
 
-  http://bugzilla.open-bio.org/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich, Matthew Hahn
 
 Email jason-at-bioperl-dot-org
-Email matthew-dot-hahn-at-duke-dot-edu
+Matt Hahn E<lt>matthew.hahn-at-duke.dukeE<gt>
+
+=head1 CONTRIBUTORS
+
+Additional contributors names and emails here
 
 =head1 APPENDIX
 
@@ -146,10 +127,12 @@ Internal methods are usually preceded with a _
 
 
 package Bio::PopGen::Statistics;
+use vars qw(@ISA);
 use strict;
 
+use Bio::Root::Root;
 
-use base qw(Bio::Root::Root);
+@ISA = qw(Bio::Root::Root );
 
 =head2 new
 
@@ -166,12 +149,12 @@ use base qw(Bio::Root::Root);
 =head2 fu_and_li_D
 
  Title   : fu_and_li_D
- Usage   : my $D = $statistics->fu_and_li_D(\@ingroup,$extmutations);
+ Usage   : my $D = $statistics->fu_an_li_D(\@ingroup,$extmutations);
  Function: Fu and Li D statistic for a list of individuals
            given an outgroup and the number of external mutations
            (either provided or calculated from list of outgroup individuals)
  Returns : decimal
- Args    : $individuals - array reference which contains ingroup individuals 
+ Args    : $individuals - array refence which contains ingroup individuals 
            (L<Bio::PopGen::Individual> or derived classes)
            $extmutations - number of external mutations OR
            arrayref of outgroup individuals
@@ -181,14 +164,16 @@ use base qw(Bio::Root::Root);
 sub fu_and_li_D { 
     my ($self,$ingroup,$outgroup) = @_;
 
-    my ($seg_sites,$n,$ancestral,$derived) = (0,0,0,0);
+    my ($seg_sites,$pi,$sample_size,$ancestral,$derived);
     if( ref($ingroup) =~ /ARRAY/i ) {
-	$n = scalar @$ingroup;
+	$sample_size = scalar @$ingroup;
 	# pi - all pairwise differences 
+	$pi          = $self->pi($ingroup);  
 	$seg_sites   = $self->segregating_sites_count($ingroup);
     } elsif( ref($ingroup) && 
 	     $ingroup->isa('Bio::PopGen::PopulationI')) {
-	$n = $ingroup->get_number_individuals;
+	$sample_size = $ingroup->get_number_individuals;
+	$pi          = $self->pi($ingroup);
 	$seg_sites   = $self->segregating_sites_count($ingroup);
     } else { 
 	$self->throw("expected an array reference of a list of Bio::PopGen::IndividualI OR a Bio::PopGen::PopulationI object to fu_and_li_D");
@@ -205,56 +190,32 @@ sub fu_and_li_D {
 	return 0;
     } elsif( ref($outgroup) ) {
 	($ancestral,$derived) = $self->derived_mutations($ingroup,$outgroup);
-	$ancestral = 0 unless defined $ancestral;
     } else { 
 	$ancestral = $outgroup;
     }
-   
-    return $self->fu_and_li_D_counts($n,$seg_sites,
-				     $ancestral,$derived);
-}
-
-=head2 fu_and_li_D_counts
-
- Title   : fu_li_D_counts
- Usage   : my $D = $statistics->fu_and_li_D_counts($samps,$sites,
-                                                   $external);
- Function: Fu and Li D statistic for the raw counts of the number
-           of samples, sites, external and internal mutations
- Returns : decimal number
- Args    : number of samples (N)
-           number of segregating sites (n)
-           number of external mutations (n_e)
-
-=cut
-
-
-sub fu_and_li_D_counts {
-    my ($self,$n,$seg_sites, $external_mut) = @_;
-    my $a_n = 0;
-    for(my $k= 1; $k < $n; $k++ ) {
-	$a_n += ( 1 / $k );
+    my $a = 0;
+    for(my $k= 1; $k < $sample_size; $k++ ) {
+        $a += ( 1 / $k );
     }
+
     my $b = 0;
-    for(my $k= 1; $k < $n; $k++ ) {
+    for(my $k= 1; $k < $sample_size; $k++ ) {
         $b += ( 1 / $k**2 );
     }
 
-    my $c = 2 * ( ( ( $n * $a_n ) - (2 * ( $n -1 ))) /
-                  ( ( $n - 1) * ( $n - 2 ) ) );
+    my $c = 2 * ( ( ( $sample_size * $a ) - (2 * ( $sample_size -1 ))) /
+                  ( ( $sample_size - 1) * ( $sample_size - 2 ) ) );
 
-    my $v = 1 + ( ( $a_n**2 / ( $b + $a_n**2 ) ) * 
-		  ( $c - ( ( $n + 1) /
-			   ( $n - 1) ) ));
-    
-    my $u = $a_n - 1 - $v;
+    my $v = 1 + ( ( $a**2 / ( $b + $a**2 ) ) * ( $c - ( ( $sample_size + 1) /
+                                                        ( $sample_size - 1) ) ));
 
-    ($seg_sites - $a_n * $external_mut) / 
-	sqrt( ($u * $seg_sites) + ($v * $seg_sites*$seg_sites));
-    
+    my $u = $a - 1 - $v;
+    my $D = ( $seg_sites - (  $a * $ancestral) ) /
+	( sqrt ( ($u * $seg_sites ) + ( $v * $seg_sites **2) ) );
+
+    return $D;
 }
 
-
 =head2 fu_and_li_D_star
 
  Title   : fu_and_li_D_star
@@ -274,15 +235,18 @@ sub fu_and_li_D_counts {
 sub fu_and_li_D_star {
     my ($self,$individuals) = @_;
 
-    my ($seg_sites,$n,$singletons);
+    my ($seg_sites,$pi,$sample_size,$singletons);
     if( ref($individuals) =~ /ARRAY/i ) {
-	$n = scalar @$individuals;
+	$sample_size = scalar @$individuals;
+	# pi - all pairwise differences 
+	$pi          = $self->pi($individuals);  
 	$seg_sites   = $self->segregating_sites_count($individuals);
 	$singletons  = $self->singleton_count($individuals);
     } elsif( ref($individuals) && 
 	     $individuals->isa('Bio::PopGen::PopulationI')) {
 	my $pop = $individuals;
-	$n = $pop->get_number_individuals;
+	$sample_size = $pop->get_number_individuals;
+	$pi          = $self->pi($pop);
 	$seg_sites   = $self->segregating_sites_count($pop);
 	$singletons  = $self->singleton_count($pop);
     } else { 
@@ -290,66 +254,47 @@ sub fu_and_li_D_star {
 	return 0;
     }
 
-    return $self->fu_and_li_D_star_counts($n,$seg_sites, $singletons);
-}
-
-=head2 fu_and_li_D_star_counts
-
- Title   : fu_li_D_star_counts
- Usage   : my $D = $statistics->fu_and_li_D_star_counts($samps,$sites,
-                                                        $singletons);
-
- Function: Fu and Li D statistic for the raw counts of the number
-           of samples, sites, external and internal mutations
- Returns : decimal number
- Args    : number of samples (N)
-           number of segregating sites (n)
-           singletons (n_s)
-
-=cut
-
-
-sub fu_and_li_D_star_counts {
-    my ($self,$n,$seg_sites, $singletons) = @_;
-    my $a_n;
-    for(my $k = 1; $k < $n; $k++ ) {
-	$a_n += ( 1 / $k );
+    my $a = 0;
+    for(my $k= 1; $k < $sample_size; $k++ ) {
+	$a += ( 1 / $k );
     }
 
-    my $a1 = $a_n + 1 / $n;
+    my $a1 = 0;
+    for(my $k= 1; $k <= $sample_size; $k++ ) {
+	$a1 += ( 1 / $k );
+    }
 
     my $b = 0;
-    for(my $k= 1; $k < $n; $k++ ) {
-        $b += ( 1 / $k**2 );
+    for(my $k= 1; $k < $sample_size; $k++ ) {
+	$b += ( 1 / $k**2 );
     }
 
-    my $c = 2 * ( ( ( $n * $a_n ) - (2 * ( $n -1 ))) /
-                  ( ( $n - 1) * ( $n - 2 ) ) );
+    my $c = 2 * ( ( ( $sample_size * $a ) - (2 * ( $sample_size -1 ))) / 
+		  ( ( $sample_size - 1) * ( $sample_size - 2 ) ) );
 
-    my $d = $c + ($n -2) / ($n - 1)**2 +
-	2 / ($n -1) * 
-	( 1.5 - ( (2*$a1 - 3) / ($n -2) ) - 
-	  1 / $n ); 
-    
-    my $v_star = ( ( ($n/($n-1) )**2)*$b + (($a_n**2)*$d) -
-		 (2*( ($n*$a_n*($a_n+1)) )/(($n-1)**2)) )  /
-		   (($a_n**2) + $b);
+    my $d = $c + ( ($sample_size -2) / ($sample_size - 1)**2 ) +
+	( 2 / ($sample_size -1) * 
+	  ( (3/2) - ( (2*$a1 - 3) / ($sample_size -2) ) - 
+	    ( 1/ $sample_size) ) 
+	  );
+    my $v_star = ( ( ($sample_size/($sample_size-1) )**2)*$b + (($a**2)*$d) -
+		 (2*( ($sample_size*$a*($a+1)) )/(($sample_size-1)**2)) )  /
+		   (($a**2) + $b);
 
-    my $u_star = ( ($n/($n-1))*
-		   ($a_n - ($n/
-			  ($n-1)))) - $v_star;
+    my $u_star = ( ($sample_size/($sample_size-1))*
+		   ($a - ($sample_size/
+			  ($sample_size-1)))) - $v_star;
 
-
-    return (($n / ($n - 1)) * $seg_sites - 
-	    $a_n * $singletons) / 
-	    sqrt( ($u_star * $seg_sites) + ($v_star * $seg_sites*$seg_sites));
+    my $D_star = ( (($sample_size/($sample_size-1))*$seg_sites) -
+		   ($a*$singletons) ) / 
+		   ( sqrt( ($u_star*$seg_sites) + ($v_star*($seg_sites**2)) ));
+    return $D_star;
 }
 
-
 =head2 fu_and_li_F
 
  Title   : fu_and_li_F
- Usage   : my $F = Bio::PopGen::Statistics->fu_and_li_F(\@ingroup,$ext_muts);
+ Usage   : my $D = Bio::PopGen::Statistics->fu_and_li_F(\@ingroup,$ext_muts);
  Function: Calculate Fu and Li's F on an ingroup with either the set of 
            outgroup individuals, or the number of external mutations
  Returns : decimal number
@@ -363,19 +308,19 @@ sub fu_and_li_D_star_counts {
 
 sub fu_and_li_F {
     my ($self,$ingroup,$outgroup) = @_;
-    my ($seg_sites,$pi,$n,$external,$internal);
+    my ($seg_sites,$pi,$sample_size,$external,$internal);
     if( ref($ingroup) =~ /ARRAY/i ) {
-	$n = scalar @$ingroup;
+	$sample_size = scalar @$ingroup;
 	# pi - all pairwise differences 
 	$pi          = $self->pi($ingroup);  
 	$seg_sites   = $self->segregating_sites_count($ingroup);
     } elsif( ref($ingroup) && 
 	     $ingroup->isa('Bio::PopGen::PopulationI')) {
-	$n = $ingroup->get_number_individuals;
+	$sample_size = $ingroup->get_number_individuals;
 	$pi          = $self->pi($ingroup);
 	$seg_sites   = $self->segregating_sites_count($ingroup);
     } else { 
-	$self->throw("expected an array reference of a list of Bio::PopGen::IndividualI OR a Bio::PopGen::PopulationI object to Fu and Li's F");
+	$self->throw("expected an array reference of a list of Bio::PopGen::IndividualI OR a Bio::PopGen::PopulationI object to tajima_D");
 	return 0;
     }
     
@@ -387,53 +332,34 @@ sub fu_and_li_F {
     } else { 
 	$external = $outgroup;
     }
-    $self->fu_and_li_F_counts($n,$pi,$seg_sites,$external);
-}
-
-=head2 fu_and_li_F_counts
-
- Title   : fu_li_F_counts
- Usage   : my $F = $statistics->fu_and_li_F_counts($samps,$pi,
-                                                   $sites,
-                                                   $external);
- Function: Fu and Li F statistic for the raw counts of the number
-           of samples, sites, external and internal mutations
- Returns : decimal number
- Args    : number of samples (N)
-           average pairwise differences (pi)
-           number of segregating sites (n)
-           external mutations (n_e)
-
-=cut
-
-
-sub fu_and_li_F_counts {
-    my ($self,$n,$pi,$seg_sites, $external) = @_;
-    my $a_n = 0;
-    for(my $k= 1; $k < $n; $k++ ) {
-	$a_n += ( 1 / $k );
+    
+    my $a = 0;
+    for(my $k= 1; $k < $sample_size; $k++ ) {
+	$a += ( 1 / $k );
     }
 
-    my $a1 = $a_n + (1 / $n );
+    my $a1 = 0;
+    for(my $k= 1; $k <= $sample_size; $k++ ) {
+	$a1 += ( 1 / $k );
+    }
 
     my $b = 0;
-    for(my $k= 1; $k < $n; $k++ ) {
+    for(my $k= 1; $k < $sample_size; $k++ ) {
 	$b += ( 1 / $k**2 );
     }
 
-    my $c = 2 * ( ( ( $n * $a_n ) - (2 * ( $n -1 ))) / 
-		  ( ( $n - 1) * ( $n - 2 ) ) );
+    my $c = 2 * ( ( ( $sample_size * $a ) - (2 * ( $sample_size -1 ))) / 
+		  ( ( $sample_size - 1) * ( $sample_size - 2 ) ) );
 
-    my $v_F = ( $c + ( (2*(($n**2)+$n+3)) / 
-		       ( (9*$n)*($n-1) ) ) -
-		(2/($n-1)) ) / ( ($a_n**2)+$b );
+    my $v_F = ( $c + ( (2*(($sample_size**2)+$sample_size+3)) / 
+		       ( (9*$sample_size)*($sample_size-1) ) ) -
+		(2/($sample_size-1)) ) / ( ($a**2)+$b );
 
-    my $u_F = ( 1 + ( ($n+1)/(3*($n-1)) )-
-		( 4*( ($n+1)/(($n-1)**2) ))*
-		($a1 - ((2*$n)/($n+1))) ) /
-		$a_n - $v_F;
+    my $u_F = ( 1 + ( ($sample_size+1)/(3*($sample_size-1)) )-
+		( 4*( ($sample_size+1)/(($sample_size-1)**2) ))*
+		($a1 - ((2*$sample_size)/($sample_size+1))) ) /
+		($a - $v_F);
 
-    # warn("$v_F vf $u_F uf n = $n\n");
     my $F = ($pi - $external) / ( sqrt( ($u_F*$seg_sites) +
 					($v_F*($seg_sites**2)) ) );
 
@@ -443,7 +369,7 @@ sub fu_and_li_F_counts {
 =head2 fu_and_li_F_star
 
  Title   : fu_and_li_F_star
- Usage   : my $F = Bio::PopGen::Statistics->fu_and_li_F_star(\@ingroup);
+ Usage   : my $D = Bio::PopGen::Statistics->fu_and_li_F_star(\@ingroup);
  Function: Calculate Fu and Li's F* on an ingroup without an outgroup
            It uses count of singleton alleles instead 
  Returns : decimal number
@@ -458,9 +384,9 @@ sub fu_and_li_F_counts {
 sub fu_and_li_F_star {
     my ($self,$individuals) = @_;
 
-    my ($seg_sites,$pi,$n,$singletons);
+    my ($seg_sites,$pi,$sample_size,$singletons);
     if( ref($individuals) =~ /ARRAY/i ) {
-	$n = scalar @$individuals;
+	$sample_size = scalar @$individuals;
 	# pi - all pairwise differences 
 	$pi          = $self->pi($individuals);  
 	$seg_sites   = $self->segregating_sites_count($individuals);
@@ -468,74 +394,54 @@ sub fu_and_li_F_star {
     } elsif( ref($individuals) && 
 	     $individuals->isa('Bio::PopGen::PopulationI')) {
 	my $pop = $individuals;
-	$n = $pop->get_number_individuals;
+	$sample_size = $pop->get_number_individuals;
 	$pi          = $self->pi($pop);
 	$seg_sites   = $self->segregating_sites_count($pop);
 	$singletons  = $self->singleton_count($pop);
     } else { 
-	$self->throw("expected an array reference of a list of Bio::PopGen::IndividualI OR a Bio::PopGen::PopulationI object to fu_and_li_F_star");
+	$self->throw("expected an array reference of a list of Bio::PopGen::IndividualI OR a Bio::PopGen::PopulationI object to tajima_D");
 	return 0;
     }
-    return $self->fu_and_li_F_star_counts($n,
-					  $pi,
-					  $seg_sites,
-					  $singletons);
-} 
-
-=head2 fu_and_li_F_star_counts
-
- Title   : fu_li_F_star_counts
- Usage   : my $F = $statistics->fu_and_li_F_star_counts($samps,
-                                                   $pi,$sites,
-                                                   $singletons);
- Function: Fu and Li F statistic for the raw counts of the number
-           of samples, sites, external and internal mutations
- Returns : decimal number
- Args    : number of samples (N)
-           average pairwise differences (pi)
-           number of segregating sites (n)
-           singleton  mutations (n_s)
-
-=cut
-
-
-sub fu_and_li_F_star_counts {
-    my ($self,$n,$pi,$seg_sites, $singletons) = @_;
-    if( $n <= 1 ) {
-	$self->warn("N must be > 1\n");
-	return;
+    my $a = 0;
+    for(my $k= 1; $k < $sample_size; $k++ ) {
+	$a += ( 1 / $k );
     }
-    if( $n == 2) { 
-	return 0;
-    } 
-
-    my $a_n = 0;
     
+    my $a1 = 0;
+    for(my $k= 1; $k <= $sample_size; $k++ ) {
+	$a1 += ( 1 / $k );
+    }
 
     my $b = 0;
-    for(my $k= 1; $k < $n; $k++ ) {
-	$b += (1 / ($k**2));
-	$a_n += ( 1 / $k );     # Eq (2)
+    for(my $k= 1; $k < $sample_size; $k++ ) {
+	$b += ( 1 / $k**2 );
     }
-    my $a1 = $a_n + (1 / $n );
-
-    # warn("a_n is $a_n a1 is $a1 n is $n b is $b\n");
-
-    # From Simonsen et al (1995) instead of Fu and Li 1993
-    my $v_F_star = ( (( 2 * $n ** 3 + 110 * $n**2 - (255 * $n) + 153)/
-		      (9 * ($n ** 2) * ( $n - 1))) +
-		     ((2 * ($n - 1) * $a_n ) / $n ** 2) -
-		     (8 * $b / $n) ) / 
-		     ( ($a_n ** 2) + $b );
+    # eq (14) 
+    my $c = 2 * ( (($sample_size * $a) - (2 * ( $sample_size -1 ))) / 
+		  (( $sample_size - 1) * ($sample_size - 2)) );
+    # eq (46) 
+    my $d = $c + ( ($sample_size -2)/ (($sample_size - 1)**2)) +
+	     ((2/($sample_size -1))*
+	      ((3/2) - ((2*$a1 - 3)/($sample_size -2)) - 
+	       (1/$sample_size)));
     
-    my $u_F_star = ((( (4* ($n**2)) + (19 * $n) + 3 - (12 * ($n + 1)* $a1)) /
-		    (3 * $n * ( $n - 1))) / $a_n) - $v_F_star;
-
-    # warn("vf* = $v_F_star uf* = $u_F_star n = $n\n");
-    my $F_star = ( $pi - ($singletons*( ( $n-1) / $n)) ) /
-	sqrt ( $u_F_star*$seg_sites + $v_F_star*$seg_sites**2);
+    my $v_F_star = ( $d + ( 2*($sample_size**2+$sample_size+3) /
+			    (9*$sample_size*($sample_size-1))) -
+		     ( (2/($sample_size-1))*
+		       (4*$b - 6 + (8/$sample_size))) )/
+		       ($a**2 + $b);
+    
+    my $u_F_star = ( ($sample_size / ($sample_size-1)) + 
+		     (($sample_size+1)/(3*($sample_size-1))) -
+		     ( 2 * (2 / ($sample_size * ($sample_size-1)))) +
+		     (2*( ($sample_size+1)/($sample_size-1)**2)*
+		      ($a1 - ((2*$sample_size)/($sample_size+1))) )) /
+		      ($a - $v_F_star);
+    
+    my $F_star = ( $pi - (( ($sample_size-1)/ $sample_size)*$singletons)) /
+	sqrt ( ($u_F_star*$seg_sites) + ($v_F_star*($seg_sites**2)));
     return $F_star;
-}
+} 
 
 =head2 tajima_D
 
@@ -554,10 +460,10 @@ sub fu_and_li_F_star_counts {
 
 sub tajima_D {
     my ($self,$individuals) = @_;
-    my ($seg_sites,$pi,$n);
+    my ($seg_sites,$pi,$sample_size);
 
     if( ref($individuals) =~ /ARRAY/i ) {
-	$n = scalar @$individuals;
+	$sample_size = scalar @$individuals;
 	# pi - all pairwise differences 
 	$pi          = $self->pi($individuals);  
 	$seg_sites = $self->segregating_sites_count($individuals);
@@ -565,49 +471,30 @@ sub tajima_D {
     } elsif( ref($individuals) && 
 	     $individuals->isa('Bio::PopGen::PopulationI')) {
 	my $pop = $individuals;
-	$n = $pop->get_number_individuals;
+	$sample_size = $pop->get_number_individuals;
 	$pi          = $self->pi($pop);
 	$seg_sites = $self->segregating_sites_count($pop);
     } else { 
 	$self->throw("expected an array reference of a list of Bio::PopGen::IndividualI OR a Bio::PopGen::PopulationI object to tajima_D");
 	return 0;
     }
-    $self->tajima_D_counts($n,$seg_sites,$pi);
-}
-
-=head2 tajima_D_counts
-
- Title   : tajima_D_counts
- Usage   : my $D = $statistics->tajima_D_counts($samps,$sites,$pi);
- Function: Tajima's D statistic for the raw counts of the number
-           of samples, sites, and avg pairwise distances (pi)
- Returns : decimal number
- Args    : number of samples (N)
-           number of segregating sites (n)
-           average pairwise differences (pi)
-
-=cut
-
-#'
-
-sub tajima_D_counts {
-    my ($self,$n,$seg_sites,$pi) = @_;
     my $a1 = 0; 
-    for(my $k= 1; $k < $n; $k++ ) {
+    for(my $k= 1; $k < $sample_size; $k++ ) {
 	$a1 += ( 1 / $k );
     }
 
      my $a2 = 0;
-     for(my $k= 1; $k < $n; $k++ ) {
+     for(my $k= 1; $k < $sample_size; $k++ ) {
 	 $a2 += ( 1 / $k**2 );
      }
+
     
-    my $b1 = ( $n + 1 ) / ( 3* ( $n - 1) );
-    my $b2 = ( 2 * ( $n ** 2 + $n + 3) ) / 
-	     ( ( 9 * $n) * ( $n - 1) );
+    my $b1 = ( $sample_size + 1 ) / ( 3* ( $sample_size - 1) );
+    my $b2 = ( 2 * ( $sample_size ** 2 + $sample_size + 3) ) / 
+	     ( ( 9 * $sample_size) * ( $sample_size - 1) );
     my $c1 = $b1 - ( 1 / $a1 );
-    my $c2 = $b2 - ( ( $n + 2 ) /
-		     ( $a1 * $n))+( $a2 / $a1 ** 2);
+    my $c2 = $b2 - ( ( $sample_size + 2 ) /
+		     ( $a1 * $sample_size))+( $a2 / $a1 ** 2);
     my $e1 = $c1 / $a1;
     my $e2 = $c2 / ( $a1**2 + $a2 );
     
@@ -617,15 +504,14 @@ sub tajima_D_counts {
     return $D;
 }
 
-
 =head2 pi
 
  Title   : pi
  Usage   : my $pi = Bio::PopGen::Statistics->pi(\@inds)
- Function: Calculate pi (average number of pairwise differences) given
-           a list of individuals which have the same number of markers
-           (also called sites) as available from the get_Genotypes()
-           call in L<Bio::PopGen::IndividualI>
+ Function: Calculate pi (...explain here...) given a list of individuals 
+           which have the same number of markers/sites/mutation as 
+           available from the get_Genotypes() call in 
+           L<Bio::PopGen::IndividualI>
  Returns : decimal number
  Args    : Arg1= array ref of L<Bio::PopGen::IndividualI> objects
              which have markers/mutations.  We expect all individuals to
@@ -641,14 +527,14 @@ sub tajima_D_counts {
 
 sub pi {
     my ($self,$individuals,$numsites) = @_;
-    my (%data,@marker_names,$n);
+    my (%data,@marker_names,$sample_size);
 
     if( ref($individuals) =~ /ARRAY/i ) {
 	# one possible argument is an arrayref of Bio::PopGen::IndividualI objs
 	@marker_names = $individuals->[0]->get_marker_names;
-	$n = scalar @$individuals;
+	$sample_size = scalar @$individuals;
 
-	# Here we are calculating the allele frequencies
+	# Here we're calculating the allele frequencies
 	my %marker_total;
 	foreach my $ind ( @$individuals ) {
 	    if( ! $ind->isa('Bio::PopGen::IndividualI') ) {
@@ -672,7 +558,7 @@ sub pi {
     } elsif( ref($individuals) && 
 	     $individuals->isa('Bio::PopGen::PopulationI') ) {
 	my $pop = $individuals;
-	$n = $pop->get_number_individuals;
+	$sample_size = $pop->get_number_individuals;
 	foreach my $marker( $pop->get_Markers ) {
 	    push @marker_names, $marker->name;
 	    $data{$marker->name} = {$marker->get_Allele_Frequencies};
@@ -692,7 +578,7 @@ sub pi {
 	foreach my $al ( @alleles ) { $totalalleles += $markerdat->{$al} }
 	for( my $i =0; $i < scalar @alleles -1; $i++ ) {
 	    my ($a1,$a2) = ( $alleles[$i], $alleles[$i+1]);
-	    $pi += $self->heterozygosity($n, 
+	    $pi += $self->heterozygosity($sample_size, 
 					 $markerdat->{$a1} / $totalalleles,
 					 $markerdat->{$a2} / $totalalleles);
 	}
@@ -705,16 +591,14 @@ sub pi {
     }
 }
 
-
 =head2 theta
 
  Title   : theta
  Usage   : my $theta = Bio::PopGen::Statistics->theta($sampsize,$segsites);
- Function: Calculates Watterson's theta from the sample size 
+ Function: Calculates theta (...explanation here... ) from the sample size 
            and the number of segregating sites.
            Providing the third parameter, total number of sites will
-           return theta per site.
-           This is also known as K-hat = K / a_n   
+           return theta per site          
  Returns : decimal number 
  Args    : sample size (integer),
            num segregating sites (integer)
@@ -728,30 +612,28 @@ sub pi {
 
 =cut
 
-#'
-
 sub theta {
     my $self = shift;    
-    my ( $n, $seg_sites,$totalsites) = @_;
-    if( ref($n) =~ /ARRAY/i ) {
-	my $samps = $n;
+    my ( $sample_size, $seg_sites,$totalsites) = @_;
+    if( ref($sample_size) =~ /ARRAY/i ) {
+	my $samps = $sample_size;
 	$totalsites = $seg_sites; # only 2 arguments if one is an array
 	my %data;
 	my @marker_names = $samps->[0]->get_marker_names;
 	# we need to calculate number of polymorphic sites
 	$seg_sites = $self->segregating_sites_count($samps);
-	$n = scalar @$samps;
+	$sample_size = scalar @$samps;
 
-    } elsif(ref($n) &&
-	    $n->isa('Bio::PopGen::PopulationI') ) {
+    } elsif(ref($sample_size) &&
+	    $sample_size->isa('Bio::PopGen::PopulationI') ) {
 	# This will handle the case when we pass in a PopulationI object
-	my $pop = $n;
+	my $pop = $sample_size;
 	$totalsites = $seg_sites; # shift the arguments over by one
-	$n = $pop->haploid_population->get_number_individuals;
+	$sample_size = $pop->get_number_individuals;
 	$seg_sites = $self->segregating_sites_count($pop);
     }
     my $a1 = 0; 
-    for(my $k= 1; $k < $n; $k++ ) {
+    for(my $k= 1; $k < $sample_size; $k++ ) {
 	$a1 += ( 1 / $k );
     }    
     if( $totalsites ) { # 0 and undef are the same can't divide by them
@@ -863,7 +745,7 @@ sub segregating_sites_count{
 	   $seg_sites++ if( keys %$site > 1 );
        }
    } elsif( $type && $individuals->isa('Bio::PopGen::PopulationI') ) {
-       foreach my $marker ( $individuals->haploid_population->get_Markers ) {  
+       foreach my $marker ( $individuals->get_Markers ) {  
 	   my @alleles = $marker->get_Alleles;	    
 	   $seg_sites++ if ( scalar @alleles > 1 );
        }
@@ -902,7 +784,6 @@ sub heterozygosity {
     return $h;
 }
 
-
 =head2 derived_mutations
 
  Title   : derived_mutations
@@ -951,7 +832,7 @@ sub derived_mutations{
        }	   
    } elsif( ref($ingroup) && $ingroup->isa('Bio::PopGen::PopulationI') ) {
        @marker_names = $ingroup->get_marker_names;
-       for my $ind ( $ingroup->haploid_population->get_Individuals() ) {
+       for my $ind ( $ingroup->get_Individuals() ) {
 	   for my $m ( @marker_names ) {
 	       for my $allele ( map { $_->get_Alleles} 
 				    $ind->get_Genotypes($m) ) {
@@ -980,7 +861,7 @@ sub derived_mutations{
        }
    
    } elsif( $otype->isa('Bio::PopGen::PopulationI') ) {
-       for my $ind ( $outgroup->haploid_population->get_Individuals() ) {
+       for my $ind ( $outgroup->get_Individuals() ) {
 	   for my $m ( @marker_names ) {
 	       for my $allele ( map { $_->get_Alleles} 
 				    $ind->get_Genotypes($m) ) {
@@ -1071,10 +952,6 @@ sub composite_LD {
 
 	foreach my $ind ( @inds ) {
 	    my ($genotype) = $ind->get_Genotypes(-marker => $marker_name);
-	    if( ! defined $genotype ) { 
-		$self->warn("no genotype for marker $marker_name for individual ". $ind->unique_id. "\n");
-		next;
-	    }
 	    my @alleles  = sort $genotype->get_Alleles;
 	    next if( scalar @alleles != 2);
 	    my $genostr  = join(',', @alleles);
@@ -1205,7 +1082,7 @@ sub composite_LD {
 
 	    my $homozA_site1 = join(",", ($allele1_site1,$allele1_site1));
 	    my $homozB_site2 = join(",", ($allele1_site2,$allele1_site2));
-	my $p_AA = ($genotypes{$site1}->{$homozA_site1} || 0) / $n;
+	   	my $p_AA = ($genotypes{$site1}->{$homozA_site1} || 0) / $n;
 	    my $p_BB = ($genotypes{$site2}->{$homozB_site2} || 0) / $n;
 	    my $p_A  = $allele_freqs{$site1}->{$allele1_site1} || 0;	# an individual allele freq
 	    my $p_a  =  1 - $p_A;
@@ -1244,5 +1121,4 @@ sub composite_LD {
 }
 
 
-
 1;
