@@ -1,4 +1,4 @@
-# $Id: Enzyme.pm,v 1.16.4.2 2006/11/17 09:32:41 sendu Exp $
+# $Id: Enzyme.pm,v 1.8 2003/12/15 11:50:38 heikki Exp $
 #------------------------------------------------------------------
 #
 # BioPerl module Bio::Restriction::Enzyme
@@ -211,16 +211,17 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists. Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+   bioperl-l@bioperl.org              - General discussion
+   http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution. Bug reports can be submitted via the
-web:
+the bugs and their resolution. Bug reports can be submitted via email
+or the web:
 
-  http://bugzilla.open-bio.org/
+    bioperl-bugs@bio.perl.org
+    http://bugzilla.bioperl.org/
 
 =head1 AUTHOR
 
@@ -228,7 +229,7 @@ Rob Edwards, redwards@utmem.edu
 
 =head1 CONTRIBUTORS
 
-Heikki Lehvaslaiho, heikki-at-bioperl-dot-org
+Heikki Lehvaslaiho, heikki@ebi.ac.uk
 Peter Blaiklock, pblaiklo@restrictionmapper.org
 
 =head1 COPYRIGHT
@@ -256,12 +257,14 @@ purposes only.
 package Bio::Restriction::Enzyme;
 use strict;
 
+use Bio::Root::Root;
 use Bio::PrimarySeq;
+use Bio::Restriction::EnzymeI;
 
 use Data::Dumper;
 
-use vars qw (%TYPE);
-use base qw(Bio::Root::Root Bio::Restriction::EnzymeI);
+use vars qw (@ISA %TYPE );
+@ISA = qw(Bio::Root::Root Bio::Restriction::EnzymeI);
 
 BEGIN {
     my %TYPE = (I => 1, II => 1, III => 1);
@@ -439,7 +442,7 @@ sub site {
     if ( $site ) {
 
         $self->throw("Unrecognized characters in site: [$site]")
-            if $site =~ /[^ATGCMRWSYKVHDBN\^]/i;
+            unless $site =~ /[ATGCMRWSYKVHDBN\^]+/i;
         # we may have to redefine this if there is a ^ in the sequence
 
         # first, check and see if we have a cut site in the sequence
@@ -495,7 +498,7 @@ sub revcom_site {
     }
     if ($site) {
         $self->throw("Unrecognized characters in revcom site: [$site]")
-            if $site =~ /[^ATGCMRWSYKVHDBN\^]/i;
+	    unless $site =~ /[ATGCMRWSYKVHDBN\^]+/i;
 	
         # we may have to redefine this if there is a ^ in the sequence
 
@@ -812,6 +815,8 @@ sub cutter {
     return $cutter;
 }
 
+=cut
+
 
 =head2 is_palindromic
 
@@ -1101,14 +1106,7 @@ REBASE
 sub isoschizomers {
     my ($self) = shift;
     push @{$self->{_isoschizomers}}, @_ if @_;
-          # make sure that you don't dereference if null
-          # chad believes quite strongly that you should return
-          # a reference to an array anyway. don't bother dereferencing.
-          # i'll post that to the list.
-     if ($self->{'_isoschizomers'}) {
-         return @{$self->{_isoschizomers}};
-     }
-     
+    return @{$self->{_isoschizomers}};
 }
 
 
@@ -1248,9 +1246,7 @@ Added for compatibility to REBASE
 sub vendors {
     my $self = shift;
     push @{$self->{_vendors}}, @_ if @_;
-    if ($self->{'_vendors'}) {
-         return @{$self->{'_vendors'}};
-    }
+    return @{$self->{'_vendors'}};
 }
 
 
