@@ -1,4 +1,4 @@
-# $Id: IO.pm,v 1.37 2002/12/01 00:09:11 jason Exp $
+# $Id: IO.pm,v 1.50 2003/11/21 03:03:38 lstein Exp $
 #
 # BioPerl module for Bio::Root::IO
 #
@@ -54,6 +54,13 @@ platforms. At present, File::Spec and File::Temp are attempted. This
 module defines $PATHSEP, $TEMPDIR, and $ROOTDIR, which will always be set, 
 and $OPENFLAGS, which will be set if either of File::Spec or File::Temp fails.
 
+The -noclose boolean (accessed via the noclose method) prevents a
+filehandle from being closed when the IO object is cleaned up.  This
+is special behavior when a object like a parser might share a
+filehandle with an object like an indexer where it is not proper to
+close the filehandle as it will continue to be reused until the end of the
+stream is reached.  In general you won't want to play with this flag.
+
 =head1 FEEDBACK
 
 =head2 Mailing Lists
@@ -211,10 +218,15 @@ sub new {
 
  Title   : initialize_io
  Usage   : $self->_initialize_io(@params);
- Function: Initializes filehandle and other properties.
+ Function: Initializes filehandle and other properties from the parameters.
 
-           Currently recognized the following named parameters: -file, -fh
- Example :
+           Currently recognizes the following named parameters:
+              -file     name of file to open
+              -input    name of file, or GLOB, or IO::Handle object
+              -fh       file handle (mutually exclusive with -file)
+              -flush    boolean flag to autoflush after each write
+              -noclose  boolean flag, when set to true will not close a
+                        filehandle (must explictly call close($io->_fh)
  Returns : TRUE
  Args    : named parameters
 
@@ -226,13 +238,14 @@ sub _initialize_io {
 
     $self->_register_for_cleanup(\&_io_cleanup);
 
-    my ($input, $file, $fh, $flush) = $self->_rearrange([qw(INPUT 
+    my ($input, $noclose, $file, $fh, $flush) = $self->_rearrange([qw(INPUT 
+							    NOCLOSE
 							    FILE FH 
 							    FLUSH)], @args);
     
     delete $self->{'_readbuffer'};
     delete $self->{'_filehandle'};
-
+    $self->noclose( $noclose) if defined $noclose;
     # determine whether the input is a file(name) or a stream
     if($input) {
 	if(ref(\$input) eq "SCALAR") {
@@ -258,7 +271,7 @@ sub _initialize_io {
     if(defined($file) && ($file ne '')) {
 	$fh = Symbol::gensym();
 	open ($fh,$file) ||
-	    $self->throw("Could not open $file for reading: $!");
+	    $self->throw("Could not open $file: $!");
 	$self->file($file);
     }
     $self->_fh($fh) if $fh; # if not provided, defaults to STDIN and STDOUT
@@ -272,7 +285,7 @@ sub _initialize_io {
 
  Title   : _fh
  Usage   : $obj->_fh($newval)
- Function:
+ Function: Get/set the file handle for the stream encapsulated.
  Example :
  Returns : value of _filehandle
  Args    : newvalue (optional)
@@ -333,6 +346,9 @@ sub mode {
 
       if($^O eq 'linux'){
         $obj->{'_mode'} = 'r';
+        my $line = $iotest->getline;
+        $obj->_pushback($line) if defined $line;
+        $obj->{'_mode'} = defined $line ? 'r' : 'w';
         return $obj->{'_mode'};
       } else {
         my $line = $iotest->getline;
@@ -359,7 +375,7 @@ sub mode {
 
  Title   : file
  Usage   : $obj->file($newval)
- Function:
+ Function: Get/set the filename, if one has been designated.
  Example :
  Returns : value of file
  Args    : newvalue (optional)
@@ -387,7 +403,7 @@ sub file {
 
 sub _print {
     my $self = shift;
-    my $fh = $self->_fh || \*STDOUT;
+    my $fh = $self->_fh() || \*STDOUT;
     print $fh @_;
 }
 
@@ -416,15 +432,14 @@ sub _print {
 sub _readline {
     my $self = shift;
     my %param =@_;
-
-    my $fh = $self->_fh || \*STDIN;
+    my $fh = $self->_fh or return;
     my $line;
 
     # if the buffer been filled by _pushback then return the buffer
     # contents, rather than read from the filehandle
-	$line = shift @{$self->{'_readbuffer'}} || <$fh>;
+    $line = shift @{$self->{'_readbuffer'}} || <$fh>;
 
-    #don't stip line endings if -raw is specified
+    #don't strip line endings if -raw is specified
     $line =~ s/\r\n/\n/g if( (!$param{-raw}) && (defined $line) );
 
     return $line;
@@ -454,15 +469,26 @@ sub _pushback {
  Title   : close
  Usage   : $io->close()
  Function: Closes the file handle associated with this IO instance.
- Example :
- Returns :
- Args    :
+           Will not close the FH if  -noclose is specified
+ Returns : none
+ Args    : none
 
 =cut
 
 sub close {
-   my ($self, @args) = @_;
-
+   my ($self) = @_;
+   return if $self->noclose; # don't close if we explictly asked not to
+   if( defined $self->{'_filehandle'} ) {
+       $self->flush;
+       return if( \*STDOUT == $self->_fh ||
+		  \*STDERR == $self->_fh ||
+		  \*STDIN == $self->_fh
+		  ); # don't close STDOUT fh
+       if( ! ref($self->{'_filehandle'}) ||
+	   ! $self->{'_filehandle'}->isa('IO::String') ) {
+	   close($self->{'_filehandle'});
+       }
+   }
    $self->{'_filehandle'} = undef;
    delete $self->{'_readbuffer'};
 }
@@ -473,9 +499,8 @@ sub close {
  Title   : flush
  Usage   : $io->flush()
  Function: Flushes the filehandle
- Example :
- Returns :
- Args    :
+ Returns : none
+ Args    : none
 
 =cut
 
@@ -494,20 +519,40 @@ sub flush {
     $self->{'_filehandle'}->flush();
   }
 }
-  
 
+=head2 noclose
+
+ Title   : noclose
+ Usage   : $obj->noclose($newval)
+ Function: Get/Set the NOCLOSE flag - setting this to true will
+           prevent a filehandle from being closed
+           when an object is cleaned up or explicitly closed
+           This is a bit of hack 
+ Returns : value of noclose (a scalar)
+ Args    : on set, new value (a scalar or undef, optional)
+
+
+=cut
+
+sub noclose{
+    my $self = shift;
+
+    return $self->{'_noclose'} = shift if @_;
+    return $self->{'_noclose'};
+}
 
 sub _io_cleanup {
     my ($self) = @_;
 
     $self->close();
+    my $v = $self->verbose;
 
     # we are planning to cleanup temp files no matter what    
     if( exists($self->{'_rootio_tempfiles'}) &&
 	ref($self->{'_rootio_tempfiles'}) =~ /array/i) { 
-	if( $self->verbose > 0 ) {
-	    print STDERR "going to remove files ", 
-	    join(",",  @{$self->{'_rootio_tempfiles'}}), "\n";
+	if( $v > 0 ) {
+	    warn( "going to remove files ", 
+		  join(",",  @{$self->{'_rootio_tempfiles'}}), "\n");
 	}
 	unlink  (@{$self->{'_rootio_tempfiles'}} );
     }
@@ -516,9 +561,9 @@ sub _io_cleanup {
 	exists($self->{'_rootio_tempdirs'}) &&
 	ref($self->{'_rootio_tempdirs'}) =~ /array/i) {	
 
-	if( $self->verbose > 0 ) {
-	    print STDERR "going to remove dirs ", 
-	    join(",",  @{$self->{'_rootio_tempdirs'}}), "\n";
+	if( $v > 0 ) {
+	    warn( "going to remove dirs ", 
+		  join(",",  @{$self->{'_rootio_tempdirs'}}), "\n");
 	}
 	$self->rmtree( $self->{'_rootio_tempdirs'});
     }
@@ -764,11 +809,11 @@ sub catfile {
 sub rmtree {
     my($self,$roots, $verbose, $safe) = @_;
     if( $FILEPATHLOADED ) { 
-	return File::Path::rmtree ($roots, $verbose, $safe); 
-    }				
-    
+	return File::Path::rmtree ($roots, $verbose, $safe);
+    }
+
     my $force_writeable = ($^O eq 'os2' || $^O eq 'dos' || $^O eq 'MSWin32'
-		       || $^O eq 'amigaos');
+		       || $^O eq 'amigaos' || $^O eq 'cygwin');
     my $Is_VMS = $^O eq 'VMS';
 
     my(@files);
@@ -840,7 +885,7 @@ sub rmtree {
 	    chmod 0666, $root
 	      or $self->warn( "Can't make file $root writeable: $!")
 		if $force_writeable;
-	    print "unlink $root\n" if $verbose;
+	    warn "unlink $root\n" if $verbose;
 	    # delete all versions under VMS
 	    for (;;) {
 		unless (unlink $root) {
