@@ -1,4 +1,4 @@
-# $Id: GenericResult.pm,v 1.23.2.7 2006/10/16 17:08:16 sendu Exp $
+# $Id: GenericResult.pm,v 1.17 2003/04/09 03:34:03 sac Exp $
 #
 # BioPerl module for Bio::Search::Result::GenericResult
 #
@@ -12,9 +12,7 @@
 
 =head1 NAME
 
-Bio::Search::Result::GenericResult - Generic Implementation of
-Bio::Search::Result::ResultI interface applicable to most search
-results.
+Bio::Search::Result::GenericResult - Generic Implementation of Bio::Search::Result::ResultI interface applicable to most search results.
 
 =head1 SYNOPSIS
 
@@ -23,10 +21,10 @@ results.
     use Bio::SearchIO;
     my $io = new Bio::SearchIO(-format => 'blast',
                                 -file   => 't/data/HUMBETGLOA.tblastx');
-    while( my $result = $io->next_result ) {
+    while( my $result = $io->next_result) {
         # process all search results within the input stream
-        while( my $hit = $result->next_hit ) {  
-            # insert code here for hit processing
+        while( my $hit = $result->next_hits()) {  
+        # insert code here for hit processing
         }
     }
 
@@ -92,16 +90,17 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org              - General discussion
+  http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via the
-web:
+of the bugs and their resolution. Bug reports can be submitted via
+email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bioperl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich and Steve Chervitz
 
@@ -110,7 +109,7 @@ Email sac@bioperl.org
 
 =head1 CONTRIBUTORS
 
-Sendu Bala, bix@sendu.me.uk
+Additional contributors names and emails here
 
 =head1 APPENDIX
 
@@ -124,16 +123,17 @@ Internal methods are usually preceded with a _
 
 
 package Bio::Search::Result::GenericResult;
+use vars qw(@ISA);
 use strict;
 
-use Bio::Search::GenericStatistics;
-use Bio::Tools::Run::GenericParameters;
+use Bio::Root::Root;
+use Bio::Search::Result::ResultI;
 
 # bug #1420
 #use overload 
 #    '""' => \&to_string;
 
-use base qw(Bio::Root::Root Bio::Search::Result::ResultI);
+@ISA = qw(Bio::Root::Root Bio::Search::Result::ResultI);
 
 =head2 new
 
@@ -154,8 +154,6 @@ use base qw(Bio::Root::Root Bio::Search::Result::ResultI);
            -algorithm         => program name (blastx)
            -algorithm_version   => version of the algorithm (2.1.2)
            -algorithm_reference => literature reference string for this algorithm
-           -hit_factory       => Bio::Factory::ObjectFactoryI capable of making
-                                 Bio::Search::Hit::HitI objects
 
 =cut
 
@@ -166,13 +164,13 @@ sub new {
 
   $self->{'_hits'} = [];
   $self->{'_hitindex'} = 0;
-  $self->{'_statistics'} = new Bio::Search::GenericStatistics;
-  $self->{'_parameters'} = new Bio::Tools::Run::GenericParameters;
+  $self->{'_statistics'} = {};
+  $self->{'_parameters'} = {};
 
   my ($qname,$qacc,$qdesc,$qlen,
       $dbname,$dblet,$dbent,$params,   
       $stats, $hits, $algo, $algo_v,
-      $prog_ref, $algo_r, $hit_factory) = $self->_rearrange([qw(QUERY_NAME
+      $prog_ref, $algo_r) = $self->_rearrange([qw(QUERY_NAME
                                                   QUERY_ACCESSION
                                                   QUERY_DESCRIPTION
                                                   QUERY_LENGTH
@@ -186,7 +184,6 @@ sub new {
                                                   ALGORITHM_VERSION
                                                   PROGRAM_REFERENCE
                                                   ALGORITHM_REFERENCE
-                                                  HIT_FACTORY
                                                  )],@args);
 
   $algo_r ||= $prog_ref;         
@@ -202,24 +199,20 @@ sub new {
   defined $dblet  && $self->database_letters($dblet);
   defined $dbent  && $self->database_entries($dbent);
 
-  defined $hit_factory && $self->hit_factory($hit_factory);
-  
   if( defined $params ) {
       if( ref($params) !~ /hash/i ) {
-          $self->throw("Must specify a hash reference with the parameter '-parameters");
+          $self->throw("Must specify a hash reference with the the parameter '-parameters");
       }
       while( my ($key,$value) = each %{$params} ) {
-          $self->{'_parameters'}->set_parameter($key   =>   $value);
-               # $self->add_parameter($key,$value);
+          $self->add_parameter($key,$value);
       }
   }
   if( defined $stats ) {
       if( ref($stats) !~ /hash/i ) {
-          $self->throw("Must specify a hash reference with the parameter '-statistics");
+          $self->throw("Must specify a hash reference with the the parameter '-statistics");
       }
       while( my ($key,$value) = each %{$stats} ) {
-          $self->{'_statistics'}->set_statistic($key   =>   $value); 
-          # $self->add_statistic($key,$value);
+          $self->add_statistic($key,$value);
       }
   }
 
@@ -293,16 +286,8 @@ Bio::Search::Result::ResultI implementation
 sub next_hit {
     my ($self,@args) = @_;
     my $index = $self->_nexthitindex;
-    return if $index > scalar @{$self->{'_hits'}};
-    
-    my $hit = $self->{'_hits'}->[$index];
-    if (ref($hit) eq 'HASH') {
-        my $factory = $self->hit_factory || $self->throw("Tried to get a Hit, but it was a hash ref and we have no hit factory");
-        $hit = $factory->create_object(%{$hit});
-        $self->{'_hits'}->[$index] = $hit;
-        delete $self->{_hashes}->{$index};
-    }
-    return $hit;    
+    return undef if $index > scalar @{$self->{'_hits'}};
+    return $self->{'_hits'}->[$index];    
 }
 
 =head2 query_name
@@ -466,9 +451,9 @@ sub database_entries {
 
 =cut
 
-sub get_parameter {
+sub get_parameter{
    my ($self,$name) = @_;
-   return $self->{'_parameters'}->get_parameter($name);
+   return $self->{'_parameters'}->{$name};
 }
 
 =head2 available_parameters
@@ -483,7 +468,7 @@ sub get_parameter {
 
 sub available_parameters{
    my ($self) = @_;
-   return $self->{'_parameters'}->available_parameters;
+   return keys %{$self->{'_parameters'}};
 }
 
 
@@ -500,7 +485,7 @@ sub available_parameters{
 
 sub get_statistic{
    my ($self,$key) = @_;
-   return $self->{'_statistics'}->get_statistic($key);
+   return $self->{'_statistics'}->{$key};
 }
 
 =head2 available_statistics
@@ -515,7 +500,7 @@ sub get_statistic{
 
 sub available_statistics{
    my ($self) = @_;
-   return $self->{'_statistics'}->available_statistics;
+   return keys %{$self->{'_statistics'}};
 }
 
 =head2 Bio::Search::Report 
@@ -534,34 +519,15 @@ Bio::Search::Result::GenericResult specific methods
 
 sub add_hit {
     my ($self,$s) = @_;
-    if (ref($s) eq 'HASH' || $s->isa('Bio::Search::Hit::HitI') ) {
+    if( $s->isa('Bio::Search::Hit::HitI') ) { 
         push @{$self->{'_hits'}}, $s;
-    }
-    else { 
-        $self->throw("Passed in " .ref($s)." as a Hit which is not a Bio::Search::HitI.");
-    }
-    
-    if (ref($s) eq 'HASH') {
-        $self->{_hashes}->{$#{$self->{'_hits'}}} = 1;
+    } else { 
+        $self->throw("Passed in " .ref($s). 
+                     " as a Hit which is not a Bio::Search::HitI.");
     }
     return scalar @{$self->{'_hits'}};
 }
 
-=head2 hit_factory
-
- Title   : hit_factory
- Usage   : $hit->hit_factory($hit_factory)
- Function: Get/set the factory used to build HitI objects if necessary.
- Returns : Bio::Factory::ObjectFactoryI
- Args    : Bio::Factory::ObjectFactoryI
-
-=cut
-
-sub hit_factory {
-    my $self = shift;
-    if (@_) { $self->{_hit_factory} = shift }
-    return $self->{_hit_factory} || return;
-}
 
 =head2 rewind
 
@@ -604,9 +570,9 @@ sub _nexthitindex{
 
 =cut
 
-sub add_parameter {
+sub add_parameter{
    my ($self,$key,$value) = @_;
-   $self->{'_parameters'}->set_parameter($key => $value);
+   $self->{'_parameters'}->{$key} = $value;
 }
 
 
@@ -623,7 +589,7 @@ sub add_parameter {
 
 sub add_statistic {
    my ($self,$key,$value) = @_;
-   $self->{'_statistics'}->set_statistic($key => $value);
+   $self->{'_statistics'}->{$key} = $value;
    return;
 }
 
@@ -659,19 +625,12 @@ sub num_hits{
 
 =cut
 
-sub hits {
-    my ($self) = shift;
-    
-    foreach my $i (keys %{$self->{_hashes} || {}}) {
-        my $factory = $self->hit_factory || $self->throw("Tried to get a Hit, but it was a hash ref and we have no hit factory");
-        $self->{'_hits'}->[$i] = $factory->create_object(%{$self->{'_hits'}->[$i]});
-        delete $self->{_hashes}->{$i};
-    }
-    
-    my @hits = ();
-    if (ref $self->{'_hits'}) {
-        @hits = @{$self->{'_hits'}};
-    }
+sub hits{
+   my ($self) = shift;
+   my @hits = ();
+   if( ref $self->{'_hits'}) {
+       @hits = @{$self->{'_hits'}};
+   }
     return @hits;   
 }
 
