@@ -1,4 +1,4 @@
-# $Id: waba.pm,v 1.16.4.1 2006/10/02 23:10:26 sendu Exp $
+# $Id: waba.pm,v 1.11 2003/05/15 22:53:11 lapp Exp $
 #
 # BioPerl module for Bio::SearchIO::waba
 #
@@ -42,20 +42,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org              - General discussion
+  http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via the
-web:
+of the bugs and their resolution. Bug reports can be submitted via
+email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bioperl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
-Email jason-at-bioperl.org
+Email jason@bioperl.org
+
+Describe contact details here
+
+=head1 CONTRIBUTORS
+
+Additional contributors names and emails here
 
 =head1 APPENDIX
 
@@ -69,11 +76,12 @@ Internal methods are usually preceded with a _
 
 
 package Bio::SearchIO::waba;
-use vars qw(%MODEMAP %MAPPING @STATES);
+use vars qw(@ISA  %MODEMAP %MAPPING @STATES);
 use strict;
 
 # Object preamble - inherits from Bio::Root::Root
 
+use Bio::SearchIO;
 use Bio::Search::Result::ResultFactory;
 use Bio::Search::HSP::HSPFactory;
 
@@ -110,7 +118,7 @@ BEGIN {
 }
 
 
-use base qw(Bio::SearchIO);
+@ISA = qw(Bio::SearchIO );
 
 =head2 new
 
@@ -143,8 +151,6 @@ sub _initialize {
 
 sub next_result{
     my ($self) = @_;
-    local $/ = "\n";
-    local $_;
     
     my ($curquery,$curhit);
     my $state = -1;
@@ -261,7 +267,7 @@ sub next_result{
 	$self->end_element({'Name' => 'WABAOutput'});
 	return $self->end_document();
     }
-    return;
+    return undef;
 }
 
 =head2 start_element
