@@ -1,4 +1,4 @@
-# $Id: LargeSeq.pm,v 1.15.4.1 2006/10/02 23:10:27 sendu Exp $
+# $Id: LargeSeq.pm,v 1.11 2002/10/22 07:38:40 lapp Exp $
 #
 # BioPerl module for Bio::Seq::LargeSeq
 #
@@ -12,8 +12,7 @@
 
 =head1 NAME
 
-Bio::Seq::LargeSeq - SeqI compliant object that stores sequence as
-files in /tmp
+Bio::Seq::LargeSeq - SeqI compliant object that stores sequence as files in /tmp 
 
 =head1 SYNOPSIS
 
@@ -23,33 +22,35 @@ files in /tmp
 
 This object stores a sequence as a series of files in a temporary
 directory. The aim is to allow someone the ability to store very large
-sequences (eg, E<gt> 100MBases) in a file system without running out
-of memory (eg, on a 64 MB real memory machine!).
+sequences (eg, E<gt> 100MBases) in a file system without running out of memory
+(eg, on a 64 MB real memory machine!). 
 
 Of course, to actually make use of this functionality, the programs
-which use this object B<must> not call $primary_seq-E<gt>seq otherwise
-the entire sequence will come out into memory and probably paste your
-machine. However, calls $primary_seq-E<gt>subseq(10,100) will cause
-only 90 characters to be brought into real memory.
+which use this object B<must> not call $primary_seq-E<gt>seq otherwise the
+entire sequence will come out into memory and probably paste your
+machine. However, calls $primary_seq-E<gt>subseq(10,100) will cause only
+90 characters to be brought into real memory.
 
 =head1 FEEDBACK
 
 =head2 Mailing Lists
 
-User feedback is an integral part of the evolution of this and other
-Bioperl modules. Send your comments and suggestions preferably to one
-of the Bioperl mailing lists.  Your participation is much appreciated.
+User feedback is an integral part of the evolution of this
+and other Bioperl modules. Send your comments and suggestions preferably
+ to one of the Bioperl mailing lists.
+Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org          - General discussion
+  http://www.bioperl.org/MailList.html             - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via the
-web:
+ the bugs and their resolution.
+ Bug reports can be submitted via email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bio.perl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -57,8 +58,7 @@ Email birney@ebi.ac.uk
 
 =head1 APPENDIX
 
-The rest of the documentation details each of the object
-methods. Internal methods are usually preceded with a _
+The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _
 
 =cut
 
@@ -67,20 +67,21 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::Seq::LargeSeq;
-use vars qw($AUTOLOAD);
+use vars qw($AUTOLOAD @ISA);
 use strict;
 
-# Object preamble
+# Object preamble - inherits from Bio::Root::Objecttest 8, 
 
 use Bio::Seq::LargePrimarySeq;
+use Bio::Seq;
 
-use base qw(Bio::Seq Bio::Seq::LargeSeqI);
+@ISA = qw(Bio::Seq);
 
 
 sub new {
     my ($class, @args) = @_;
     my $self = $class->SUPER::new(@args);
-
+    
     my ($pseq) = $self->_rearrange([qw(PRIMARYSEQ)], @args);
 
     if( ! defined $pseq ) {
@@ -106,12 +107,14 @@ sub new {
 
 sub trunc {
     my ($self, $s, $e) = @_;
-    return new Bio::Seq::LargeSeq
-        ('-display_id' => $self->display_id,
-         '-accession_number' => $self->accession_number,
-         '-desc' => $self->desc,
-         '-alphabet' => $self->alphabet,
-         -primaryseq => $self->primary_seq->trunc($s,$e));
+    return new Bio::Seq::LargeSeq(
+				  '-display_id' => $self->display_id,
+				  '-accession_number' => $self->accession_number,
+				  '-desc' => $self->desc,
+				  '-alphabet' => $self->alphabet,
+				  -primaryseq => 
+				  $self->primary_seq->trunc($s,$e));
+
 }
 
 =head2 Bio::Seq::LargePrimarySeq methods
@@ -122,7 +125,7 @@ sub trunc {
 
  Title   : add_sequence_as_string
  Usage   : $seq->add_sequence_as_string("CATGAT");
- Function: Appends additional residues to an existing LargePrimarySeq object.
+ Function: Appends additional residues to an existing LargePrimarySeq object.  
            This allows one to build up a large sequence without storing
            entire object in memory.
  Returns : Current length of sequence
