@@ -1,4 +1,4 @@
-# $Id: SeqFeatureI.pm,v 1.43 2002/12/02 14:00:19 jason Exp $
+# $Id: SeqFeatureI.pm,v 1.51 2003/11/24 23:42:08 cjm Exp $
 #
 # BioPerl module for Bio::SeqFeatureI
 #
@@ -16,9 +16,9 @@ Bio::SeqFeatureI - Abstract interface of a Sequence Feature
 
 =head1 SYNOPSIS
 
-    # get a seqfeature somehow, eg,
+    # get a seqfeature somehow, eg, from a Sequence with Features attached
 
-    foreach $feat ( $seq->top_SeqFeatures() ) {
+    foreach $feat ( $seq->get_SeqFeatures() ) {
             print "Feature from ", $feat->start, "to ", 
 	          $feat->end, " Primary tag  ", $feat->primary_tag, 
 	          ", produced by ", $feat->source_tag(), "\n";
@@ -28,10 +28,14 @@ Bio::SeqFeatureI - Abstract interface of a Sequence Feature
             } else {
                 print "Feature on strand ", $feat->strand,"\n"; # -1,1
             }
+            print "feature location is ",$feat->start, "..",
+                  $feat->end, " on strand ", $feat->strand, "\n";
+            print "easy utility to print locations in GenBank/EMBL way ",
+                  $feat->location->to_FTstring(), "\n";
 
-            foreach $tag ( $feat->all_tags() ) {
-		print "Feature has tag ", $tag, "with values, ",
-		      join(' ',$feat->each_tag_value($tag)), "\n";
+            foreach $tag ( $feat->get_all_tags() ) {
+		print "Feature has tag ", $tag, " with values, ",
+		      join(' ',$feat->get_tag_values($tag)), "\n";
             }
 	    print "new feature\n" if $feat->has_tag('new');
 	    # features can have sub features
@@ -42,7 +46,7 @@ Bio::SeqFeatureI - Abstract interface of a Sequence Feature
 
 This interface is the functions one can expect for any Sequence
 Feature, whatever its implementation or whether it is a more complex
-type (eg, a Gene). This object doesn\'t actually provide any
+type (eg, a Gene). This object does not actually provide any
 implemention, it just provides the definitions of what methods one can
 call. See Bio::SeqFeature::Generic for a good standard implementation
 of this object
@@ -77,9 +81,14 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::SeqFeatureI;
-use vars qw(@ISA);
+use vars qw(@ISA $HasInMemory);
 use strict;
 
+BEGIN {
+    eval { require Bio::DB::InMemoryCache };
+    if( $@ ) { $HasInMemory = 0 }
+    else { $HasInMemory = 1 }
+}
 use Bio::RangeI;
 use Bio::Seq;
 
@@ -185,8 +194,9 @@ sub has_tag{
  Usage   : @values = $self->get_tag_values('some_tag')
  Function: 
  Returns : An array comprising the values of the specified tag.
- Args    :
+ Args    : a string
 
+throws an exception if there is no such tag
 
 =cut
 
@@ -194,6 +204,36 @@ sub get_tag_values {
     shift->throw_not_implemented();
 }
 
+=head2 get_tagset_values
+
+ Title   : get_tagset_values
+ Usage   : @values = $self->get_tagset_values(qw(label transcript_id product))
+ Function: 
+ Returns : An array comprising the values of the specified tags, in order of tags
+ Args    : An array of strings
+
+does NOT throw an exception if none of the tags are not present
+
+this method is useful for getting a human-readable label for a
+SeqFeatureI; not all tags can be assumed to be present, so a list of
+possible tags in preferential order is provided
+
+=cut
+
+# interface + abstract method
+sub get_tagset_values {
+    my ($self, @args) = @_;
+    my @vals = ();
+    foreach my $arg (@args) {
+        if ($self->has_tag($arg)) {
+            push(@vals, $self->get_tag_values($arg));
+        }
+    }
+    return @vals;
+}
+
+
+
 =head2 get_all_tags
 
  Title   : get_all_tags
@@ -424,8 +464,8 @@ but can be validly overwritten by subclasses
 =cut
 
 sub spliced_seq {
-    my ($self,$db) = shift;
-
+    my ($self,$db) = @_;
+   
     if( ! $self->location->isa("Bio::Location::SplitLocationI") ) {
 	return $self->seq(); # nice and easy!
     }
@@ -444,7 +484,14 @@ sub spliced_seq {
     # Might need to eventually allow this to be programable?    
     # (can I mention how much fun this is NOT! --jason)
     
-    my ($mixed,$fstrand) = (0);
+    my ($mixed,$mixedloc, $fstrand) = (0);
+    if( $db && ref($db) && ! $db->isa('Bio::DB::RandomAccessI') ) {
+	$self->warn("Must pass in a valid Bio::DB::RandomAccessI object for access to remote locations for spliced_seq");
+	$db = undef;
+    } elsif( defined $db && $HasInMemory && 
+	     ! $db->isa('Bio::DB::InMemoryCache') ) {
+	$db = new Bio::DB::InMemoryCache(-seqdb => $db);
+    }
     if( $self->isa('Bio::Das::SegmentI') &&
 	! $self->absolute ) { 
 	$self->warn("Calling spliced_seq with a Bio::Das::SegmentI which does have absolute set to 1 -- be warned you may not be getting things on the correct strand");
@@ -459,12 +506,18 @@ sub spliced_seq {
     map { 
 	$fstrand = $_->strand unless defined $fstrand;
 	$mixed = 1 if defined $_->strand && $fstrand != $_->strand;
+	if( defined $_->seq_id ) {
+	    $mixedloc = 1 if( $_->seq_id ne $seqid );
+	}
 	[ $_, $_->start* ($_->strand || 1)];	    
     } $self->location->each_Location; 
-    
+   
     if ( $mixed ) { 
 	$self->warn("Mixed strand locations, spliced seq using the input order rather than trying to sort");    
 	@locs = $self->location->each_Location; 
+    } elsif( $mixedloc ) {
+	# we'll use the prescribed location order
+	@locs = $self->location->each_Location; 
     }
 
     foreach my $loc ( @locs  ) {
@@ -477,10 +530,11 @@ sub spliced_seq {
 	}
 	# deal with remote sequences
 
-	if( $loc->seq_id ne $seqid ) {
+	if( defined $loc->seq_id && 
+	    $loc->seq_id ne $seqid ) {
 	    if( defined $db ) {
 		my $sid = $loc->seq_id;
-		$sid =~ s/\.\d+//g;
+		$sid =~ s/\.\d+$//g;
 		eval {
 		    $called_seq = $db->get_Seq_by_acc($sid);
 		};
@@ -489,6 +543,7 @@ sub spliced_seq {
 		    $called_seq = undef;
 		}
 	    } else {
+		$self->warn( "cannot get remote location for ".$loc->seq_id ." without a valid Bio::DB::RandomAccessI database handle (like Bio::DB::GenBank)");
 		$called_seq = undef;
 	    }
 	    if( !defined $called_seq ) {
@@ -512,7 +567,7 @@ sub spliced_seq {
 	}
     }
     my $out = Bio::Seq->new( -id => $self->entire_seq->display_id . "_spliced_feat",
-				      -seq => $seqstr);
+			     -seq => $seqstr);
     
     return $out;
 }
@@ -595,4 +650,21 @@ sub location {
 }
 
 
+=head2 primary_id
+
+ Title   : primary_id
+ Usage   : $obj->primary_id($newval)
+ Function: 
+ Example : 
+ Returns : value of primary_id (a scalar)
+ Args    : on set, new value (a scalar or undef, optional)
+
+
+=cut
+
+sub primary_id{
+    my $self = shift;
+    return $self->{'primary_id'} = shift if @_;
+    return $self->{'primary_id'};
+}
 1;
