@@ -178,7 +178,7 @@ my $nvtoken = ": ";  # The token used if a name/value pair has to be stuffed
 sub next_seq {
     my $self = shift;
     my ($desc);
-    my $bioSeq = $self->sequence_factory->create_sequence(-verbose =>$self->verbose());
+    my $bioSeq = $self->sequence_factory->create(-verbose =>$self->verbose());
     
     unless (exists $self->{'domtree'}) {
 	$self->throw("A BSML document has not yet been parsed.");
@@ -590,6 +590,9 @@ sub to_bsml {
 	my $seqDesc = [];
 	push @{$seqDesc}, ["comment" , "This file generated to BSML 2.2 standards - joins will be collapsed to a single feature enclosing all members of the join"];
 	push @{$seqDesc}, ["description" , eval{$bioSeq->desc}];
+	foreach my $kwd ( eval{$bioSeq->get_keywords} ) {
+	    push @{$seqDesc}, ["keyword" , $kwd];
+	}
 	push @{$seqDesc}, ["keyword" , eval{$bioSeq->keywords}];
 	push @{$seqDesc}, ["version" , eval{$bioSeq->seq_version}];
 	push @{$seqDesc}, ["division" , eval{$bioSeq->division}];
@@ -630,7 +633,8 @@ sub to_bsml {
 
 
 	foreach my $a (keys %attr) {
-	    $xmlSeq->setAttribute($a, $attr{$a}) if ($attr{$a} ne "");
+	    $xmlSeq->setAttribute($a, $attr{$a}) if (defined $attr{$a} &&
+						     $attr{$a} ne "");
 	}
 	# Orphaned Attributes:
 	$xmlSeq->setAttribute('topology', 'circular') 
@@ -686,10 +690,11 @@ sub to_bsml {
 
 #>>>>	# Perhaps it is better to loop through top_Seqfeatures?...
 #>>>>	# ...however, BSML does not have a hierarchy for Features
-	
-	if ($args->{SKIPFEAT} eq 'all') {
+
+	if (defined $args->{SKIPFEAT} &&
+	    $args->{SKIPFEAT} eq 'all') {
 	    $args->{SKIPFEAT} = { all => 1};
-	}
+	} else { $args->{SKIPFEAT} ||= {} }
 	foreach my $class (keys %{$args->{SKIPFEAT}}) {
 	    $args->{SKIPFEAT}{lc($class)} = $args->{SKIPFEAT}{$class};
 	}
@@ -720,7 +725,8 @@ sub to_bsml {
 		$self->_parse_location($xml, $xmlFeat, $bioFeat);
 
 		# loop through the tags, add them as <Qualifiers>
-		next if ($args->{SKIPTAGS} =~ /all/i);
+		next if (defined $args->{SKIPTAGS} &&
+			 $args->{SKIPTAGS} =~ /all/i);
 		# Tags can consume a lot of CPU cycles, and can often be
 		# rather non-informative, so -skiptags can allow total or
 		# selective omission of tags.
@@ -773,7 +779,8 @@ sub to_bsml {
 	}
     }
 
-    if ($args->{RETURN} =~ /seq/i) {
+    if (defined $args->{RETURN} &&
+	$args->{RETURN} =~ /seq/i) {
 	return \@xmlSequences;
     } else {
 	return $xml;
