@@ -1,4 +1,4 @@
-# $Id: fasta.pm,v 1.41 2002/12/24 12:14:14 birney Exp $
+# $Id: fasta.pm,v 1.47 2003/12/22 18:33:15 heikki Exp $
 # BioPerl module for Bio::SeqIO::fasta
 #
 # Cared for by Ewan Birney <birney@ebi.ac.uk>
@@ -39,10 +39,9 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.
- Bug reports can be submitted via email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
   http://bugzilla.bioperl.org/
 
 =head1 AUTHORS - Ewan Birney & Lincoln Stein
@@ -50,6 +49,9 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 Email: birney@ebi.ac.uk
        lstein@cshl.org
 
+=head1 CONTRIBUTORS
+
+Jason Stajich, jason-at-bioperl.org
 
 =head1 APPENDIX
 
@@ -61,7 +63,7 @@ methods. Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::SeqIO::fasta;
-use vars qw(@ISA $WIDTH);
+use vars qw(@ISA $WIDTH @SEQ_ID_TYPES $DEFAULT_SEQ_ID_TYPE);
 use strict;
 # Object preamble - inherits from Bio::Root::Object
 
@@ -71,6 +73,9 @@ use Bio::Seq::SeqFastaSpeedFactory;
 
 @ISA = qw(Bio::SeqIO);
 
+@SEQ_ID_TYPES = qw(accession accession.version display primary);
+$DEFAULT_SEQ_ID_TYPE = 'display';
+
 BEGIN { $WIDTH = 60}
 
 sub _initialize {
@@ -78,11 +83,8 @@ sub _initialize {
   $self->SUPER::_initialize(@args);  
   my ($width) = $self->_rearrange([qw(WIDTH)], @args);
   $width && $self->width($width);
-  if( ! defined $self->sequence_factory ) {
-      # we default to the speed factory
-      # print STDERR "Going to load default sequence factory...\n";
+  unless ( defined $self->sequence_factory ) {
       $self->sequence_factory(Bio::Seq::SeqFastaSpeedFactory->new());
-      #$self->sequence_factory(new Bio::Seq::SeqFactory(-verbose => $self->verbose(), -type => 'Bio::Seq'));      
   }
 }
 
@@ -103,20 +105,30 @@ sub next_seq {
     local $/ = "\n>";
     return unless my $entry = $self->_readline;
 
-    if ($entry eq '>')  {	# very first one
+    chomp($entry);
+    if ($entry =~ m/\A\s*\Z/s)  { # very first one
 	return unless $entry = $self->_readline;
+	chomp($entry);
+    }
+    $entry =~ s/^>//;
+
+    my ($top,$sequence) = split(/\n/,$entry,2);
+    defined $sequence && $sequence =~ s/>//g;
+#    my ($top,$sequence) = $entry =~ /^>?(.+?)\n+([^>]*)/s
+#	or $self->throw("Can't parse fasta entry");
+
+    my ($id,$fulldesc);
+    if( $top =~ /^\s*(\S+)\s*(.*)/ ) {
+	($id,$fulldesc) = ($1,$2);
     }
     
-    my ($top,$sequence) = $entry =~ /^>?(.+?)\n([^>]*)/s
-	or $self->throw("Can't parse fasta entry");
-    my ($id,$fulldesc) = $top =~ /^\s*(\S+)\s*(.*)/
-	or $self->throw("Can't parse fasta header");
-    if ($id eq '') {$id=$fulldesc;} # FIX incase no space between > and name \AE
-    $sequence =~ s/\s//g;	# Remove whitespace
+    if (defined $id && $id eq '') {$id=$fulldesc;} # FIX incase no space 
+                                                   # between > and name \AE
+    defined $sequence && $sequence =~ s/\s//g;	# Remove whitespace
 
     # for empty sequences we need to know the mol.type
     $alphabet = $self->alphabet();
-    if(length($sequence) == 0) {
+    if(defined $sequence && length($sequence) == 0) {
 	if(! defined($alphabet)) {
 	    # let's default to dna
 	    $alphabet = "dna";
@@ -167,7 +179,25 @@ sub write_seq {
 	   unless defined $seq && ref($seq) && $seq->isa('Bio::PrimarySeqI');
 
        my $str = $seq->seq;
-       my $top = $seq->display_id();
+       my $top;
+
+       # Allow for different ids 
+       my $id_type = $self->preferred_id_type;
+       if( $id_type =~ /^acc/i ) {
+	   $top = $seq->accession_number();
+	   if( $id_type =~ /vers/i ) {
+	       $top .= "." . $seq->version();
+	   }
+       } elsif($id_type =~ /^displ/i ) {
+
+           $self->warn("No whitespace allowed in FASTA ID [". $seq->display_id. "]")
+               if $seq->display_id =~ /\s/;
+
+	   $top = $seq->display_id();
+       } elsif($id_type =~ /^pri/i ) {
+	   $top = $seq->primary_id();
+       }
+
        if ($seq->can('desc') and my $desc = $seq->desc()) {
 	   $desc =~ s/\n//g;
 	   $top .= " $desc";
@@ -203,4 +233,32 @@ sub width{
     return $self->{'width'} || $WIDTH;
 }
 
+=head2 preferred_id_type
+
+ Title   : preferred_id_type
+ Usage   : $obj->preferred_id_type('accession')
+ Function: Get/Set the preferred type of identifier to use in the ">ID" position
+           for FASTA output.
+ Returns : string, one of values defined in @Bio::SeqIO::fasta::SEQ_ID_TYPES.
+           Default = $Bio::SeqIO::fasta::DEFAULT_SEQ_ID_TYPE ('display').
+ Args    : string when setting. This must be one of values defined in 
+           @Bio::SeqIO::fasta::SEQ_ID_TYPES. Allowable values:
+           accession, accession.version, display, primary
+ Throws  : fatal exception if the supplied id type is not in @SEQ_ID_TYPES.
+
+=cut
+
+sub preferred_id_type {
+    my ($self,$type) = @_;
+    if( defined $type ) {
+	if( ! grep lc($type) eq $_, @SEQ_ID_TYPES) {
+	    $self->throw(-class=>'Bio::Root::BadParameter',
+			 -text=>"Invalid ID type \"$type\". Must be one of: @SEQ_ID_TYPES");
+	}
+	$self->{'_seq_id_type'} = lc($type);
+#	print STDERR "Setting preferred_id_type=$type\n";
+    }
+    $self->{'_seq_id_type'} || $DEFAULT_SEQ_ID_TYPE;
+}
+
 1;
