@@ -1,8 +1,10 @@
-# $Id: game.pm,v 1.38.4.1 2006/10/02 23:10:29 sendu Exp $
+# $Id: game.pm,v 1.31 2003/12/16 16:58:51 smckay Exp $
 #
 # BioPerl module for Bio::SeqIO::game
 #
-# Cared for by Sheldon McKay <mckays@cshl.edu>
+# Cared for by Sheldon McKay <smckay@bcgsc.bc.ca>
+#
+# Copyright Sheldon McKay
 #
 # You may distribute this module under the same terms as perl itself
 #
@@ -15,8 +17,6 @@ Bio::SeqIO::game -- a class for parsing and writing game-XML
 
 =head1 SYNOPSIS
 
-This module is not used directly, use SeqIO.
-
  use Bio::SeqIO;
 
  my $in = Bio::SeqIO->new ( -file    => 'file.xml', 
@@ -28,11 +28,13 @@ This module is not used directly, use SeqIO.
 =head1 DESCRIPTION
 
 Bio::SeqIO::game will parse game XML (version 1.2) or write game XML from 
-a Bio::SeqI implementing object.  The XML is readable by the genome 
-annotation editor 'Apollo' (www.gmod.org).  It is not backwards compatible 
-with the previous version of game XML.  The XML format currently used by 
-Apollo contains a single 'main' annotated sequence, so we will only get a 
-single annotated sequence in the stream when parsing a game-XML record.
+a Bio::SeqI implementing object.  The XML is readable by the genome annotation
+editor 'Apollo' (www.gmod.org).  It is not backwards compatible with the previous
+version of game XML.  The XML format currently used by Apollo 
+contains a single 'main' annotated sequence, so we will only get a single 
+annotated sequence in the stream when parsing a game-XML record.
+
+This modules is not used directly
 
 =head1 FEEDBACK
 
@@ -45,20 +47,21 @@ to one of the Bioperl mailing lists.
 Your participation is much appreciated.
 
   bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  http://bioperl.org/MailList.shtml      - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution.
 
-Bug reports can be submitted via the web:
+Bug reports can be submitted via email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bioperl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Sheldon McKay
 
-Email mckays@cshl.edu
+Email smckay@bcgsc.bc.ca
 
 =head1 APPENDIX
 
@@ -69,10 +72,12 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::SeqIO::game;
 
+use Bio::SeqIO;
 use Bio::SeqIO::game::gameHandler;
 use Bio::SeqIO::game::gameWriter;
 
-use base qw(Bio::SeqIO);
+use vars qw{ @ISA };
+@ISA = qw { Bio::SeqIO };
 
 sub _initialize {
     my ($self, @args) = @_;
@@ -87,7 +92,6 @@ sub _initialize {
  Returns : a Bio::Seq::RichSeq object
  Args    : none
 
-
 =cut
 
 sub next_seq {
