@@ -1,8 +1,8 @@
-# $Id: genbank.pm,v 1.76 2002/12/29 00:51:43 lapp Exp $
+# $Id: genbank.pm,v 1.99 2003/12/22 18:33:15 heikki Exp $
 #
 # BioPerl module for Bio::SeqIO::GenBank
 #
-# Cared for by Elia Stupka <elia@ebi.ac.uk>
+# Cared for by Elia Stupka <elia@tll.org.sg>
 #
 # Copyright Elia Stupka
 #
@@ -47,7 +47,7 @@ record.
 
 =item GI number
 
-$seq->primary_id
+$seq-E<gt>primary_id
 
 =back
 
@@ -84,8 +84,50 @@ L<Bio::Annotation::Collection> object.
 reference       - Should contain Bio::Annotation::Reference objects
 comment         - Should contain Bio::Annotation::Comment objects
 
+segment         - Should contain a Bio::Annotation::SimpleValue object
+origin          - Should contain a Bio::Annotation::SimpleValue object
+
 =back
 
+=head1 Where does the data go?
+
+Data parsed in Bio::SeqIO::genbank is stored in a variety of data
+fields in the sequence object that is returned.  More information in
+the HOWTOs about exactly what each field means and where it goes.
+Here is a partial list of fields.
+
+Items listed as RichSeq or Seq or PrimarySeq and then NAME() tell you
+the top level object which defines a function called NAME() which
+stores this information.
+
+Items listed as Annotation 'NAME' tell you the data is stored the
+associated Bio::Annotation::Colection object which is associated with
+Bio::Seq objects.  If it is explictly requested that no annotations
+should be stored when parsing a record of course they won't be
+available when you try and get them.  If you are having this problem
+look at the type of SeqBuilder that is being used to contruct your
+sequence object.
+
+Comments             Annotation 'comment'
+References           Annotation 'reference'
+Segment              Annotation 'segment'
+Origin               Annotation 'origin'
+
+Accessions           PrimarySeq accession_number()
+Secondary accessions RichSeq get_secondary_accessions()
+Keywords             RichSeq keywords()
+Dates                RichSeq get_dates()
+Molecule             RichSeq molecule()
+Seq Version          RichSeq seq_version()
+PID                  RichSeq pid()
+Division             RichSeq division()
+Features             Seq get_SeqFeatures()
+Alphabet             PrimarySeq alphabet()
+Definition           PrimarySeq description() or desc()
+Version              PrimarySeq version()
+
+Sequence             PrimarySeq seq()
+
 =head1 FEEDBACK
 
 =head2 Mailing Lists
@@ -109,7 +151,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 
 =head1 AUTHOR - Elia Stupka
 
-Email elia@ebi.ac.uk
+Email elia@tll.org.sg
 
 =head1 CONTRIBUTORS
 
@@ -119,17 +161,19 @@ Chris Mungall cjm@fruitfly.bdgp.berkeley.edu
 Lincoln Stein lstein@cshl.org
 Heikki Lehvaslaiho, heikki@ebi.ac.uk
 Hilmar Lapp, hlapp@gmx.net
+Donald G. Jackson, donald.jackson@bms.com
 
 =head1 APPENDIX
 
-The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _
+The rest of the documentation details each of the object
+methods. Internal methods are usually preceded with a _
 
 =cut
 
 # Let the code begin...
 
 package Bio::SeqIO::genbank;
-use vars qw(@ISA);
+use vars qw(@ISA %FTQUAL_NO_QUOTE);
 use strict;
 
 use Bio::SeqIO;
@@ -143,7 +187,25 @@ use Bio::Annotation::Reference;
 use Bio::Annotation::DBLink;
 
 @ISA = qw(Bio::SeqIO);
- 
+
+%FTQUAL_NO_QUOTE=(
+		  'anticodon'    => 1,
+		  'citation'     => 1,
+		  'codon'        => 1,
+		  'codon_start'  => 1,
+		  'cons_splice'  => 1,
+		  'direction'    => 1,
+		  'evidence'     => 1,
+		  'label'        => 1,
+		  'mod_base'     => 1,
+		  'number'       => 1,
+		  'rpt_type'     => 1,
+		  'rpt_unit'     => 1,
+		  'transl_except'=> 1,
+		  'transl_table' => 1,
+		  'usedin'       => 1,
+		  );
+
 sub _initialize {
     my($self,@args) = @_;
     
@@ -191,7 +253,7 @@ sub next_seq {
 	  last if index($buffer,'LOCUS       ') == 0;
       }
       return undef if( !defined $buffer ); # end of file
-      $buffer =~ /^LOCUS\s+(\S.*)$/ ||
+      $buffer =~ /^LOCUS\s+(\S.*)$/o ||
 	  $self->throw("GenBank stream with bad LOCUS line. Not GenBank in my book. Got '$buffer'");
       
       my @tokens = split(' ', $1);
@@ -221,8 +283,33 @@ sub next_seq {
 	  $params{'-division'} = shift(@tokens);
       }
       my $date = join(' ', @tokens); # we lump together the rest
-      if($date =~ s/.*(\d\d-\w\w\w-\d\d\d\d).*/$1/) {
-	  $params{'-date'} = [$date];
+
+      # this is per request bug #1513
+      # we can handle
+      # 9-10-2003
+      # 9-10-03
+      #09-10-2003
+      #09-10-03
+      if($date =~ s/\s*((\d{1,2})-(\w{3})-(\d{2,4})).*/$1/) {
+	  if( length($date) < 11 ) { # improperly formatted date
+	                             # But we'll be nice and fix it for them
+	      my ($d,$m,$y) = ($2,$3,$4);
+	      if( length($d) == 1 ) {
+		  $d = "0$d";
+	      }
+	      # guess the century here
+	      if( length($y) == 2 ) {
+		  if( $y > 60 ) {  # arbitrarily guess that '60' means 1960
+		      $y = "19$y";
+		  } else { 
+		      $y = "20$y";
+		  }
+		  $self->warn("Date was malformed, guessing the century for $date to be $y\n");
+	      }
+	      $params{'-dates'} = [join('-',$d,$m,$y)];
+	  } else { 
+	      $params{'-dates'} = [$date];
+	  }
       }
       # set them all at once
       $builder->add_slot_value(%params);
@@ -239,7 +326,6 @@ sub next_seq {
 	  $annotation = new Bio::Annotation::Collection;
       }
       $buffer = $self->_readline();
-
       until( !defined ($buffer) ) {
 	  $_ = $buffer;
 	  
@@ -247,7 +333,7 @@ sub next_seq {
 	  if (/^DEFINITION\s+(\S.*\S)/) {
 	      my @desc = ($1);
 	      while ( defined($_ = $self->_readline) ) { 
-		  /^\s+(.*)/ && do { push (@desc, $1); next;};
+		  if( /^\s+(.*)/ ) { push (@desc, $1); next };		  
 		  last;
 	      }
 	      $builder->add_slot_value(-desc => join(' ', @desc));
@@ -256,7 +342,13 @@ sub next_seq {
 	  }
 	  # accession number (there can be multiple accessions)
 	  if( /^ACCESSION\s+(\S.*\S)/ ) {
-	      push(@acc, split(' ',$1));
+	      push(@acc, split(/\s+/,$1));
+	      while( defined($_ = $self->_readline) ) { 
+		  /^\s+(.*)/ && do { push (@acc, split(/\s+/,$1)); next };
+		  last;
+	      }
+	      $buffer = $_;
+	      next;
 	  }
 	  # PID
 	  elsif( /^PID\s+(\S+)/ ) {
@@ -275,10 +367,17 @@ sub next_seq {
 	  }
 	  #Keywords
 	  elsif( /^KEYWORDS\s+(.*)/ ) {
-	      my $keywords = $1;
-	      $keywords =~ s/\;//g;
-	      $keywords =~ s/\.$//; # remove possibly trailing dot
-	      $params{'-keywords'} = $keywords;
+	      my @kw = split(/\s*\;\s*/,$1);
+	      while( defined($_ = $self->_readline) ) { 
+		  chomp;
+		  /^\s+(.*)/ && do { push (@kw, split(/\s*\;\s*/,$1)); next };
+		  last;
+	      }
+	      
+	      @kw && $kw[-1] =~ s/\.$//;
+	      $params{'-keywords'} = \@kw;
+	      $buffer = $_;
+	      next;
 	  }
 	  # Organism name and phylogenetic information
 	  elsif (/^SOURCE/) {
@@ -326,9 +425,28 @@ sub next_seq {
 		  }
 	      }
 	      next;
+	  } elsif( /^SEGMENT\s+(.+)/ ) {
+	      if($annotation) {
+		  my $segment = $1;
+		  while (defined($_ = $self->_readline)) {
+		      last if (/^\S/);
+		      $segment .= $_; 
+		  }
+		  $segment =~ s/\n/ /og;
+		  $segment =~ s/ {2,}/ /og;
+		  $annotation->add_Annotation(
+					      'segment',
+					      Bio::Annotation::SimpleValue->new(-value => $segment));
+		  $buffer = $_;
+	      } else {
+		  while(defined($buffer = $self->_readline())) {
+		      last if substr($buffer,0,1) ne ' ';
+		  }
+	      }
+	      next;
 	  }
 	  # Exit at start of Feature table, or start of sequence
-	  last if( /^(FEATURES)|(ORIGIN)/ );
+	  last if( /^(FEATURES|ORIGIN)/ );
 	  # Get next line and loop again
 	  $buffer = $self->_readline;
       }
@@ -348,7 +466,7 @@ sub next_seq {
       }      
       
       # some "minimal" formats may not necessarily have a feature table
-      if($builder->want_slot('features') && defined($_) && /^FEATURES/) {
+      if($builder->want_slot('features') && defined($_) && /^FEATURES/o) {
 	  # need to read the first line of the feature table
 	  $buffer = $self->_readline;
 	  
@@ -357,8 +475,8 @@ sub next_seq {
 	  while( defined($buffer) ) {
 	      # check immediately -- not at the end of the loop
 	      # note: GenPept entries obviously do not have a BASE line
-	      last if(($buffer =~ /^BASE/) || ($buffer =~ /^ORIGIN/) ||
-		      ($buffer =~ /^CONTIG/) );
+	      last if(($buffer =~ /^BASE/o) || ($buffer =~ /^ORIGIN/o) ||
+		      ($buffer =~ /^CONTIG/o) );
 
 	      # slurp in one feature at a time -- at return, the start of
 	      # the next feature will have been read already, so we need
@@ -372,7 +490,8 @@ sub next_seq {
 	      if( !defined $ftunit ) {
 		  # GRRRR. We have fallen over. Try to recover
 		  $self->warn("Unexpected error in feature table for ".$params{'-display_id'}." Skipping feature, attempting to recover");
-		  unless( ($buffer =~ /^\s{5,5}\S+/) or ($buffer =~ /^\S+/)) {
+		  unless( ($buffer =~ /^\s{5,5}\S+/o) or 
+			  ($buffer =~ /^\S+/o)) {
 		      $buffer = $self->_readline();
 		  }
 		  next; # back to reading FTHelpers
@@ -398,7 +517,7 @@ sub next_seq {
 	  $_ = $buffer;
       }
       if( defined ($_) ) {
-	  if( /^CONTIG/ && $builder->want_slot('features')) {
+	  if( /^CONTIG/o && $builder->want_slot('features')) {
 	      $b = "     $_"; # need 5 spaces to treat it like a feature
 	      my $ftunit = $self->_read_FTHelper_GenBank(\$b);
 	      if( ! defined $ftunit ) {
@@ -408,9 +527,9 @@ sub next_seq {
 		       $ftunit->_generic_seqfeature($self->location_factory(),
 						    $display_id));
 	      }	
-	  } elsif(! /^ORIGIN/) {     # advance to the section with the sequence
+	  } elsif(! /^(ORIGIN|\/\/)/ ) {    # advance to the sequence, if any
 	      while (defined( $_ = $self->_readline) ) {
-		  last if /^ORIGIN/;
+		  last if /^(ORIGIN|\/\/)/;
 	      }
 	  }
       }
@@ -419,16 +538,26 @@ sub next_seq {
 	  next RECORDSTART;
       }
       if($builder->want_slot('seq')) {
-	  my $seqc = '';
-	  while( defined($_ = $self->_readline) ) {
-	      /^\/\// && last;
-	      $_ = uc($_);
-	      s/[^A-Za-z]//g;
-	      $seqc .= $_;
+	  # the fact that we want a sequence does not necessarily mean that
+	  # there also is a sequence ...
+	  if(defined($_) && s/^ORIGIN//) {
+	      chomp;
+	      if( $annotation && length($_) > 0 ) {
+		  $annotation->add_Annotation('origin',
+					      Bio::Annotation::SimpleValue->new(-value => $_));
+	      }
+	      my $seqc = '';
+	      while( defined($_ = $self->_readline) ) {
+		  /^\/\// && last;
+		  $_ = uc($_);
+		  s/[^A-Za-z]//g;
+		  $seqc .= $_;
+	      }
+	      $self->debug("sequence length is ". length($seqc) ."\n");
+	      $builder->add_slot_value(-seq => $seqc);
 	  }
-	  $self->debug("sequence length is ". length($seqc) ."\n");
-	  $builder->add_slot_value(-seq => $seqc);
-      } else {
+      } elsif ( defined($_) && (substr($_,0,2) ne '//')) {
+	  # advance to the end of the record
 	  while( defined($_ = $self->_readline) ) {
 	      last if substr($_,0,2) eq '//';
 	  }
@@ -465,7 +594,6 @@ sub write_seq {
 	    $self->warn(" $seq is not a SeqI compliant module. Attempting to dump, but may fail!");
 	}
 
-	my $i;
 	my $str = $seq->seq;
 
 	my ($div, $mol);
@@ -475,9 +603,9 @@ sub write_seq {
 	    $div=$seq->division;
 	} 
 	if( !defined $div || ! $div ) { $div = 'UNK'; }
-
+	my $alpha = $seq->alphabet;
 	if( !$seq->can('molecule') || ! defined ($mol = $seq->molecule()) ) {
-	    $mol = $seq->alphabet || 'DNA';
+	    $mol =  $alpha || 'DNA';
 	}
 
 	my $circular = 'linear  ';
@@ -493,9 +621,13 @@ sub write_seq {
 	    if( $seq->can('get_dates') ) { 	    
 		($date) = $seq->get_dates();
 	    }
+
+            $self->warn("No whitespace allowed in GenBank display id [". $seq->display_id. "]")
+                if $seq->display_id =~ /\s/;
+
 	    $temp_line = sprintf ("%-12s%-15s%13s %s%4s%-8s%-8s %3s %-s", 
 				  'LOCUS', $seq->id(),$len,
-				  (lc($mol) eq 'protein') ? ('aa','', '') : 
+				  (lc($alpha) eq 'protein') ? ('aa','', '') : 
 				  ('bp', '',$mol),$circular,
 				  $div,$date);
 	} 
@@ -548,10 +680,17 @@ sub write_seq {
 	    $self->_print("KEYWORDS    $temp_line\n");   
 	} else {
 	    if( $seq->can('keywords') ) {
-		$self->_print("KEYWORDS    ",$seq->keywords,"\n");
+		my $kw = $seq->keywords;
+		$kw .= '.' if( $kw !~ /\.$/ );
+		$self->_print("KEYWORDS    $kw\n");
 	    }
 	} 
 
+	# SEGMENT if it exists
+	foreach my $ref ( $seq->annotation->get_Annotations('segment') ) {
+	    $self->_print(sprintf ("%-11s %s\n",'SEGMENT',
+				  $ref->value));
+	}
 
 	# Organism lines
 	if (my $spec = $seq->species) {
@@ -565,7 +704,7 @@ sub write_seq {
 	    if (my $ssp = $spec->sub_species) {
 		$OS .= " $ssp";
 	    }
-	    $self->_print("SOURCE      $OS.\n");
+	    $self->_print("SOURCE      $OS\n");
 	    $self->_print("  ORGANISM  ",
 			  ($spec->organelle() ? $spec->organelle()." " : ""),
 			  "$genus $species", "\n");
@@ -577,10 +716,12 @@ sub write_seq {
 	# Reference lines
 	my $count = 1;
 	foreach my $ref ( $seq->annotation->get_Annotations('reference') ) {
-	    $temp_line = sprintf ("REFERENCE   $count  (%s %d to %d)",
+            $temp_line = "REFERENCE   $count";
+	    $temp_line .= sprintf ("  (%s %d to %d)",
 				  ($seq->alphabet() eq "protein" ?
 				   "residues" : "bases"),
-				  $ref->start,$ref->end);
+				  $ref->start,$ref->end)
+                if $ref->start;
 	    $self->_print("$temp_line\n");
 	    $self->_write_line_GenBank_regex("  AUTHORS   ",' 'x12,
 					     $ref->authors,"\\s\+\|\$",80);
@@ -669,42 +810,43 @@ sub write_seq {
 
 	    my $base_count = sprintf("BASE COUNT %8s a %6s c %6s g %6s t%s\n",
 				     $alen,$clen,$glen,$tlen,
-				     ( $olen > 0 ) ? sprintf("%6s others",$olen) : '');
+				     ( $olen > 0 ) ? 
+				     sprintf("%6s others",$olen) : '');
 	    $self->_print($base_count); 
 	}
-	$self->_print(sprintf("ORIGIN%6s\n",''));
-	my $di;
 
-	# push sequence blocks to be printed into an array
-	# so we can comply with genbank format and not have 
-	# block \n
-	#     ^^ space after last block before newline is not allowed
-	#        in the strictest sense of the format 
-	my @seqline;
-	for ($i = 0; $i < length($str); $i += 10) {
+	my ($o) = $seq->annotation->get_Annotations('origin');
+	$self->_print(sprintf("%-6s%s\n",'ORIGIN',$o ? $o->value : ''));
+        # print out the sequence
+	my $nuc = 60;		# Number of nucleotides per line
+	my $whole_pat = 'a10' x 6; # Pattern for unpacking a whole line
+	my $out_pat   = 'A11' x 6; # Pattern for packing a line
+	my $length = length($str);
 
-	    $di=$i+11;
+	# Calculate the number of nucleotides which fit on whole lines
+	my $whole = int($length / $nuc) * $nuc;
 
-	    #first line
-	    if ($i==0) {
-		$self->_print(sprintf("%9d ",1));
-	    }
-	    #print sequence, spaced by 10
-	    push @seqline, substr($str,$i,10);
-	    #break line and print number at beginning of next line
-	    if(($i+10)%60 == 0) {	    
-		$self->_print(join(' ', @seqline), "\n");
-		$self->_print(sprintf("%9d ",$di));
-		@seqline = ();
-	    }
+	# Print the whole lines
+	my $i;
+	for ($i = 0; $i < $whole; $i += $nuc) {
+	    my $blocks = pack $out_pat,
+	    unpack $whole_pat,
+	    substr($str, $i, $nuc);
+            chop $blocks;
+	    $self->_print(sprintf("%9d $blocks\n", $i + $nuc - 59));
 	}
-	# fencepost problem, print whatever is left in the queue
-	# again we are doing this to comply with  genbank in the 
-	# strictest sense
-	$self->_print(join(' ', @seqline), ' ') if( @seqline );
 
+	# Print the last line
+	if (my $last = substr($str, $i)) {
+	    my $last_len = length($last);
+	    my $last_pat = 'a10' x int($last_len / 10) .'a'. $last_len % 10;
+	    my $blocks = pack $out_pat,
+	    unpack($last_pat, $last);
+            $blocks =~ s/ +$//;
+	    $self->_print(sprintf("%9d $blocks\n", $length - $last_len + 1));
+	}
 
-	$self->_print("\n//\n");
+	$self->_print("//\n");
 
 	$self->flush if $self->_flush_on_write && defined $self->_fh;
 	return 1;
@@ -724,7 +866,7 @@ sub write_seq {
 =cut
 
 sub _print_GenBank_FTHelper {
-   my ($self,$fth,$always_quote) = @_;
+   my ($self,$fth) = @_;
    
    if( ! ref $fth || ! $fth->isa('Bio::SeqIO::FTHelper') ) {
        $fth->warn("$fth is not a FTHelper class. Attempting to print, but there could be tears!");   
@@ -739,8 +881,6 @@ sub _print_GenBank_FTHelper {
 					$fth->loc,"\,\|\$",80);
    }
 
-   if( !defined $always_quote) { $always_quote = 0; }
-   
    foreach my $tag ( keys %{$fth->field} ) {
        foreach my $value ( @{$fth->field->{$tag}} ) {
 	   $value =~ s/\"/\"\"/g;
@@ -749,11 +889,14 @@ sub _print_GenBank_FTHelper {
 						" "x21,
 						"/$tag","\.\|\$",80);
 	   }
-           elsif( $always_quote == 1 || $value !~ /^\d+$/ ) {
-              my ($pat) = ($value =~ /\s/ ? '\s|$' : '.|$');	      
+           # there are almost 3x more quoted qualifier values and they
+           # are more common too so we take quoted ones first
+           elsif (!$FTQUAL_NO_QUOTE{$tag}) {
+              my ($pat) = ($value =~ /\s/ ? '\s|$' : '.|$');
 	      $self->_write_line_GenBank_regex(" "x21,
 					       " "x21,
 					       "/$tag=\"$value\"",$pat,80);
+
            } else {
               $self->_write_line_GenBank_regex(" "x21,
 					       " "x21,
@@ -770,7 +913,6 @@ sub _print_GenBank_FTHelper {
  Title   : _read_GenBank_References
  Usage   :
  Function: Reads references from GenBank format. Internal function really
- Example :
  Returns : 
  Args    :
 
@@ -792,46 +934,48 @@ sub _read_GenBank_References{
 
    my (@title,@loc,@authors,@com,@medline,@pubmed);
 
-   while( defined($_) || defined($_ = $self->_readline) ) {
-       if (/^  AUTHORS\s+(.*)/) { 
+   REFLOOP: while( defined($_) || defined($_ = $self->_readline) ) {
+       if (/^\s{2}AUTHORS\s+(.*)/o) { 
 	   push (@authors, $1);   
 	   while ( defined($_ = $self->_readline) ) {
-	       /^\s{3,}(.*)/ && do { push (@authors, $1);next;};
+	       /^\s{3,}(.*)/o && do { push (@authors, $1);next;};
 	       last;
 	   }
 	   $ref->authors(join(' ', @authors));
        }
-       if (/^  TITLE\s+(.*)/)  { 
+       if (/^\s{2}TITLE\s+(.*)/o)  { 
 	   push (@title, $1);
 	   while ( defined($_ = $self->_readline) ) {
-	       /^\s{3,}(.*)/ && do { push (@title, $1);
+	       /^\s{3,}(.*)/o && do { push (@title, $1);
 				     next;
 				 };
 	       last;
 	   }
 	   $ref->title(join(' ', @title));
        }
-       if (/^  JOURNAL\s+(.*)/) { 
+       if (/^\s{2}JOURNAL\s+(.*)/o) { 
 	   push(@loc, $1);
 	   while ( defined($_ = $self->_readline) ) {
-	       /^\s{3,}(.*)/ && do { push(@loc, $1);
+	       /^\s{3,}(.*)/o && do { push(@loc, $1);
 				     next;
 				 };
 	       last;
 	   }
 	   $ref->location(join(' ', @loc));
+	   redo REFLOOP;
        }
-       if (/^  REMARK\s+(.*)/) { 
+       if (/^\s{2}REMARK\s+(.*)/o) { 
 	   push (@com, $1);
 	   while ( defined($_ = $self->_readline) ) {	       
-	       /^\s{3,}(.*)/ && do { push(@com, $1);
+	       /^\s{3,}(.*)/o && do { push(@com, $1);
 				     next;
 				 };
 	       last;
 	   }
 	   $ref->comment(join(' ', @com));
+	   redo REFLOOP;
        }
-       if( /^  MEDLINE\s+(.*)/ ) {
+       if( /^\s{2}MEDLINE\s+(.*)/ ) {
 	   push(@medline,$1);
 	   while ( defined($_ = $self->_readline) ) {	       
 	       /^\s{4,}(.*)/ && do { push(@medline, $1);
@@ -840,8 +984,9 @@ sub _read_GenBank_References{
 	       last;
 	   }
 	   $ref->medline(join(' ', @medline));
-       }       
-       if( /^   PUBMED\s+(.*)/ ) {
+	   redo REFLOOP;
+       }
+       if( /^\s{3}PUBMED\s+(.*)/ ) {
 	   push(@pubmed,$1);
 	   while ( defined($_ = $self->_readline) ) {	       
 	       /^\s{5,}(.*)/ && do { push(@pubmed, $1);
@@ -850,9 +995,10 @@ sub _read_GenBank_References{
 	       last;
 	   }
 	   $ref->pubmed(join(' ', @pubmed));
+	   redo REFLOOP;
        }
        
-       /^REFERENCE/ && do {
+       /^REFERENCE/o && do {
 	   # store current reference
 	   $self->_add_ref_to_array(\@refs,$ref) if $ref;
 	   # reset
@@ -871,7 +1017,7 @@ sub _read_GenBank_References{
 	   }
        };
 
-       /^(FEATURES)|(COMMENT)/ && last;
+       /^(FEATURES)|(COMMENT)/o && last;
 
        $_ = undef; # Empty $_ to trigger read of next line
    }
@@ -931,21 +1077,22 @@ sub _read_GenBank_Species {
 
     $_ = $$buffer;
     
-    my( $sub_species, $species, $genus, $common, $organelle, @class );
+    my( $sub_species, $species, $genus, $common, $organelle, @class, $ns_name );
     # upon first entering the loop, we must not read a new line -- the SOURCE
     # line is already in the buffer (HL 05/10/2000)
     while (defined($_) || defined($_ = $self->_readline())) {
 	# de-HTMLify (links that may be encountered here don't contain
 	# escaped '>', so a simple-minded approach suffices)
         s/<[^>]+>//g;
-	if (/^SOURCE\s+(.*)/) {
+	if (/^SOURCE\s+(.*)/o) {
 	    # FIXME this is probably mostly wrong (e.g., it yields things like
 	    # Homo sapiens adult placenta cDNA to mRNA
 	    # which is certainly not what you want)
 	    $common = $1;
 	    $common =~ s/\.$//; # remove trailing dot
-	} elsif (/^\s+ORGANISM/) {
+	} elsif (/^\s{2}ORGANISM/o) {
 	    my @spflds = split(' ', $_);
+            ($ns_name) = $_ =~ /\w+\s+(.*)/o;
 	    shift(@spflds); # ORGANISM
 	    if(grep { $_ =~ /^$spflds[0]/i; } @organell_names) {
 		$organelle = shift(@spflds);
@@ -957,7 +1104,7 @@ sub _read_GenBank_Species {
 		$species = "sp.";
 	    }
 	    $sub_species = shift(@spflds) if(@spflds);
-        } elsif (/^\s+(.+)/) {
+        } elsif (/^\s+(.+)/o) {
 	    # only split on ';' or '.' so that 
 	    # classification that is 2 words will 
 	    # still get matched
@@ -973,10 +1120,13 @@ sub _read_GenBank_Species {
     $$buffer = $_;
     
     # Don't make a species object if it's empty or "Unknown" or "None"
-    return unless $genus and  $genus !~ /^(Unknown|None)$/i;
+    return unless $genus and  $genus !~ /^(Unknown|None)$/oi;
     
     # Bio::Species array needs array in Species -> Kingdom direction
-    if ($class[$#class] eq $genus) {
+    if ($class[0] eq 'Viruses') {
+        push( @class, $ns_name );
+    }
+    elsif ($class[$#class] eq $genus) {
         push( @class, $species );
     } else {
         push( @class, $genus, $species );
@@ -986,7 +1136,9 @@ sub _read_GenBank_Species {
     my $make = Bio::Species->new();
     $make->classification( \@class, "FORCE" ); # no name validation please
     $make->common_name( $common      ) if $common;
-    $make->sub_species( $sub_species ) if $sub_species;
+    unless ($class[-1] eq 'Viruses') {
+        $make->sub_species( $sub_species ) if $sub_species;
+    }
     $make->organelle($organelle) if $organelle;
     return $make;
 }
@@ -1011,7 +1163,7 @@ sub _read_FTHelper_GenBank {
         );
     my @qual = (); # An arrray of lines making up the qualifiers
     
-    if ($$buffer =~ /^     (\S+)\s+(.+?)\s*$/o) {
+    if ($$buffer =~ /^\s{5}(\S+)\s+(.+?)\s*$/o) {
         $key = $1;
         $loc = $2;
         # Read all the lines up to the next feature
@@ -1075,7 +1227,7 @@ sub _read_FTHelper_GenBank {
                 while ($value !~ /\"$/ or $value =~ tr/"/"/ % 2) {
 		    if($i >= $#qual) {
 		       $self->warn("Unbalanced quote in:\n" .
-				   join('', map("$_\n", @qual)) .
+				   join("\n", @qual) .
 				   "No further qualifiers will " .
 				   "be added for this feature");
 		       last QUAL;
@@ -1086,7 +1238,7 @@ sub _read_FTHelper_GenBank {
 
                     # add to value with a space unless the value appears
 		    # to be a sequence (translation for example)
-		    if(($value.$next) =~ /[^A-Za-z"-]/) {
+		    if(($value.$next) =~ /[^A-Za-z\"\-]/o) {
 			$value .= " ";
 		    }
                     $value .= $next;
@@ -1095,7 +1247,26 @@ sub _read_FTHelper_GenBank {
                 $value =~ s/^"|"$//g;
                 # Undouble internal quotes
                 $value =~ s/""/\"/g;
-            }
+            } elsif ( $value =~ /^\(/ ) { # values quoted by ()s
+		# Keep addingto value until we find the trailing bracket
+		# and the ()s are balanced
+		my $left = ($value =~ tr/\(/\(/); # count left parens
+		my $right = ($value =~ tr/\)/\)/); # count right parens
+		while( $value !~ /\)$/ or $left != $right ) {
+		    if( $i >= $#qual) {
+			$self->warn("Unbalanced parens in:\n".
+				    join("\n", @qual).
+				    "No further qualifiers will ".
+				    "be added for this feature");
+			last QUAL;
+		    }
+		    $i++;
+		    my $next = $qual[$i];		    
+		    $value .= $next;
+		    $left += ($next =~ tr/\(/\(/);
+		    $right += ($next =~ tr/\)/\)/);
+		}
+	    }
         } else {
             $value = '_no_value';
         }
@@ -1160,9 +1331,9 @@ sub _write_line_GenBank_regex {
 
    $length || $self->throw( "Miscalled write_line_GenBank without length. Programming error!");
 
-   if( length $pre1 != length $pre2 ) {
-       $self->throw( "Programming error - cannot called write_line_GenBank_regex with different length pre1 and pre2 tags!");
-   }
+#   if( length $pre1 != length $pre2 ) {
+#       $self->throw( "Programming error - cannot called write_line_GenBank_regex with different length pre1 and pre2 tags!");
+#   }
 
    my $subl = $length - (length $pre1) - 2;
    my @lines = ();
