@@ -1,8 +1,8 @@
-# $Id: swiss.pm,v 1.66 2002/12/24 12:14:14 birney Exp $
+# $Id: swiss.pm,v 1.75 2003/12/22 18:33:15 heikki Exp $
 #
 # BioPerl module for Bio::SeqIO::swiss
 #
-# Cared for by Elia Stupka <elia@ebi.ac.uk>
+# Cared for by Elia Stupka <elia@tll.org.sg>
 #
 # Copyright Elia Stupka
 #
@@ -93,7 +93,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 
 =head1 AUTHOR - Elia Stupka
 
-Email elia@ebi.ac.uk
+Email elia@tll.org.sg
 
 Describe contact details here
 
@@ -154,8 +154,8 @@ sub _initialize {
 sub next_seq {
    my ($self,@args) = @_;
    my ($pseq,$c,$line,$name,$desc,$acc,$seqc,$mol,$div,
-       $date,$comment,@date_arr, @sec);
-   my ($keywords,$acc_string);
+       $date,$comment,@date_arr);
+   
    my $genename = "";
    my ($annotation, %params, @features) = ( new Bio::Annotation::Collection);
 
@@ -176,9 +176,11 @@ sub next_seq {
    
    # fixed to allow _DIVISION to be optional for bug #946
    # see bug report for more information
-   $line =~ /^ID\s+([^\s_]+)(_([^\s_]+))?\s+([^\s;]+);\s+([^\s;]+);/ 
-     || $self->throw("swissprot stream with no ID. Not swissprot in my book");
-   
+   if( !($line =~ /^ID\s+([^\s_]+)(_([^\s_]+))?\s+([^\s;]+);\s+([^\s;]+);/)
+       && !($line =~ /^ID\s+([^\s_]+)(_([^\s_]+))?/ ) )  {
+       $self->throw("swissprot stream with no ID. Not swissprot in my book");
+   }
+
    if( $3 ) {
        $name = "$1$2";
        $params{'-division'} = $3;
@@ -225,7 +227,10 @@ sub next_seq {
        }
        #accession number(s)
        elsif( /^AC\s+(.+)/) {
-           $acc_string .= $acc_string ? " $1" : $1;
+	   my @accs = split(/[; ]+/, $1); # allow space in addition
+	   $params{'-accession_number'} = shift @accs 
+	       unless defined $params{'-accession_number'};
+	   push @{$params{'-secondary_accessions'}}, @accs;
        }
        #version number
        elsif( /^SV\s+(\S+);?/ ) {
@@ -301,7 +306,9 @@ sub next_seq {
        }
        #keywords
        elsif( /^KW\s+(.*)$/ ) {
-           $keywords .= $keywords ? " $1" : $1;
+	   my @kw = split(/\s*\;\s*/,$1);
+	   defined $kw[-1] && $kw[-1] =~ s/\.$//;
+	   push @{$params{'-keywords'}}, @kw;   
        }
 
        # Get next line. Getting here assumes that we indeed need to read the
@@ -340,17 +347,12 @@ sub next_seq {
        s/[^A-Za-z]//g;
        $seqc .= $_;
    }
-   $acc_string =~ s/\;\s*/ /g;
-   ( $acc, @sec ) = split " ",$acc_string;
 
    my $seq=  $self->sequence_factory->create
        (-verbose  => $self->verbose,
 	%params,
 	-seq      => $seqc,
 	-desc     => $desc,
-	-keywords => $keywords,
-	-accession_number => $acc,
-	-secondardy_accessions => \@sec,
 	-features => \@features,
 	-annotation => $annotation,
 	);
@@ -396,6 +398,9 @@ sub write_seq {
 	    $mol = 'XXX';
 	}
 
+        $self->warn("No whitespace allowed in SWISS-PROT display id [". $seq->display_id. "]")
+            if $seq->display_id =~ /\s/;
+
 	my $temp_line;
 	if( $self->_id_generation_func ) {
 	    $temp_line = &{$self->_id_generation_func}($seq);
@@ -409,7 +414,7 @@ sub write_seq {
 	    # Hence, switch to display_id(); _every_ sequence is supposed to have
 	    # this. HL 2000/09/03
 	    $mol =~ s/protein/PRT/;
-	    $temp_line = sprintf ("%10s     STANDARD;      %3s;   %d AA.",
+	    $temp_line = sprintf ("%-10s     STANDARD;      %3s;   %d AA.",
 				  $seq->display_id(), $mol, $len);
 	}
 
@@ -466,16 +471,21 @@ sub write_seq {
 	    my($species, @class) = $spec->classification();
 	    my $genus = $class[0];
 	    my $OS = "$genus $species";
-	    if ($class[$#class] =~ /viruses/i) {
-		# different OS / OC syntax for viruses LP 09/16/2000
-		shift @class;
-	    }
-	    if (my $ssp = $spec->sub_species) {
-		$OS .= " $ssp";
-	    }
-	    foreach (($spec->variant, $spec->common_name)) {
-		$OS .= " ($_)" if $_;
-	    }
+            if ($class[-1] eq 'Viruses') {
+                $OS = $species;
+                $OS .=  " ". $spec->sub_species if $spec->sub_species;
+            } else {
+                if ($class[$#class] =~ /viruses/i) {
+                    # different OS / OC syntax for viruses LP 09/16/2000
+                    shift @class;
+                }
+                if (my $ssp = $spec->sub_species) {
+                    $OS .= " $ssp";
+                }
+                foreach (($spec->variant, $spec->common_name)) {
+                    $OS .= " ($_)" if $_;
+                }
+            }
 	    $self->_print( "OS   $OS.\n");
 	    my $OC = join('; ', reverse(@class)) .'.';
 	    $self->_write_line_swissprot_regex("OC   ","OC   ",$OC,"\; \|\$",80);
@@ -541,40 +551,43 @@ sub write_seq {
 		$self->_print("DR   ",$dblink->database,"; ",$dblink->primary_id,"; ",
 			      $dblink->optional_id,"; ",$dblink->comment,".\n");
 	    } elsif($dblink->optional_id) {
-		$self->_print("DR   ",$dblink->database,"; ",$dblink->primary_id,"; ",
+		$self->_print("DR   ",$dblink->database,"; ",
+			      $dblink->primary_id,"; ",
 			      $dblink->optional_id,".\n");
 	    }
 	    else {
-		$self->_print("DR   ",$dblink->database,"; ",$dblink->primary_id,"; ",
-			      "-.\n");
+		$self->_print("DR   ",$dblink->database,
+			      "; ",$dblink->primary_id,"; ","-.\n");
 	    }
 	}   
 
 	# if there, write the kw line
-
-	if( $self->_kw_generation_func ) {
-	    $temp_line = &{$self->_kw_generation_func}($seq);
-	    $self->_print( "KW   $temp_line\n");
-	} else {
-	    if( $seq->can('keywords') ) {
-		$self->_write_line_swissprot_regex("KW   ","KW   ",
-						   $seq->keywords,"\\s\+\|\$",80);       
-	    }
+	{
+	    my( $kw );
+	    if( my $func = $self->_kw_generation_func ) {
+		$kw = &{$func}($seq);
+	    } elsif( $seq->can('keywords') ) {		
+		$kw = $seq->keywords;
+		if( ref($kw) =~ /ARRAY/i ) {
+		    $kw = join("; ", @$kw);
+		}
+		$kw .= '.' if( $kw !~ /\.$/ );
+	    }	    
+	    $self->_write_line_swissprot_regex("KW   ","KW   ",
+					       $kw, "\\s\+\|\$",80);           
 	}
 
-#Check if there is seqfeatures before printing the FT line
-	my @feats = $seq->top_SeqFeatures;
+        #Check if there is seqfeatures before printing the FT line
+	my @feats = $seq->can('top_SeqFeatures') ? $seq->top_SeqFeatures : ();
 	if ($feats[0]) {
 	    if( defined $self->_post_sort ) {
 
 		# we need to read things into an array. Process. Sort them. Print 'em
-
+		
 		my $post_sort_func = $self->_post_sort();
 		my @fth;
 
-		my @feats = $seq->top_SeqFeatures;
-
-		foreach my $sf ( $seq->top_SeqFeatures ) {
+		foreach my $sf ( @feats ) {
 		    push(@fth,Bio::SeqIO::FTHelper::from_SeqFeature($sf,$seq));
 		}
 		@fth = sort { &$post_sort_func($a,$b) } @fth;
@@ -586,7 +599,7 @@ sub write_seq {
 		# not post sorted. And so we can print as we get them.
 		# lower memory load...
 
-		foreach my $sf ( $seq->top_SeqFeatures ) {
+		foreach my $sf ( @feats ) {
 		    my @fth = Bio::SeqIO::FTHelper::from_SeqFeature($sf,$seq);
 		    foreach my $fth ( @fth ) {
 			if( ! $fth->isa('Bio::SeqIO::FTHelper') ) {
@@ -789,7 +802,7 @@ sub _print_swissprot_FTHelper {
 					      substr($fth->key,0,8),
 					      $start,$end),
 				      "FT                                ",
-				      $desc,'\s+|$',80);
+				      $desc.'.','\s+|$',80);
 }
 #'
 
@@ -877,7 +890,7 @@ sub _read_swissprot_Species {
     my $org;
 
     $_ = $$buffer;
-    my( $subspecies, $species, $genus, $common, $variant, $ncbi_taxid );
+    my( $subspecies, $species, $genus, $common, $variant, $ncbi_taxid, $ns_name );
     my @class;
     my ($binomial, $descr);
     my $osline = "";
@@ -891,6 +904,8 @@ sub _read_swissprot_Species {
 	    $osline .= $1;
 	    if($osline =~ s/(,|, and|\.)$//) {
 		($binomial, $descr) = $osline =~ /(\S[^\(]+)(.*)/;
+                ($ns_name) = $binomial;
+                $ns_name =~ s/\s+$//; #####
 		($genus, $species, $subspecies) = split(/\s+/, $binomial);
 		$species = "sp." unless $species;
 		while($descr =~ /\(([^\)]+)\)/g) {
@@ -925,7 +940,7 @@ sub _read_swissprot_Species {
 		my @virusnames = split(/\s+/, $binomial);
 		$species = (@virusnames > 1) ? pop(@virusnames) : '';
 		$genus = join(" ", @virusnames);
-		$subspecies = undef;
+		$subspecies = $descr;
 	    }
         }
 	elsif (/^OG\s+(.*)/) {
@@ -949,7 +964,13 @@ sub _read_swissprot_Species {
     # Don't make a species object if it is "Unknown" or "None"
     return if $genus =~ /^(Unknown|None)$/i;
 
-    if ($class[$#class] eq $genus) {
+    if ($class[0] eq 'Viruses') {
+        push( @class, $ns_name );
+    }
+    elsif ($class[0] eq 'Viruses') {
+        push( @class, $ns_name );
+    }
+    elsif ($class[$#class] eq $genus) {
         push( @class, $species );
     } else {
         push( @class, $genus, $species );
