@@ -1,4 +1,4 @@
-# $Id: ztr.pm,v 1.14.4.1 2006/10/02 23:10:30 sendu Exp $
+# $Id: ztr.pm,v 1.8 2002/10/22 07:38:42 lapp Exp $
 # BioPerl module for Bio::SeqIO::ztr
 #
 # Cared for by Aaron Mackey <amackey@virginia.edu>
@@ -30,16 +30,17 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org            - General discussion
+  http://bioperl.org/MailList.shtml - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.
-Bug reports can be submitted via the web:
+ the bugs and their resolution.
+ Bug reports can be submitted via email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bio.perl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHORS - Aaron Mackey
 
@@ -57,6 +58,7 @@ methods. Internal methods are usually preceded with a _
 package Bio::SeqIO::ztr;
 use vars qw(@ISA $READ_AVAIL);
 use strict;
+# Object preamble - inherits from Bio::Root::Object
 
 use Bio::SeqIO;
 use Bio::Seq::SeqFactory;
@@ -77,7 +79,7 @@ sub _initialize {
   my($self,@args) = @_;
   $self->SUPER::_initialize(@args);  
   if( ! defined $self->sequence_factory ) {
-      $self->sequence_factory(new Bio::Seq::SeqFactory(-verbose => $self->verbose(), -type => 'Bio::Seq::Quality'));      
+      $self->sequence_factory(new Bio::Seq::SeqFactory(-verbose => $self->verbose(), -type => 'Bio::Seq::SeqWithQuality'));      
   }
 
   my ($compression) = $self->_rearrange([qw[COMPRESSION]], @args);
@@ -96,7 +98,7 @@ sub _initialize {
  Title   : next_seq
  Usage   : $seq = $stream->next_seq()
  Function: returns the next sequence in the stream
- Returns : Bio::Seq::Quality object
+ Returns : Bio::SeqWithQuality object
  Args    : NONE
 
 =cut
