@@ -97,16 +97,17 @@ UsUser feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org          - General discussion
+  http://bio.perl.org/MailList.html             - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via the
-web:
+the bugs and their resolution.  Bug reports can be submitted via email
+or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bio.perl.org
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Ed Green
 
@@ -123,10 +124,12 @@ Internal methods are preceded with a _.
 
 package Bio::Structure::SecStr::STRIDE::Res;
 use strict;
+use vars qw(@ISA);
+use Bio::Root::Root;
 use Bio::Root::IO;
 use Bio::PrimarySeq;
 
-use base qw(Bio::Root::Root);
+@ISA = qw(Bio::Root::Root);
 
 our %ASGTable = ( 'aa'         =>  0,
 		  'resNum'     =>  1,
