@@ -1,4 +1,4 @@
-# $Id: Sbjct.pm,v 1.23 2002/10/22 07:45:23 lapp Exp $
+# $Id: Sbjct.pm,v 1.26 2003/05/17 19:03:58 heikki Exp $
 ###############################################################################
 # Bio::Tools::BPlite::Sbjct
 ###############################################################################
@@ -26,15 +26,16 @@ Bio::Tools::BPlite::Sbjct - A Blast Subject (database search Hit)
 
 =head1 SYNOPSIS
 
- use Bio::Tools::BPlite
- my $report = new Bio::Tools::BPlite(-fh=>\*STDIN);
- while(my $sbjct = $report->nextSbjct) {
-     $sbjct->name;    # access to the hit name
-     "$sbjct";        # overloaded to return name
-     $sbjct->nextHSP; # gets the next HSP from the sbjct
-     while(my $hsp = $sbjct->nextHSP) {
-	 # canonical form is again a while loop
-     }
+  use Bio::Tools::BPlite;
+  my $report = new Bio::Tools::BPlite(-fh=>\*STDIN);
+  while(my $sbjct = $report->nextSbjct) {
+      $sbjct->name;    # access to the hit name
+      "$sbjct";        # overloaded to return name
+      $sbjct->nextHSP; # gets the next HSP from the sbjct
+      while (my $hsp = $sbjct->nextHSP) {
+ 	 # canonical form is again a while loop
+      }
+  }
 
 =head1 DESCRIPTION
 
@@ -211,9 +212,12 @@ sub nextHSP {
   }
   $positive = $match if not defined $positive;
   $gaps = '0' if not defined $gaps;
-  my ($p)        = $scoreline =~ /[Sum ]*P[\(\d+\)]* = (\S+)/;
-  if (not defined $p) {(undef, $p) = $scoreline =~ /Expect(\(\d+\))? =\s+(\S+)/}
-  $self->throw("Unable to parse '$scoreline'") if not defined $score;
+  my ($p)        = ($scoreline =~ /[Sum ]*P[\(\d+\)]* = (\S+)/);
+  unless (defined $p) {(undef, $p) = $scoreline =~ /Expect(\(\d+\))? =\s+(\S+)/}
+  my ($exp) = ($scoreline =~ /Expect(?:\(\d+\))? =\s+([^\s,]+)/);
+  $exp = -1 unless( defined $exp );
+
+  $self->throw("Unable to parse '$scoreline'") unless defined $score;
   
   #######################
   # get alignment lines #
@@ -228,7 +232,7 @@ sub nextHSP {
       elsif ($_ =~ /^\s*Strand/)    {next} # NCBI-BLAST non-data
       elsif ($_ =~ /^\s*Score/)     {$self->_pushback($_); last}
 
-      elsif ($_ =~ /^>|^Parameters|^\s+Database:|^CPU\stime|^\s*Lambda/)   
+      elsif ($_ =~ /^>|^Histogram|^Searching|^Parameters|^\s+Database:|^CPU\stime|^\s*Lambda/)   
       {    
 	  #ps 5/28/01	
 	  # elsif ($_ =~ /^>|^Parameters|^\s+Database:|^CPU\stime/)   {
@@ -265,7 +269,7 @@ sub nextHSP {
   
   for(my $i=0;$i<@hspline;$i+=3) {
     # warn $hspline[$i], $hspline[$i+2];
-    $hspline[$i]   =~ /^Query:\s+(\d+)\s*([\D\S]+)\s+(\d+)/;
+    $hspline[$i]   =~ /^(?:Query|Trans):\s+(\d+)\s*([\D\S]+)\s+(\d+)/;
     $ql = $2; $qb = $1 unless $qb; $qe = $3;
     
     my $offset = index($hspline[$i], $ql);
@@ -299,6 +303,7 @@ sub nextHSP {
        '-gaps'       => $gaps,
        '-hsplength'  => $hsplength,
        '-p'          => $p,
+       '-exp'        => $exp,
        '-queryBegin' => $qb, 
        '-queryEnd'   => $qe, 
        '-sbjctBegin' => $sb,
