@@ -1,4 +1,4 @@
-# $Id: RemoteBlast.pm,v 1.14 2002/12/23 13:01:50 birney Exp $
+# $Id: RemoteBlast.pm,v 1.17 2003/10/15 20:45:06 jason Exp $
 #
 # BioPerl module for Bio::Tools::Run::RemoteBlast
 #
@@ -40,7 +40,7 @@ via HTTP
 
   my $v = 1;
   #$v is just to turn on and off the messages
-  
+
   my $str = Bio::SeqIO->new(-file=>'amino.fa' , '-format' => 'fasta' );
 
   while (my $input = $str->next_seq()){
@@ -132,7 +132,7 @@ methods. Internal methods are usually preceded with a _
 package Bio::Tools::Run::RemoteBlast;
 
 use vars qw($AUTOLOAD @ISA %BLAST_PARAMS $URLBASE %HEADER %RETRIEVALHEADER
-	    $RIDLINE);
+	    $RIDLINE $MODVERSION);
 use strict;
 
 use Bio::Root::Root;
@@ -143,7 +143,11 @@ use Bio::Tools::BPlite;
 use Bio::SearchIO;
 use LWP;
 use HTTP::Request::Common;
+
+@ISA = qw(Bio::Root::Root Bio::Root::IO);
+
 BEGIN {
+    $MODVERSION = $Bio::Root::Version::VERSION;
     $URLBASE = 'http://www.ncbi.nlm.nih.gov/blast/Blast.cgi';
     %HEADER = ('CMD'                          => 'Put',
 	       'PROGRAM'                      => '',
@@ -165,7 +169,7 @@ BEGIN {
 			'FORMAT_TYPE'    => 'Text',
 			);
 
-    $RIDLINE = 'RID\s+=\s+(\d+-\d+-\d+)';
+    $RIDLINE = 'RID\s+=\s+(\S+)';
 
     %BLAST_PARAMS = ( 'prog' => 'blastp',
 		       'data' => 'nr',
@@ -175,8 +179,6 @@ BEGIN {
 
 }
 
-@ISA = qw(Bio::Root::Root Bio::Root::IO);
-
 sub new {
     my ($caller, @args) = @_;
     # chained new
@@ -316,9 +318,12 @@ sub expect {
 =cut
 
 sub ua {
-    my ($self, $value) = @_;
+    my ($self, $value) = @_;    
     if( ! defined $self->{'_ua'} ) {
-	$self->{'_ua'} = new LWP::UserAgent;
+	$self->{'_ua'} = new LWP::UserAgent();
+	my $nm = ref($self);
+	$nm =~ s/::/_/g;
+	$self->{'_ua'}->agent("bioperl-$nm/$MODVERSION");
     }
     return $self->{'_ua'};
 }
