@@ -1,4 +1,4 @@
-# $Id: WrapperBase.pm,v 1.21.4.2 2006/10/17 09:12:57 sendu Exp $
+# $Id: WrapperBase.pm,v 1.12 2003/03/28 22:42:54 jason Exp $
 #
 # BioPerl module for Bio::Tools::Run::WrapperBase
 #
@@ -42,20 +42,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+  bioperl-l@bioperl.org              - General discussion
+  http://bioperl.org/MailList.shtml  - About the mailing lists
 
 =head2 Reporting Bugs
 
-Report bugs to the Bioperl bug tracking system to help us keep track of
-the bugs and their resolution. Bug reports can be submitted via the
-web:
+Report bugs to the Bioperl bug tracking system to help us keep track
+of the bugs and their resolution. Bug reports can be submitted via
+email or the web:
 
-  http://bugzilla.open-bio.org/
+  bioperl-bugs@bioperl.org
+  http://bioperl.org/bioperl-bugs/
 
 =head1 AUTHOR - Jason Stajich
 
-Email jason-at-bioperl.org
+Email jason@bioperl.org
+
+Describe contact details here
+
+=head1 CONTRIBUTORS
+
+Additional contributors names and emails here
 
 =head1 APPENDIX
 
@@ -69,14 +76,15 @@ Internal methods are usually preceded with a _
 
 
 package Bio::Tools::Run::WrapperBase;
+use vars qw(@ISA);
 use strict;
 
 # Object preamble - inherits from Bio::Root::Root
 
-use base qw(Bio::Root::Root);
+use Bio::Root::RootI;
+use Bio::Root::IO;
 
-use File::Spec;
-use File::Path qw(); # don't import anything
+@ISA = qw(Bio::Root::RootI);
 
 =head2 run
 
@@ -115,25 +123,6 @@ sub error_string{
     return $self->{'_error_string'} || '';
 }
 
-=head2 arguments
-
- Title   : arguments
- Usage   : $obj->arguments($newval)
- Function: Commandline parameters 
- Returns : value of arguments
- Args    : newvalue (optional)
-
-
-=cut
-
-sub arguments {
-  my ($self,$value) = @_;
-  if(defined $value) {
-    $self->{'_arguments'} = $value;
-  }
-  return $self->{'_arguments'} || '';
-}
-
 
 =head2 no_param_checks
 
@@ -209,16 +198,15 @@ sub outfile_name{
 =cut
 
 sub tempdir{
-   my ($self) = shift;
-
-   $self->{'_tmpdir'} = shift if @_;
+   my ($self) = @_;
+   
    unless( $self->{'_tmpdir'} ) {
        $self->{'_tmpdir'} = $self->io->tempdir(CLEANUP => ! $self->save_tempfiles );
    }
-   unless( -d $self->{'_tmpdir'} ) {
+   unless( -d $self->{'_tmpdir'} ) { 
        mkdir($self->{'_tmpdir'},0777);
    }
-   return $self->{'_tmpdir'};
+   $self->{'_tmpdir'};
 }
 
 =head2 cleanup
@@ -235,9 +223,9 @@ sub tempdir{
 sub cleanup{
    my ($self) = @_;
    $self->io->_io_cleanup();
-   if( defined $self->{'_tmpdir'} && -d $self->{'_tmpdir'} ) {
-       # $self->io->rmtree($self->{'_tmpdir'});
-       File::Path->rmtree( $self->{'_tmpdir'} );
+   if( defined $self->{'_tmpdir'} &&
+       -d $self->{'_tmpdir'} ) {
+       $self->io->rmtree($self->{'_tmpdir'});
    }
 }
 
@@ -245,8 +233,8 @@ sub cleanup{
 
  Title   : io
  Usage   : $obj->io($newval)
- Function: Gets a Bio::Root::IO object
- Returns : Bio::Root::IO object
+ Function: Gets a L<Bio::Root::IO> object
+ Returns : L<Bio::Root::IO> object
  Args    : none
 
 
@@ -273,7 +261,7 @@ sub io{
 
 sub version{
    my ($self,@args) = @_;
-   return;
+   return undef;
 }
 
 =head2 executable
@@ -327,7 +315,7 @@ sub program_path {
     push @path, $self->program_dir if $self->program_dir;
     push @path, $self->program_name.($^O =~ /mswin/i ?'.exe':'');
 
-    return File::Spec->catfile(@path);
+    return Bio::Root::IO->catfile(@path);
 }
 
 =head2 program_dir
