@@ -1,4 +1,4 @@
-# $Id: TreeEventBuilder.pm,v 1.11 2002/10/22 07:45:25 lapp Exp $
+# $Id: TreeEventBuilder.pm,v 1.18 2003/12/06 18:10:26 jason Exp $
 #
 # BioPerl module for Bio::TreeIO::TreeEventBuilder
 #
@@ -46,9 +46,7 @@ email or the web:
 
 =head1 AUTHOR - Jason Stajich
 
-Email jason@bioperl.org
-
-Describe contact details here
+Email jason-at-bioperl.org
 
 =head1 CONTRIBUTORS
 
@@ -91,10 +89,22 @@ sub new {
   my($class,@args) = @_;
 
   my $self = $class->SUPER::new(@args);
-  my ($treetype, $nodetype) = $self->_rearrange([qw(TREETYPE NODETYPE)], @args);
-  $self->treetype($treetype || 'Bio::Tree::Tree');
-  $self->nodetype($nodetype || 'Bio::Tree::Node');
-  
+  my ($treetype, $nodetype) = $self->_rearrange([qw(TREETYPE 
+						    NODETYPE)], @args);
+  $treetype ||= 'Bio::Tree::Tree';
+  $nodetype ||= 'Bio::Tree::Node';
+
+  eval { 
+      $self->_load_module($treetype);
+      $self->_load_module($nodetype);
+  };
+
+  if( $@ ) {
+      $self->throw("Could not load module $treetype or $nodetype. \n$@\n")
+  }
+  $self->treetype($treetype);
+  $self->nodetype($nodetype);
+  $self->{'_treelevel'} = 0;
   return $self;
 }
 
@@ -171,7 +181,7 @@ sub start_document {
 
 sub end_document {
     my ($self) = @_; 
-    my $root = $self->nodetype->new;
+    my $root = $self->nodetype->new(-verbose => $self->verbose);
     # aggregate the nodes into trees basically ad-hoc.
     while ( @{$self->{'_currentnodes'}} ) {	
 	my ($node) = ( shift @{$self->{'_currentnodes'}});
@@ -184,7 +194,8 @@ sub end_document {
 	    $self->debug("node is ". $node->to_string(). "\n");
 	}
     }
-    my $tree = $self->treetype->new(-root => $root);
+    my $tree = $self->treetype->new(-verbose => $self->verbose,
+				    -root => $root);
     return $tree;       
 }
 
@@ -203,16 +214,15 @@ sub start_element{
    my ($self,$data) =@_;
    $self->{'_lastitem'}->{$data->{'Name'}}++;   
 
-   $self->debug("starting element: $data->{Name}\n");
-
+   $self->debug("starting element: $data->{Name}\n");   
    push @{$self->{'_lastitem'}->{'current'}},$data->{'Name'};
-
+   
    my %data;
    
    if( $data->{'Name'} eq 'node' ) {
        push @{$self->{'_currentitems'}}, \%data; 
    } elsif ( $data->{Name} eq 'tree' ) {
-       $self->{_treelevel}++;
+       $self->{'_treelevel'}++;
    }
 }
 
@@ -230,25 +240,39 @@ sub end_element{
    my ($self,$data) = @_;   
 
    $self->debug("end of element: $data->{Name}\n");
+   # this is the stack where we push/pop items from it
+   my $curcount = scalar @{$self->{'_currentnodes'}};
+   my $level   = $self->{'_treelevel'};
+   my $levelct = $self->{'_nodect'}->[$self->{'_treelevel'}+1] || 0;
 
    if( $data->{'Name'} eq 'node' ) {
        my $tnode;
        my $node = pop @{$self->{'_currentitems'}};	   
 
-       $tnode = $self->nodetype->new( %{$node});
-       unless ( $node->{'-id'} ) { 
-	   for ( splice( @{$self->{_currentnodes}}, -$self->{_nodect}->[$self->{_treelevel} + 1]) ) {
+       $tnode = $self->nodetype->new( -verbose => $self->verbose,
+				      %{$node});       
+       $self->debug( "new node will be ".$tnode->to_string."\n");
+       if ( !$node->{'-leaf'} && $levelct > 0) {
+	   $self->debug(join(',', map { $_->to_string } 
+			     @{$self->{'_currentnodes'}}). "\n");
+	   $self->throw("something wrong with event construction treelevel ".
+			"$level is recorded as having $levelct nodes  ".
+			"but current nodes at this level is $curcount\n")
+	       if( $levelct > $curcount);	
+	   for ( splice( @{$self->{'_currentnodes'}}, - $levelct)) {
 	       $self->debug("adding desc: " . $_->to_string . "\n");
 	       $tnode->add_Descendent($_);
 	   }
-	   $self->{_nodect}->[$self->{_treelevel}+1] = 0;
+	   $self->{'_nodect'}->[$self->{'_treelevel'}+1] = 0;
        }
        push @{$self->{'_currentnodes'}}, $tnode;
-       $self->{_nodect}->[$self->{_treelevel}]++;
-       $self->debug ("added node: nodes in stack is ". scalar @{$self->{'_currentnodes'}} . ", treelevel: $self->{_treelevel}, nodect: $self->{_nodect}->[$self->{_treelevel}]\n");
+       $self->{'_nodect'}->[$self->{'_treelevel'}]++;
+       
+       $self->debug ("added node: nodes in stack is $curcount, treelevel: $level, nodect: $levelct\n");
+       
    } elsif(  $data->{'Name'} eq 'tree' ) { 
-       $self->debug("end of tree: nodes in stack is ". scalar @{$self->{'_currentnodes'}}. "\n");
-       $self->{_treelevel}--;
+       $self->debug("end of tree: nodes in stack is $curcount\n");
+       $self->{'_treelevel'}--;
    }
 
    $self->{'_lastitem'}->{ $data->{'Name'} }--; 
@@ -323,6 +347,8 @@ sub characters{
        } elsif ( $self->in_element('tag_value') ) {
 	   $hash->{'-nhx'}->{$hash->{'-NHXtagname'}} = $ch;
 	   delete $hash->{'-NHXtagname'};
+       } elsif( $self->in_element('leaf') ) {
+	   $hash->{'-leaf'} = $ch;
        }
        push @{$self->{'_currentitems'}}, $hash;
    }
