@@ -1,4 +1,4 @@
-# $Id: newick.pm,v 1.13 2002/10/22 07:45:25 lapp Exp $
+# $Id: newick.pm,v 1.24 2003/12/16 16:53:43 jason Exp $
 #
 # BioPerl module for Bio::TreeIO::newick
 #
@@ -13,13 +13,14 @@
 =head1 NAME
 
 Bio::TreeIO::newick - TreeIO implementation for parsing 
-    Newick/New Hampshire/PHYLIP format.
+  Newick/New Hampshire/PHYLIP format.
 
 =head1 SYNOPSIS
 
   # do not use this module directly
   use Bio::TreeIO;
-  my $treeio = new Bio::TreeIO(-format => 'newick', -file => 'tree.dnd');
+  my $treeio = new Bio::TreeIO(-format => 'newick', 
+                               -file => 't/data/LOAD_Ccd1.dnd');
   my $tree = $treeio->next_tree;
 
 =head1 DESCRIPTION
@@ -95,7 +96,11 @@ sub next_tree{
    my ($self) = @_;
    local $/ = ";\n";
    return unless $_ = $self->_readline;
-   s/\s+//g;
+   s/[\r\n]//gs;
+   my $despace = sub {my $dirty = shift; $dirty =~ s/\s+//gs; return $dirty};
+   my $dequote = sub {my $dirty = shift; $dirty =~ s/^"?\s*(.+?)\s*"?$/$1/; return $dirty};
+   s/([^"]*)(".+?")([^"]*)/$despace->($1) . $dequote->($2) . $despace->($3)/egsx;
+
    $self->debug("entry is $_\n");
 #   my $empty = chr(20);
  
@@ -108,65 +113,87 @@ sub next_tree{
 
    my $chars = '';
    $self->_eventHandler->start_document;
-   my $lastevent = '';
+   my ($prev_event,$lastevent,$id) = ('','','');
    foreach my $ch ( split(//,$_) ) {
        if( $ch eq ';' ) { 	   
 	   return $self->_eventHandler->end_document;
        } elsif( $ch eq '(' ) {
 	   $chars = '';
 	   $self->_eventHandler->start_element( {'Name' => 'tree'} );
-	   $lastevent = $ch;
        } elsif($ch eq ')' ) {
-	   if( $chars ) {
+	   if( length($chars) ) {
 	       if( $lastevent eq ':' ) {
 		   $self->_eventHandler->start_element( { 'Name' => 'branch_length'});
 		   $self->_eventHandler->characters($chars);
 		   $self->_eventHandler->end_element( {'Name' => 'branch_length'});
+		   $lastevent = $prev_event;
 	       } else { 
 		   $self->debug("id with no branchlength is $chars\n");
 		   $self->_eventHandler->start_element( { 'Name' => 'node' } );
 		   $self->_eventHandler->start_element( { 'Name' => 'id' } );
 		   $self->_eventHandler->characters($chars);
 		   $self->_eventHandler->end_element( { 'Name' => 'id' } );
+		   $id = $chars;
 	       }
-	       
+	       my $leafstatus = 0;
+	       if( $lastevent ne ')' ) {
+		   $leafstatus = 1;
+	       }
+
+	       $self->_eventHandler->start_element({'Name' => 'leaf'});
+	       $self->_eventHandler->characters($leafstatus);
+	       $self->_eventHandler->end_element({'Name' => 'leaf'});
+	       $id = '';
 	   } else {
-	       $self->_eventHandler->start_element( {'Name' => 'node'} )
+	       $self->_eventHandler->start_element( {'Name' => 'node'} );
 	   }
+
 	   $self->_eventHandler->end_element( {'Name' => 'node'} );
 	   $self->_eventHandler->end_element( {'Name' => 'tree'} );
 	   $chars = '';
-	   $lastevent = $ch;
        } elsif ( $ch eq ',' ) {
-	   if( $chars ) {
+	   if( length($chars) ) {
 	       if( $lastevent eq ':' ) {
 		   $self->_eventHandler->start_element( { 'Name' => 'branch_length'});
 		   $self->_eventHandler->characters($chars);
 		   $self->_eventHandler->end_element( {'Name' => 'branch_length'});
+		   $lastevent = $prev_event;
+		   $chars = '';		   
 	       } else { 
 		   $self->debug("id with no branchlength is $chars\n");
 		   $self->_eventHandler->start_element( { 'Name' => 'node' } );
 		   $self->_eventHandler->start_element( { 'Name' => 'id' } );
 		   $self->_eventHandler->characters($chars);
 		   $self->_eventHandler->end_element( { 'Name' => 'id' } );
-	       }   
+		   $id = $chars;
+	       }
 	   } else {
 	       $self->_eventHandler->start_element( { 'Name' => 'node' } );
 	   }
+	   my $leafstatus = 0;
+	   if( $lastevent ne ')' ) {
+	       $leafstatus = 1;
+	   }
+	   $self->_eventHandler->start_element({'Name' => 'leaf'});
+	   $self->_eventHandler->characters($leafstatus);
+	   $self->_eventHandler->end_element({'Name' => 'leaf'});
 	   $self->_eventHandler->end_element( {'Name' => 'node'} );
 	   $chars = '';
-	   $lastevent = $ch;
+	   $id    = '';
        } elsif( $ch eq ':' ) {
 	   $self->debug("id with a branchlength coming is $chars\n");
 	   $self->_eventHandler->start_element( { 'Name' => 'node' } );
 	   $self->_eventHandler->start_element( { 'Name' => 'id' } );	   
 	   $self->_eventHandler->characters($chars);
 	   $self->_eventHandler->end_element( { 'Name' => 'id' } );	   
+	   $id = $chars;
 	   $chars = '';
-	   $lastevent = $ch;
        } else { 	   
 	   $chars .= $ch;
+	   next;
        }
+       $prev_event = $lastevent;
+       $lastevent = $ch;
    }
    return undef;
 }
@@ -208,7 +235,15 @@ sub _write_tree_Helper {
     if( @data > 1 ) {
 	$data[0] = "(" . $data[0];
 	$data[-1] .= ")";
-	$data[-1] .= ":". $node->branch_length if( $node->branch_length);
+	# let's explicitly write out the bootstrap if we've got it
+	my $b;
+	if( defined ($b = $node->bootstrap) ) {
+	    $data[-1] .= $b;
+	} elsif( defined ($b = $node->id) ) {
+	    $data[-1] .= $b;
+	}
+	$data[-1] .= ":". $node->branch_length if( defined $node->branch_length);
+	
     } else {
 	if( defined $node->id || defined $node->branch_length ) { 
 	    push @data, sprintf("%s%s",
