@@ -4,7 +4,7 @@
 # PURPOSE  : Demonstrate possible uses of Bio::Tools::StandAloneBlast.pm
 # AUTHOR   : Peter Schattner schattner@alum.mit.edu
 # CREATED  : Nov 01 2000
-# REVISION : $Id: standaloneblast.pl,v 1.5 2006/01/16 16:50:55 bosborne Exp $
+# REVISION : $Id: standaloneblast.pl,v 1.1 2003/07/15 13:45:27 bosborne Exp $
 #
 # INSTALLATION
 #
@@ -29,95 +29,86 @@ my $amino_database = 'swissprot';
 #	to identify an alignment to use for blasting or whether an external alignment is given to 
 #	psiblast
 #
-use strict;
+
 use Getopt::Long;
 use Bio::SimpleAlign;
 use Bio::Tools::Run::StandAloneBlast;
-use Bio::SearchIO;
+use Bio::Tools::BPlite::Sbjct;
 use Bio::AlignIO;
 use Bio::SeqIO;
-use Bio::Root::IO;
+use strict;
 
 # set some default values
-my $queryseq = Bio::Root::IO->catfile(qw(t data cysprot1.fa) );
+my $queryseq = 't/cysprot1.fa';
 my $executable = 'blastpgp';
-my $queryaln = Bio::Root::IO->catfile(qw(t data cysprot.msf) );
+my $queryaln = 't/cysprot.msf';
 my @params = ('database' => $amino_database);
-# string listing examples to be executed. Default is to execute
-# all tests (ie 1,2 and 3)
-my $do_only = ''; 	
+my $do_only = ''; 	# string listing examples to be executed. Default is to execute
+			# all tests (ie 1,2 and 3)
 my $example1param = 'MATRIX';  # parameter to be varied in example 1
 my $example2param = 'GAP';  # parameter to be varied in example 1
-my $example1values = [ 'BLOSUM62', 'BLOSUM80', 'PAM70']; # MATRIX values to try
+my $example1values = [ 'BLOSUM62', 'BLOSUM80', 'PAM70']; # MATRIX values to be tried
 my $example2values = [ 7, 9, 25]; # GAP values to be tried
 my $queryalnformat = 'msf';
-my $jiter = 2;
-# only use pos. specific scoring matrix if > 50% of residues have
-# consensus letter (and compare with 25% or 75% cut off)
-my  $maskvalues = [50, 25, 75] ; my $helpflag = 0;   # Flag to show usage info.
+my  $jiter = 2;
+my  $maskvalues = [50, 25, 75] ; # only use pos. specific scoring matrix if > 50% of residues have
+		     # consensus letter (and compare with 25% or 75% cut off)
+my $helpflag = 0;   # Flag to show usage info.
 
 # get user options
 my @argv       = @ARGV;  # copy ARGV before GetOptions() massacres it.
 my $paramvalstring;
 my $maskvalstring;
 
-&GetOptions("h!"          => \$helpflag, 
-				"help!"       => \$helpflag,
-				"in=s"        => \$queryseq,
-				"inaln=s"     => \$queryaln,
-				"alnfmt=s"    => \$queryalnformat,
-				"param=s"     => \$example1param,
-				"exec=s"      => \$executable,
-				"paramvals=s" => \$paramvalstring,
-				"do=i"        =>  \$do_only,
-				"maskvals=s"  => \$maskvalstring,
-				"iter=i"      =>  \$jiter,
+&GetOptions("h!" => \$helpflag, "help!" => \$helpflag,
+	"in=s" => \$queryseq,
+	"inaln=s" => \$queryaln,
+	"alnfmt=s" => \$queryalnformat,
+	"param=s" => \$example1param,
+	"exec=s" => \$executable,
+	"paramvals=s" => \$paramvalstring,
+	"do=i" =>  \$do_only,
+	"maskvals=s" => \$maskvalstring,
+	"iter=i" =>  \$jiter,
 	) ;
 
 if ($paramvalstring) { @$example1values = split (":", $paramvalstring); }
-if ($maskvalstring)  { @$maskvalues     = split (":", $maskvalstring);  }
+if ($maskvalstring) { @$maskvalues = split (":", $maskvalstring); }
 
-if ($helpflag) { &example_usage(); exit 0;}
+ if ($helpflag) { &example_usage(); exit 0;}
 
 # create factory & set user-specified global blast parameters
 foreach my $argv (@argv) {
-	next unless ($argv =~ /^(.*)=>(.*)$/);
+	unless ($argv =~ /^(.*)=>(.*)$/) { next;}
 	push (@params, $1 => $2);
 }
 my  $factory = Bio::Tools::Run::StandAloneBlast->new(@params);
 	
 # If "do" variable not set, do all four examples
 if ( ! $do_only)  {
-    &vary_params($queryseq, $example1param, $example1values); # ex. 1
+	&vary_params($queryseq, $example1param, $example1values);   # ex. 1
+# To compare gap penalties of 7, 9 and 25 we need to set the scoring matrix to BLOSUM62
+#  and extension penalty to 2 (these are limitations of BLAST)
 
-    # To compare gap penalties of 7, 9 and 25 we need to set the
-    # scoring matrix to BLOSUM62 and extension penalty to 2 (these are
-    # limitations of BLAST)
+	$factory->MATRIX('BLOSUM62');  
 
-    $factory->MATRIX('BLOSUM62');  
-
-    $factory->EXTENSION(2);  
-    &vary_params($queryseq, $example2param, $example2values); # ex. 2
- 
-    # For the psiblast tests we want to restore gap opening and
-    # extension values to their defaults
-
-    $factory->GAP(11);
-    $factory->EXTENSION(1);
-    # now do the mask comparison example and ..
-    &vary_masks($queryseq, $maskvalues); # ex. 3
-    # do the jumpstart-align vs multiple iteration examples with the 
-    # mask value set to 50%
-    &aligned_blast($queryseq, $queryaln, $queryalnformat, 
-		   $jiter, $maskvalues->[0]); # ex. 4
+	$factory->EXTENSION(2);  
+	&vary_params($queryseq, $example2param, $example2values);   # ex. 2
+# For the psiblast tests we want to restore gap opening and extension values to their defaults
+	$factory->GAP(11);
+	$factory->EXTENSION(1);
+# now do the mask comparison example and ..
+	&vary_masks($queryseq, $maskvalues);       # ex. 3
+# do the jumpstart-align vs multiple iteration examples with the mask value set to 50%
+	&aligned_blast($queryseq, $queryaln, $queryalnformat, $jiter, $maskvalues->[0]);   # ex. 4
 } elsif ($do_only  == 1) {
-    &vary_params($queryseq,$example1param, $example1values);
+	&vary_params($queryseq,$example1param, $example1values);
 } elsif ($do_only  == 3) {
-    &vary_masks($queryseq, $maskvalues);
+	&vary_masks($queryseq, $maskvalues);
 } elsif ($do_only  == 4 ) {
-    &aligned_blast($queryseq, $queryaln, $queryalnformat, $jiter, $maskvalues->[0]);
+	&aligned_blast($queryseq, $queryaln, $queryalnformat, $jiter, $maskvalues->[0]);
 }  else {
-    &example_usage();
+	&example_usage();
 }
 
 exit 0;
@@ -126,6 +117,8 @@ exit 0;
 ## End of "main"
 
 
+
+
 #################################################
 #   compare_runs(): Prints out display of which hits were found by different methods
 #	Various methods are labeled by "tags" found in array @runtags
@@ -139,24 +132,23 @@ exit 0;
 #  returns: nothing  
 
 sub compare_runs {
-    my $typetag = shift;
-    my $runtags = shift;
+my $typetag = shift;
+my $runtags = shift;
 
-    my $hashhits = shift;   
+my $hashhits = shift;   
 
-    my ($tag, @taghits);
+my ($tag, @taghits);
 
-    print "Comparing BLAST results... \n";
+print "Comparing BLAST results... \n";
 
 # Get total number of hits found by any method
-    my $numhits = keys %$hashhits ; # scalar context to get total number of hits by all methods
-    print  "Total number of hits found: $numhits \n";
+my $numhits = keys %$hashhits ;   # scalar context to get total number of hits by all methods
+print  "Total number of hits found: $numhits \n";
 
 # Get total number of hits found by every method
-    my $alltags =  join ( "" ,  @$runtags );
-    my  @alltaghits = grep  $$hashhits{$_} =~ /$alltags/  ,  keys %$hashhits;
-    print  " Number of hits found by every method / parameter-value: " ,   
-    scalar(@alltaghits), "\n";
+my $alltags =  join ( "" ,  @$runtags );
+my  @alltaghits = grep  $$hashhits{$_} =~ /$alltags/  ,  keys %$hashhits;
+print  " Number of hits found by every method / parameter-value: " ,   scalar(@alltaghits), "\n";
 
 # If one desires to see the hits found by all methods, uncomment next 2 lines
 #print  " Hits were found all methods / parameters: \n";
@@ -164,12 +156,12 @@ sub compare_runs {
 
 # For each method/parameter-value (labeled by type) display  hits found 
 # exclusively by that method
-    foreach $tag (@$runtags)  {
-		 @taghits = grep  $$hashhits{$_} =~ /^$tag$/  ,  keys %$hashhits;
-		 print  " Hits found only when $typetag was $tag: \n";
-		 print   join ( "\n", @taghits ) ,  "\n";
-    }
-    return 1;
+  foreach $tag (@$runtags)  {
+	@taghits = grep  $$hashhits{$_} =~ /^$tag$/  ,  keys %$hashhits;
+	print  " Hits found only when $typetag was $tag: \n";
+	print   join ( "\n", @taghits ) ,  "\n";
+  }
+return 1;
 }
 
 
@@ -186,51 +178,44 @@ sub compare_runs {
 
 sub vary_params {
 
-    my $queryseq = shift;   
-    my $param = shift;   
-    my $values = shift;  
+my $queryseq = shift;   
+my $param = shift;   
+my $values = shift;  
 
 
-    print "Beginning $param parameter-varying example... \n";
+print "Beginning $param parameter-varying example... \n";
 
-    # Now we'll perform several blasts, 1 for each value of the
-    # selected parameter.  In the first default case, we vary the
-    # MATRIX substitution parameter, creating 3 BLAST reports, using
-    # MATRIX values of BLOSUM62, BLOSUM80 or PAM70.
+# Now we'll perform several blasts, 1 for each value of the selected parameter.
+# In the first default case, we vary the MATRIX substitution parameter,
+# creating 3 BLAST reports, using MATRIX values of  BLOSUM62, BLOSUM80 or PAM70.
+# In the second default case, we vary the GAP penalty parameter,  
+# creating 3 BLAST reports, using GAP penalties of 7, 9 and 25.
+# In either case we then automatically parse the resulting report to identify which hits
+# are found with any of the parameter values and which with only one of them.
+#
+# To test the BLAST results to some other parameter it is only necessary to change the 
+# parameters passed to the script on the commandline.  The only tricky part is that the BLAST
+# program itself only supports a limited range of parameters.  See the BLAST documentation.
 
-    # In the second default case, we vary the GAP penalty parameter,
-    # creating 3 BLAST reports, using GAP penalties of 7, 9 and 25. In
-    # either case we then automatically parse the resulting report to
-    # identify which hits are found with any of the parameter values
-    # and which with only one of them.
+my ($report, $sbjct, $paramvalue);
 
- 
-    # To test the BLAST results to some other parameter it is only
-    # necessary to change the parameters passed to the script on the
-    # commandline.  The only tricky part is that the BLAST program
-    # itself only supports a limited range of parameters.  See the
-    # BLAST documentation.
+my $hashhits = { };     # key is hit id, value is string of param values for which hit was found
 
-    my ($report, $sbjct, $paramvalue);
+foreach $paramvalue (@$values)  {
 
-    my $hashhits = { };	# key is hit id, value is string of param values for which hit was found
-    
-    foreach $paramvalue (@$values)  {
-	
-	$factory->$param($paramvalue); # set parameter value
+	$factory->$param($paramvalue);  # set parameter value
 
-	print "Performing BLAST with $param = $paramvalue \n";
+            print "Performing BLAST with $param = $paramvalue \n";
 
 	$report = $factory->$executable($queryseq);
-	my $r = $report->next_result;
-	while( my $hit = $r->next_hit ) {
-	    $hashhits->{$hit->name} .= "$paramvalue";
+	while ($sbjct = $report->nextSbjct) {
+		$hashhits->{$sbjct->name} .= "$paramvalue";			
 	}
-    }
+}
 
-    &compare_runs( $param , $values , $hashhits);  
-
-    return 1;
+&compare_runs( $param , $values , $hashhits);  
+ 
+return 1;
 
 }
 
@@ -269,15 +254,14 @@ my $aln = $str->next_aln();
 
 foreach $maskvalue (@$values)  {
 
-    print "Performing BLAST with mask threshold = $maskvalue % \n";
+        print "Performing BLAST with mask threshold = $maskvalue % \n";
 
-    # Create the proper mask for 'jumpstarting'
-    my $mask = &create_mask($aln, $maskvalue);
-    my $report2 = $factory->blastpgp($queryseq, $aln, $mask);
-    my $r = $report2->next_result;
-    while($sbjct = $r->next_hit) {
-	$hashhits->{$sbjct->name} .= "$maskvalue";			
-    }
+	# Create the proper mask for 'jumpstarting'
+	my $mask = &create_mask($aln, $maskvalue);
+	my $report2 = $factory->blastpgp($queryseq, $aln, $mask);
+	while($sbjct = $report2->nextSbjct) {
+		$hashhits->{$sbjct->name} .= "$maskvalue";			
+	}
 }
 
 &compare_runs( 'mask threshold' , $values , $hashhits);
@@ -345,7 +329,7 @@ my $mask = &create_mask($aln, $maskvalue);
 
 
 my $report2 = $factory->blastpgp($queryseq, $aln, $mask);
-while($sbjct = $report2->next_result) {
+while($sbjct = $report2->nextSbjct) {
 		$hashhits->{$sbjct->name} .= "$tag1";			
 }
 
@@ -363,7 +347,7 @@ my $total_iterations = $report1->number_of_iterations;
 my $last_iteration = $report1->round($total_iterations);
 
 
- while($sbjct = $last_iteration->next_result) {
+ while($sbjct = $last_iteration->nextSbjct) {
 		$hashhits->{$sbjct->name} .= "$tag2";			
 	}
 
@@ -380,40 +364,36 @@ return 1;
 
 
 #################################################
-
-
 # create_mask(): creates a mask for the psiblast jumpstart alignment
-#                that determines at what residues position-specific
-#                scoring matrices (PSSMs) are used and at what
-#                residues default scoring matrices (eg BLOSUM) are
-#                used. See psiblast documentation for more details,
-
+#	that determines at what residues position-specific scoring matrices (PSSMs)
+#	are used and at what residues default scoring matrices (eg BLOSUM)
+#	are used. See psiblast documentation for more details,
+#
 #  args: 
 #	$aln  -  SimpleAlign object with alignment
 #	$maskvalue  -  label describing type of "tags"
-#  returns: actual mask, ie a string of 0's and 1's which is the 
-#           same length as each sequence in the alignment and has 
-#           a "1" at locations where (PSSMs) are to be used
-#           and a "0" at all other locations.
-
+#  returns: actual mask, ie a string of 0's and 1's which is the same length as each
+#		sequence in the alignment and has a "1" at locations where (PSSMs) are to be used
+#		and a "0" at all other locations.
 
 sub create_mask {
-	my $aln = shift;
-	my $maskvalue = shift;
-	my $mask = "";
+my $aln = shift;
+my $maskvalue = shift;
+my $mask = "";
 
-	die "psiblast jumpstart requires all sequences to be same length \n"
-	  unless $aln->is_flush();
-	my $len = $aln->length();
+unless ( $aln->is_flush() )  { die "psiblast jumpstart requires all sequences to be same length \n"; }
+my $len = $aln->length_aln();
 
-	if ($maskvalue =~ /^(\d){1,3}$/  ) {
-		$mask = $aln->consensus_string($maskvalue) ;
-		$mask =~ s/[^\?]/1/g ;
-		$mask =~ s/\?/0/g ;
-	}
-	else { die "maskvalue must be an integer between 0 and 100 \n"; }
-	return $mask ;
+if ($maskvalue =~ /^(\d){1,3}$/  ) {
+   $mask = $aln->consensus_string($maskvalue) ;
+   $mask =~ s/[^\?]/1/g ;
+   $mask =~ s/\?/0/g ;
 }
+else { die "maskvalue must be an integer between 0 and 100 \n"; }
+return $mask ;
+}
+
+
 
 #----------------
 sub example_usage {
@@ -453,7 +433,7 @@ or
 or
  >standaloneblast.pl -do 4 -maskvals 0 -iter 3
 or
- >standaloneblast.pl -do 3 -maskvals '10:50:90'  -in 't/data/cysprot1.fa' -alnfmt msf -inaln 't/cysprot.msf'
+ >standaloneblast.pl -do 3 -maskvals '10:50:90'  -in 't/cysprot1.fa' -alnfmt msf -inaln 't/cysprot.msf'
 
 
 
